@@ -220,7 +220,7 @@ load_codepages_list (void)
         fprintf (stderr, "%s\n", _("Warning: cannot load codepages list"));
 
         codepages = g_ptr_array_new ();
-        g_ptr_array_add (codepages, new_codepage_desc ("ASCII", _("7-bit ASCII")));
+        g_ptr_array_add (codepages, new_codepage_desc (DEFAULT_CHARSET, _("7-bit ASCII")));
     }
 }
 
@@ -432,7 +432,7 @@ str_nconvert_to_input (char *str, int len)
 unsigned char
 convert_from_utf_to_current (const char *str)
 {
-    unsigned char buf_ch[6 + 1];
+    unsigned char buf_ch[UTF8_CHAR_LEN + 1];
     unsigned char ch = '.';
     GIConv conv;
     const char *cp_to;
@@ -467,8 +467,8 @@ convert_from_utf_to_current (const char *str)
 unsigned char
 convert_from_utf_to_current_c (const int input_char, GIConv conv)
 {
-    unsigned char str[6 + 1];
-    unsigned char buf_ch[6 + 1];
+    unsigned char str[UTF8_CHAR_LEN + 1];
+    unsigned char buf_ch[UTF8_CHAR_LEN + 1];
     unsigned char ch = '.';
 
     int res = 0;
@@ -499,7 +499,7 @@ int
 convert_from_8bit_to_utf_c (const char input_char, GIConv conv)
 {
     unsigned char str[2];
-    unsigned char buf_ch[6 + 1];
+    unsigned char buf_ch[UTF8_CHAR_LEN + 1];
     int ch = '.';
     int res = 0;
 
@@ -533,9 +533,8 @@ int
 convert_from_8bit_to_utf_c2 (const char input_char)
 {
     unsigned char str[2];
-    unsigned char buf_ch[6 + 1];
+    unsigned char buf_ch[UTF8_CHAR_LEN + 1];
     int ch = '.';
-    int res = 0;
     GIConv conv;
     const char *cp_from;
 
@@ -547,6 +546,8 @@ convert_from_8bit_to_utf_c2 (const char input_char)
 
     if (conv != INVALID_CONV)
     {
+        int res = 0;
+
         switch (str_translate_char (conv, (char *) str, -1, (char *) buf_ch, sizeof (buf_ch)))
         {
         case ESTR_SUCCESS:
