@@ -2,27 +2,26 @@
    File highlight plugin.
    Interface functions. get color pair index for highlighted file.
 
-   Copyright (C) 2009 The Free Software Foundation, Inc.
+   Copyright (C) 2009, 2011
+   The Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2009.
 
    This file is part of the Midnight Commander.
 
-   The Midnight Commander is free software; you can redistribute it
+   The Midnight Commander is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
 
-   The Midnight Commander is distributed in the hope that it will be
-   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   The Midnight Commander is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -48,7 +47,7 @@
 
 /*inline functions */
 inline static gboolean
-mc_fhl_is_file (file_entry * fe)
+mc_fhl_is_file (file_entry_t * fe)
 {
 #if S_ISREG == 0
     (void) fe;
@@ -57,13 +56,13 @@ mc_fhl_is_file (file_entry * fe)
 }
 
 inline static gboolean
-mc_fhl_is_file_exec (file_entry * fe)
+mc_fhl_is_file_exec (file_entry_t * fe)
 {
     return is_exe (fe->st.st_mode);
 }
 
 inline static gboolean
-mc_fhl_is_dir (file_entry * fe)
+mc_fhl_is_dir (file_entry_t * fe)
 {
 #if S_ISDIR == 0
     (void) fe;
@@ -72,7 +71,7 @@ mc_fhl_is_dir (file_entry * fe)
 }
 
 inline static gboolean
-mc_fhl_is_link (file_entry * fe)
+mc_fhl_is_link (file_entry_t * fe)
 {
 #if S_ISLNK == 0
     (void) fe;
@@ -81,19 +80,25 @@ mc_fhl_is_link (file_entry * fe)
 }
 
 inline static gboolean
-mc_fhl_is_link_to_dir (file_entry * fe)
+mc_fhl_is_hlink (file_entry_t * fe)
+{
+    return (fe->st.st_nlink > 1);
+}
+
+inline static gboolean
+mc_fhl_is_link_to_dir (file_entry_t * fe)
 {
     return mc_fhl_is_link (fe) && (fe->f.link_to_dir);
 }
 
 inline static gboolean
-mc_fhl_is_stale_link (file_entry * fe)
+mc_fhl_is_stale_link (file_entry_t * fe)
 {
     return mc_fhl_is_link (fe) ? fe->f.stale_link : !mc_fhl_is_file (fe);
 }
 
 inline static gboolean
-mc_fhl_is_device_char (file_entry * fe)
+mc_fhl_is_device_char (file_entry_t * fe)
 {
 #if S_ISCHR == 0
     (void) fe;
@@ -102,7 +107,7 @@ mc_fhl_is_device_char (file_entry * fe)
 }
 
 inline static gboolean
-mc_fhl_is_device_block (file_entry * fe)
+mc_fhl_is_device_block (file_entry_t * fe)
 {
 #if S_ISBLK == 0
     (void) fe;
@@ -111,7 +116,7 @@ mc_fhl_is_device_block (file_entry * fe)
 }
 
 inline static gboolean
-mc_fhl_is_special_socket (file_entry * fe)
+mc_fhl_is_special_socket (file_entry_t * fe)
 {
 #if S_ISSOCK == 0
     (void) fe;
@@ -120,7 +125,7 @@ mc_fhl_is_special_socket (file_entry * fe)
 }
 
 inline static gboolean
-mc_fhl_is_special_fifo (file_entry * fe)
+mc_fhl_is_special_fifo (file_entry_t * fe)
 {
 #if S_ISFIFO == 0
     (void) fe;
@@ -129,7 +134,7 @@ mc_fhl_is_special_fifo (file_entry * fe)
 }
 
 inline static gboolean
-mc_fhl_is_special_door (file_entry * fe)
+mc_fhl_is_special_door (file_entry_t * fe)
 {
 #if S_ISDOOR == 0
     (void) fe;
@@ -140,7 +145,7 @@ mc_fhl_is_special_door (file_entry * fe)
 
 
 inline static gboolean
-mc_fhl_is_special (file_entry * fe)
+mc_fhl_is_special (file_entry_t * fe)
 {
     return
         (mc_fhl_is_special_socket (fe) || mc_fhl_is_special_fifo (fe)
@@ -151,7 +156,7 @@ mc_fhl_is_special (file_entry * fe)
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-mc_fhl_get_color_filetype (mc_fhl_filter_t * mc_filter, mc_fhl_t * fhl, file_entry * fe)
+mc_fhl_get_color_filetype (mc_fhl_filter_t * mc_filter, mc_fhl_t * fhl, file_entry_t * fe)
 {
     gboolean my_color = FALSE;
     (void) fhl;
@@ -175,16 +180,14 @@ mc_fhl_get_color_filetype (mc_fhl_filter_t * mc_filter, mc_fhl_t * fhl, file_ent
             my_color = TRUE;
         break;
     case MC_FLHGH_FTYPE_T_LINK:
-        if (mc_fhl_is_link (fe))
+        if ((mc_fhl_is_link (fe)) || (mc_fhl_is_hlink (fe)))
             my_color = TRUE;
         break;
     case MC_FLHGH_FTYPE_T_HARDLINK:
-        /*TODO: hanlde it */
-        if (mc_fhl_is_link (fe))
+        if (mc_fhl_is_hlink (fe))
             my_color = TRUE;
         break;
     case MC_FLHGH_FTYPE_T_SYMLINK:
-        /*TODO: hanlde it */
         if (mc_fhl_is_link (fe))
             my_color = TRUE;
         break;
@@ -228,7 +231,7 @@ mc_fhl_get_color_filetype (mc_fhl_filter_t * mc_filter, mc_fhl_t * fhl, file_ent
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-mc_fhl_get_color_regexp (mc_fhl_filter_t * mc_filter, mc_fhl_t * fhl, file_entry * fe)
+mc_fhl_get_color_regexp (mc_fhl_filter_t * mc_filter, mc_fhl_t * fhl, file_entry_t * fe)
 {
     (void) fhl;
     if (mc_filter->search_condition == NULL)
@@ -247,10 +250,9 @@ mc_fhl_get_color_regexp (mc_fhl_filter_t * mc_filter, mc_fhl_t * fhl, file_entry
 /* --------------------------------------------------------------------------------------------- */
 
 int
-mc_fhl_get_color (mc_fhl_t * fhl, file_entry * fe)
+mc_fhl_get_color (mc_fhl_t * fhl, file_entry_t * fe)
 {
     guint i;
-    mc_fhl_filter_t *mc_filter;
     int ret;
 
     if (fhl == NULL)
@@ -258,6 +260,8 @@ mc_fhl_get_color (mc_fhl_t * fhl, file_entry * fe)
 
     for (i = 0; i < fhl->filters->len; i++)
     {
+        mc_fhl_filter_t *mc_filter;
+
         mc_filter = (mc_fhl_filter_t *) g_ptr_array_index (fhl->filters, i);
         switch (mc_filter->type)
         {
