@@ -115,7 +115,7 @@ mc_fhl_parse_get_regexp (mc_fhl_t * fhl, const gchar * group_name)
 
     mc_filter = g_new0 (mc_fhl_filter_t, 1);
     mc_filter->type = MC_FLHGH_T_FREGEXP;
-    mc_filter->search_condition = mc_search_new (regexp, -1);
+    mc_filter->search_condition = mc_search_new (regexp, -1, DEFAULT_CHARSET);
     mc_filter->search_condition->is_case_sensitive = TRUE;
     mc_filter->search_condition->search_type = MC_SEARCH_T_REGEX;
 
@@ -162,7 +162,7 @@ mc_fhl_parse_get_extensions (mc_fhl_t * fhl, const gchar * group_name)
 
     mc_filter = g_new0 (mc_fhl_filter_t, 1);
     mc_filter->type = MC_FLHGH_T_FREGEXP;
-    mc_filter->search_condition = mc_search_new (buf->str, buf->len);
+    mc_filter->search_condition = mc_search_new (buf->str, buf->len, DEFAULT_CHARSET);
     mc_filter->search_condition->is_case_sensitive =
         mc_config_get_bool (fhl->config, group_name, "extensions_case", TRUE);
     mc_filter->search_condition->search_type = MC_SEARCH_T_REGEX;
@@ -184,9 +184,9 @@ mc_fhl_read_ini_file (mc_fhl_t * fhl, const gchar * filename)
         return FALSE;
 
     if (fhl->config != NULL)
-        return mc_config_read_file (fhl->config, filename, FALSE);
+        return mc_config_read_file (fhl->config, filename, TRUE, FALSE);
 
-    fhl->config = mc_config_init (filename);
+    fhl->config = mc_config_init (filename, TRUE);
     return (fhl->config != NULL);
 }
 
