@@ -15,9 +15,14 @@
 gboolean g_unichar_iszerowidth (gunichar);
 #endif /* ! GLIB_CHECK_VERSION (2, 13, 0) */
 
-#if ! GLIB_CHECK_VERSION (2, 7, 0)
-gboolean g_file_set_contents (const gchar *, const gchar *, gssize, GError **);
-#endif /* ! GLIB_CHECK_VERSION (2, 7, 0) */
+#if ! GLIB_CHECK_VERSION (2, 16, 0)
+int g_strcmp0 (const char *str1, const char *str2);
+#endif /* ! GLIB_CHECK_VERSION (2, 16, 0) */
+
+#if ! GLIB_CHECK_VERSION (2, 28, 0)
+void g_slist_free_full (GSList * list, GDestroyNotify free_func);
+void g_list_free_full (GList * list, GDestroyNotify free_func);
+#endif /* ! GLIB_CHECK_VERSION (2, 28, 0) */
 
 /*** inline functions ****************************************************************************/
 
