@@ -9,6 +9,7 @@
 #define CONFIG_PANELS_SECTION "Panels"
 #define CONFIG_LAYOUT_SECTION "Layout"
 #define CONFIG_MISC_SECTION "Misc"
+#define CONFIG_EXT_EDITOR_VIEWER_SECTION "External editor or viewer parameters"
 
 /*** enums ***************************************************************************************/
 
@@ -29,8 +30,8 @@ extern mc_config_t *mc_panels_config;
 
 /* mcconfig/common.c: */
 
-mc_config_t *mc_config_init (const gchar *);
-void mc_config_deinit (mc_config_t *);
+mc_config_t *mc_config_init (const gchar * ini_path, gboolean read_only);
+void mc_config_deinit (mc_config_t * mc_config);
 
 gboolean mc_config_del_key (mc_config_t *, const char *, const gchar *);
 gboolean mc_config_del_group (mc_config_t *, const char *);
@@ -38,7 +39,7 @@ gboolean mc_config_del_group (mc_config_t *, const char *);
 gboolean mc_config_has_param (const mc_config_t *, const char *, const gchar *);
 gboolean mc_config_has_group (mc_config_t *, const char *);
 
-gboolean mc_config_read_file (mc_config_t * mc_config, const gchar * ini_path,
+gboolean mc_config_read_file (mc_config_t * mc_config, const gchar * ini_path, gboolean read_only,
                               gboolean remove_empty);
 
 gboolean mc_config_save_file (mc_config_t * config, GError ** error);
@@ -100,9 +101,7 @@ void mc_config_init_config_paths (GError ** error);
 
 void mc_config_deinit_config_paths (void);
 
-gboolean mc_config_deprecated_dir_present (void);
-
-void mc_config_migrate_from_old_place (GError ** error);
+gboolean mc_config_migrate_from_old_place (GError ** error, char **msg);
 
 const char *mc_config_get_data_path (void);
 
