@@ -13,8 +13,11 @@
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
-typedef int (*mc_search_fn) (const void *user_data, gsize char_offset, int *current_char);
-typedef int (*mc_update_fn) (const void *user_data, gsize char_offset);
+typedef enum mc_search_cbret_t mc_search_cbret_t;
+
+typedef mc_search_cbret_t (*mc_search_fn) (const void *user_data, gsize char_offset,
+                                           int *current_char);
+typedef mc_search_cbret_t (*mc_update_fn) (const void *user_data, gsize char_offset);
 
 #define MC_SEARCH__NUM_REPLACE_ARGS 64
 
@@ -44,14 +47,13 @@ typedef enum
     MC_SEARCH_T_GLOB
 } mc_search_type_t;
 
-typedef enum
+enum mc_search_cbret_t
 {
     MC_SEARCH_CB_OK = 0,
     MC_SEARCH_CB_INVALID = -1,
     MC_SEARCH_CB_ABORT = -2,
     MC_SEARCH_CB_SKIP = -3
-} mc_search_cbret_t;
-
+};
 
 /*** structures declarations (and typedefs of structures)*****************************************/
 
@@ -59,8 +61,10 @@ typedef struct mc_search_struct
 {
     /* public input data */
 
+#ifdef HAVE_CHARSET
     /* search in all charsets */
     gboolean is_all_charsets;
+#endif
 
     /* case sensitive search */
     gboolean is_case_sensitive;
@@ -106,6 +110,9 @@ typedef struct mc_search_struct
     /* original search string */
     gchar *original;
     gsize original_len;
+#ifdef HAVE_CHARSET
+    gchar *original_charset;
+#endif
 
     /* error code after search */
     mc_search_error_t error;
@@ -122,7 +129,8 @@ typedef struct mc_search_type_str_struct
 
 /*** declarations of public functions ************************************************************/
 
-mc_search_t *mc_search_new (const gchar * original, gsize original_len);
+mc_search_t *mc_search_new (const gchar * original, gsize original_len,
+                            const gchar * original_charset);
 
 void mc_search_free (mc_search_t * lc_mc_search);
 
@@ -142,7 +150,8 @@ gboolean mc_search_is_fixed_search_str (mc_search_t *);
 
 gchar **mc_search_get_types_strings_array (size_t * num);
 
-gboolean mc_search (const gchar *, const gchar *, mc_search_type_t);
+gboolean mc_search (const gchar * pattern, const gchar * pattern_charset, const gchar * str,
+                    mc_search_type_t type);
 
 int mc_search_getstart_result_by_num (mc_search_t *, int);
 int mc_search_getend_result_by_num (mc_search_t *, int);
