@@ -2,11 +2,12 @@
    Search text engine.
    Common share code for module.
 
-   Copyright (C) 2009, 2011
+   Copyright (C) 2009, 2011, 2013
    The Free Software Foundation, Inc.
 
    Written by:
-   Slava Zanko <slavazanko@gmail.com>, 2009.
+   Slava Zanko <slavazanko@gmail.com>, 2009, 2011
+   Andrew Borodin <aborodin@vmail.ru>, 2013
 
    This file is part of the Midnight Commander.
 
@@ -32,9 +33,11 @@
 #include "lib/global.h"
 #include "lib/strutil.h"
 #include "lib/search.h"
+#ifdef HAVE_CHARSET
+#include "lib/charsets.h"
+#endif
 
 #include "internal.h"
-#include "lib/charsets.h"
 
 /*** global variables ****************************************************************************/
 
@@ -62,7 +65,8 @@ mc_search__recode_str (const char *str, gsize str_len,
     gsize bytes_read;
     GIConv conv;
 
-    if (charset_from == NULL || charset_to == NULL || !strcmp (charset_to, charset_from))
+    if (charset_from == NULL || charset_to == NULL
+        || g_ascii_strcasecmp (charset_to, charset_from) == 0)
     {
         *bytes_written = str_len;
         return g_strndup (str, str_len);
@@ -137,19 +141,6 @@ mc_search__get_one_symbol (const char *charset, const char *str, gsize str_len,
 
 /* --------------------------------------------------------------------------------------------- */
 
-int
-mc_search__get_char (mc_search_t * lc_mc_search, const void *user_data, gsize current_pos)
-{
-    char *data;
-    if (lc_mc_search->search_fn)
-        return (lc_mc_search->search_fn) (user_data, current_pos);
-
-    data = (char *) user_data;
-    return (int) (unsigned char) data[current_pos];
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 GString *
 mc_search__tolower_case_str (const char *charset, const char *str, gsize str_len)
 {
