@@ -23,7 +23,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file serialize.c
+/** \file lib/serialize.c
  *  \brief Source: serialize/unserialize functionality for INI-like formats.
  */
 
@@ -281,7 +281,7 @@ mc_deserialize_config (const char *data, GError ** error)
 {
     char *current_group = NULL, *current_param = NULL, *current_value = NULL;
     size_t current_position = 0;
-    mc_config_t *ret_data = mc_config_init (NULL);
+    mc_config_t *ret_data;
     enum automat_status
     {
         WAIT_GROUP,
@@ -289,6 +289,8 @@ mc_deserialize_config (const char *data, GError ** error)
         WAIT_VALUE
     } current_status = WAIT_GROUP;
 
+    ret_data = mc_config_init (NULL, FALSE);
+
     while (data != NULL)
     {
         if ((current_status == WAIT_GROUP) && (*data == 'p') && (current_group != NULL))
