@@ -2,11 +2,12 @@
    Skins engine.
    Set of hardcoded skins
 
-   Copyright (C) 2009, 2011
+   Copyright (C) 2009, 2011, 2012
    The Free Software Foundation, Inc.
 
    Written by:
-   Slava Zanko <slavazanko@gmail.com>, 2009.
+   Slava Zanko <slavazanko@gmail.com>, 2009
+   Andrew Borodin <aborodin@vmail.ru>, 2012
 
    This file is part of the Midnight Commander.
 
@@ -75,6 +76,8 @@ mc_skin_hardcoded_blackwhite_colors (mc_skin_t * mc_skin)
     mc_config_set_string (mc_skin->config, "viewer", "viewunderline", "A_UNDERLINE");
     mc_config_set_string (mc_skin->config, "editor", "editbold", "A_BOLD");
     mc_config_set_string (mc_skin->config, "editor", "editmarked", "A_REVERSE");
+    mc_config_set_string (mc_skin->config, "editor", "editframeactive", "A_BOLD");
+    mc_config_set_string (mc_skin->config, "editor", "editframedrag", "A_REVERSE");
     mc_config_set_string (mc_skin->config, "buttonbar", "hotkey", "default");
     mc_config_set_string (mc_skin->config, "buttonbar", "button", "A_REVERSE");
 }
