@@ -415,7 +415,7 @@ load_qnx_key_defines (void)
 
     if (!_qnx_keys_defined)
     {
-        int idx, str_idx;
+        int idx;
         int term_setup_ok;
 
         __setupterm (NULL, fileno (stdout), &term_setup_ok);
@@ -424,7 +424,8 @@ load_qnx_key_defines (void)
 
         for (idx = 0; idx < sizeof (xtra_key_defines) / sizeof (xtra_key_defines[0]); idx++)
         {
-            str_idx = xtra_key_defines[idx].str_idx;
+            int str_idx = xtra_key_defines[idx].str_idx;
+
             if (__QTISOFFS (str_idx))
             {
                 if (*__QTISSTR (str_idx))
