@@ -34,10 +34,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <termios.h>
 #include <sys/types.h>          /* size_t */
 #include <unistd.h>
-#include <signal.h>
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif
+#include <termios.h>
 
 #include "lib/global.h"
 #include "lib/strutil.h"        /* str_term_form */
@@ -59,10 +61,6 @@ extern int reset_hp_softkeys;
 
 /*** file scope macro definitions ****************************************************************/
 
-#ifndef SA_RESTART
-#define SA_RESTART 0
-#endif
-
 #ifndef SLTT_MAX_SCREEN_COLS
 #define SLTT_MAX_SCREEN_COLS 512
 #endif
@@ -128,22 +126,14 @@ static const struct
     /* *INDENT-ON* */
 };
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
 static void
 tty_setup_sigwinch (void (*handler) (int))
 {
-#ifdef SIGWINCH
-    struct sigaction act, oact;
-    act.sa_handler = handler;
-    sigemptyset (&act.sa_mask);
-    act.sa_flags = 0;
-#ifdef SA_RESTART
-    act.sa_flags |= SA_RESTART;
-#endif /* SA_RESTART */
-    sigaction (SIGWINCH, &act, &oact);
-#endif /* SIGWINCH */
+    (void) SLsignal (SIGWINCH, handler);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -153,8 +143,8 @@ sigwinch_handler (int dummy)
 {
     (void) dummy;
 
-    tty_change_screen_size ();
-    mc_global.tty.winch_flag = TRUE;
+    mc_global.tty.winch_flag = 1;
+    (void) SLsignal (SIGWINCH, sigwinch_handler);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -178,12 +168,13 @@ static void
 slang_reset_softkeys (void)
 {
     int key;
-    char *send;
     static const char display[] = "                ";
     char tmp[BUF_SMALL];
 
     for (key = 1; key < 9; key++)
     {
+        char *send;
+
         g_snprintf (tmp, sizeof (tmp), "k%d", key);
         send = (char *) SLtt_tgetstr (tmp);
         if (send != NULL)
@@ -338,7 +329,6 @@ tty_init (gboolean mouse_enable, gboolean is_xterm)
     if (!mouse_enable)
         use_mouse_p = MOUSE_DISABLED;
     tty_init_xterm_support (is_xterm);  /* do it before do_enter_ca_mode() call */
-    init_mouse ();
     do_enter_ca_mode ();
     tty_keypad (TRUE);
     tty_nodelay (FALSE);
@@ -354,6 +344,7 @@ tty_shutdown (void)
     char *op_cap;
 
     disable_mouse ();
+    disable_bracketed_paste ();
     tty_reset_shell_mode ();
     tty_noraw_mode ();
     tty_keypad (FALSE);
@@ -372,6 +363,20 @@ tty_shutdown (void)
     }
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+void
+tty_change_screen_size (void)
+{
+    SLtt_get_screen_size ();
+    SLsmg_reinit_smg ();
+
+#ifdef ENABLE_SUBSHELL
+    if (mc_global.tty.use_subshell)
+        tty_resize (mc_global.tty.subshell_pty);
+#endif
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /* Done each time we come back from done mode */
 
@@ -668,11 +673,12 @@ tty_print_alt_char (int c, gboolean single)
 void
 tty_print_anychar (int c)
 {
-    char str[6 + 1];
-
     if (c > 255)
     {
-        int res = g_unichar_to_utf8 (c, str);
+        char str[UTF8_CHAR_LEN + 1];
+        int res;
+
+        res = g_unichar_to_utf8 (c, str);
         if (res == 0)
         {
             str[0] = '.';
