@@ -31,10 +31,6 @@
 
 #include <config.h>
 
-#ifndef HAVE_TEXTMODE_X11_SUPPORT
-typedef int dummy;              /* C99 forbids empty compilation unit */
-#else
-
 #include <setjmp.h>
 #include <X11/Xlib.h>
 #ifdef HAVE_GMODULE
@@ -183,14 +179,18 @@ x11_available (void)
 Display *
 mc_XOpenDisplay (const char *displayname)
 {
-    Display *retval;
-
     if (x11_available ())
     {
         if (setjmp (x11_exception) == 0)
         {
+            Display *retval;
+
+            /* cppcheck-suppress redundantAssignment */
             longjmp_allowed = TRUE;
+
             retval = func_XOpenDisplay (displayname);
+
+            /* cppcheck-suppress redundantAssignment */
             longjmp_allowed = FALSE;
             return retval;
         }
@@ -203,15 +203,20 @@ mc_XOpenDisplay (const char *displayname)
 int
 mc_XCloseDisplay (Display * display)
 {
-    int retval;
-
     if (x11_available ())
     {
         if (setjmp (x11_exception) == 0)
         {
+            int retval;
+
+            /* cppcheck-suppress redundantAssignment */
             longjmp_allowed = TRUE;
+
             retval = func_XCloseDisplay (display);
+
+            /* cppcheck-suppress redundantAssignment */
             longjmp_allowed = FALSE;
+
             return retval;
         }
     }
@@ -231,11 +236,16 @@ mc_XQueryPointer (Display * display, Window win, Window * root_return,
     {
         if (setjmp (x11_exception) == 0)
         {
+            /* cppcheck-suppress redundantAssignment */
             longjmp_allowed = TRUE;
+
             retval = func_XQueryPointer (display, win, root_return,
                                          child_return, root_x_return, root_y_return,
                                          win_x_return, win_y_return, mask_return);
+
+            /* cppcheck-suppress redundantAssignment */
             longjmp_allowed = FALSE;
+
             return retval;
         }
     }
@@ -250,5 +260,3 @@ mc_XQueryPointer (Display * display, Window win, Window * root_return,
 }
 
 /* --------------------------------------------------------------------------------------------- */
-
-#endif /* HAVE_TEXTMODE_X11_SUPPORT */
