@@ -213,6 +213,7 @@ is_year (char *str, struct tm *tim)
     if (strlen (str) != 4)
         return 0;
 
+    /* cppcheck-suppress invalidscanf */
     if (sscanf (str, "%ld", &year) != 1)
         return 0;
 
@@ -570,6 +571,7 @@ vfs_parse_filedate (int idx, time_t * t)
         {
             p[2] = p[5] = '-';
 
+            /* cppcheck-suppress invalidscanf */
             if (sscanf (p, "%2d-%2d-%d", &d[0], &d[1], &d[2]) == 3)
             {
                 /* Months are zero based */
@@ -755,14 +757,17 @@ vfs_parse_ls_lga (const char *p, struct stat * s, char **filename, char **linkna
         /* Corner case: there is no whitespace(s) between maj & min */
         if (!is_num (idx2) && idx2 == 2)
         {
+            /* cppcheck-suppress invalidscanf */
             if (!is_num (++idx2) || sscanf (columns[idx2], " %d,%d", &maj, &min) != 2)
                 goto error;
         }
         else
         {
+            /* cppcheck-suppress invalidscanf */
             if (!is_num (idx2) || sscanf (columns[idx2], " %d,", &maj) != 1)
                 goto error;
 
+            /* cppcheck-suppress invalidscanf */
             if (!is_num (++idx2) || sscanf (columns[idx2], " %d", &min) != 1)
                 goto error;
         }
@@ -800,7 +805,7 @@ vfs_parse_ls_lga (const char *p, struct stat * s, char **filename, char **linkna
     if (num_spaces != NULL)
     {
         *num_spaces = column_ptr[idx] - column_ptr[idx - 1] - strlen (columns[idx - 1]);
-        if (strcmp (columns[idx], "..") == 0)
+        if (DIR_IS_DOTDOT (columns[idx]))
             vfs_parce_ls_final_num_spaces = *num_spaces;
     }
 
