@@ -239,7 +239,7 @@ vfs_path_url_split (vfs_path_element_t * path_element, const char *path)
 {
     char *pcopy;
     const char *pend;
-    char *colon, *inner_colon, *at, *rest;
+    char *colon, *at, *rest;
 
     path_element->port = 0;
 
@@ -254,6 +254,8 @@ vfs_path_url_split (vfs_path_element_t * path_element, const char *path)
         rest = pcopy;
     else
     {
+        char *inner_colon;
+
         *at = '\0';
         inner_colon = strchr (pcopy, ':');
         if (inner_colon != NULL)
@@ -290,6 +292,7 @@ vfs_path_url_split (vfs_path_element_t * path_element, const char *path)
     if (colon != NULL)
     {
         *colon = '\0';
+        /* cppcheck-suppress invalidscanf */
         if (sscanf (colon + 1, "%d", &path_element->port) == 1)
         {
             if (path_element->port <= 0 || path_element->port >= 65536)
@@ -443,7 +446,6 @@ vfs_path_from_str_uri_parser (char *path, vfs_path_flag_t flags)
     {
         char *vfs_prefix_start;
         char *real_vfs_prefix_start = url_delimiter;
-        char *slash_pointer;
         struct vfs_s_subclass *sub = NULL;
 
         while (real_vfs_prefix_start > path && *(real_vfs_prefix_start) != PATH_SEP)
@@ -463,6 +465,8 @@ vfs_path_from_str_uri_parser (char *path, vfs_path_flag_t flags)
         sub = VFSDATA (element);
         if (sub != NULL && (sub->flags & VFS_S_REMOTE) != 0)
         {
+            char *slash_pointer;
+
             slash_pointer = strchr (url_delimiter, PATH_SEP);
             if (slash_pointer == NULL)
             {
@@ -739,7 +743,7 @@ vfs_path_from_str_flags (const char *path_str, vfs_path_flag_t flags)
     else
         vpath = vfs_path_from_str_uri_parser (path, flags);
 
-    vpath->str = vfs_path_to_str_flags (vpath, 0, VPF_NONE);
+    vpath->str = vfs_path_to_str_flags (vpath, 0, flags);
     g_free (path);
 
     return vpath;
@@ -1338,19 +1342,22 @@ vfs_path_tokens_count (const vfs_path_t * vpath)
     for (element_index = 0; element_index < vfs_path_elements_count (vpath); element_index++)
     {
         const vfs_path_element_t *element;
-        char **path_tokens, **iterator;
+        const char *token, *prev_token;
 
         element = vfs_path_get_by_index (vpath, element_index);
-        path_tokens = iterator = g_strsplit (element->path, PATH_SEP_STR, -1);
 
-        while (*iterator != NULL)
+        for (prev_token = element->path; (token = strchr (prev_token, PATH_SEP)) != NULL;
+             prev_token = token + 1)
         {
-            if (**iterator != '\0')
+            /* skip empty substring */
+            if (token != prev_token)
                 count_tokens++;
-            iterator++;
         }
-        g_strfreev (path_tokens);
+
+        if (*prev_token != '\0')
+            count_tokens++;
     }
+
     return count_tokens;
 }
 
@@ -1405,9 +1412,9 @@ vfs_path_tokens_get (const vfs_path_t * vpath, ssize_t start_position, ssize_t l
 
         g_string_assign (element_tokens, "");
         element = vfs_path_get_by_index (vpath, element_index);
-        path_tokens = iterator = g_strsplit (element->path, PATH_SEP_STR, -1);
+        path_tokens = g_strsplit (element->path, PATH_SEP_STR, -1);
 
-        while (*iterator != NULL)
+        for (iterator = path_tokens; *iterator != NULL; iterator++)
         {
             if (**iterator != '\0')
             {
@@ -1428,7 +1435,6 @@ vfs_path_tokens_get (const vfs_path_t * vpath, ssize_t start_position, ssize_t l
                 else
                     start_position--;
             }
-            iterator++;
         }
         g_strfreev (path_tokens);
         vfs_path_tokens_add_class_info (element, ret_tokens, element_tokens);
