@@ -10,6 +10,8 @@
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
+#define INPUT(x) ((WInput *)(x))
+
 /* For history load-save functions */
 #define INPUT_LAST_TEXT ((char *) 2)
 
@@ -27,6 +29,7 @@ typedef enum
 /* completion flags */
 typedef enum
 {
+    INPUT_COMPLETE_NONE = 0,
     INPUT_COMPLETE_FILENAMES = 1 << 0,
     INPUT_COMPLETE_HOSTNAMES = 1 << 1,
     INPUT_COMPLETE_COMMANDS = 1 << 2,
@@ -34,9 +37,6 @@ typedef enum
     INPUT_COMPLETE_USERNAMES = 1 << 4,
     INPUT_COMPLETE_CD = 1 << 5,
     INPUT_COMPLETE_SHELL_ESC = 1 << 6,
-
-    INPUT_COMPLETE_DEFAULT = INPUT_COMPLETE_FILENAMES
-        | INPUT_COMPLETE_HOSTNAMES | INPUT_COMPLETE_VARIABLES | INPUT_COMPLETE_USERNAMES
 } input_complete_t;
 
 /*** structures declarations (and typedefs of structures)*****************************************/
@@ -52,22 +52,25 @@ typedef struct
     gboolean highlight;         /* there is a selected block */
     int term_first_shown;       /* column of the first shown character */
     size_t current_max_size;    /* maximum length of input line (bytes) */
-    int field_width;            /* width of the editing field */
     gboolean first;             /* is first keystroke? */
     int disable_update;         /* do we want to skip updates? */
     gboolean is_password;       /* is this a password input line? */
-    char *init_text;            /* initial text of input line */
+    gboolean init_from_history; /* init text will be get from history */
     char *buffer;               /* pointer to editing buffer */
-    char *history_name;         /* name of history for loading and saving */
-    GList *history;             /* the history */
-    GList *history_current;     /* current history item */
-    gboolean history_changed;   /* the history has changed */
     gboolean need_push;         /* need to push the current Input on hist? */
     gboolean strip_password;    /* need to strip password before placing string to history */
     char **completions;         /* possible completions array */
     input_complete_t completion_flags;
     char charbuf[MB_LEN_MAX];   /* buffer for multibytes characters */
     size_t charpoint;           /* point to end of mulibyte sequence in charbuf */
+    WLabel *label;              /* label associated with this input line */
+    struct input_history_t
+    {
+        char *name;             /* name of history for loading and saving */
+        GList *list;            /* the history */
+        GList *current;         /* current history item */
+        gboolean changed;       /* the history has changed */
+    } history;
 } WInput;
 
 /*** global variables defined in .c file *********************************************************/
@@ -82,9 +85,8 @@ WInput *input_new (int y, int x, const int *input_colors,
                    int len, const char *text, const char *histname,
                    input_complete_t completion_flags);
 /* callbac is public; needed for command line */
-cb_ret_t input_callback (Widget * w, widget_msg_t msg, int parm);
+cb_ret_t input_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data);
 const int *input_get_default_colors (void);
-void input_set_origin (WInput * i, int x, int field_width);
 cb_ret_t input_handle_char (WInput * in, int key);
 int input_key_is_in_map (WInput * in, int key);
 void input_assign_text (WInput * in, const char *text);
