@@ -2,7 +2,7 @@
    Widgets for the Midnight Commander
 
    Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2010, 2011
+   2004, 2005, 2006, 2007, 2009, 2010, 2011, 2012, 2013
    The Free Software Foundation, Inc.
 
    Authors:
@@ -11,7 +11,7 @@
    Jakub Jelinek, 1995
    Andrej Borsenkow, 1996
    Norbert Warmuth, 1997
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010
+   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2011, 2012, 2013
 
    This file is part of the Midnight Commander.
 
@@ -138,15 +138,17 @@ hotkey_draw (Widget * w, const hotkey_t hotkey, gboolean focused)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-init_widget (Widget * w, int y, int x, int lines, int cols,
-             callback_fn callback, mouse_h mouse_handler)
+widget_init (Widget * w, int y, int x, int lines, int cols,
+             widget_cb_fn callback, mouse_h mouse_handler)
 {
     w->x = x;
     w->y = y;
     w->cols = cols;
     w->lines = lines;
+    w->pos_flags = WPOS_KEEP_DEFAULT;
     w->callback = callback;
     w->mouse = mouse_handler;
+    w->set_options = widget_default_set_options_callback;
     w->owner = NULL;
 
     /* Almost all widgets want to put the cursor in a suitable place */
@@ -157,19 +159,22 @@ init_widget (Widget * w, int y, int x, int lines, int cols,
 
 /* Default callback for widgets */
 cb_ret_t
-default_proc (widget_msg_t msg, int parm)
+widget_default_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
+    (void) w;
+    (void) sender;
     (void) parm;
+    (void) data;
 
     switch (msg)
     {
-    case WIDGET_INIT:
-    case WIDGET_FOCUS:
-    case WIDGET_UNFOCUS:
-    case WIDGET_DRAW:
-    case WIDGET_DESTROY:
-    case WIDGET_CURSOR:
-    case WIDGET_IDLE:
+    case MSG_INIT:
+    case MSG_FOCUS:
+    case MSG_UNFOCUS:
+    case MSG_DRAW:
+    case MSG_DESTROY:
+    case MSG_CURSOR:
+    case MSG_IDLE:
         return MSG_HANDLED;
 
     default:
@@ -179,6 +184,42 @@ default_proc (widget_msg_t msg, int parm)
 
 /* --------------------------------------------------------------------------------------------- */
 
+/**
+ * Callback for applying new options to widget.
+ *
+ * @param w       widget
+ * @param options options set
+ * @param enable  TRUE if specified options should be added, FALSE if options should be removed
+ */
+void
+widget_default_set_options_callback (Widget * w, widget_options_t options, gboolean enable)
+{
+    if (enable)
+        w->options |= options;
+    else
+        w->options &= ~options;
+
+    if (w->owner != NULL && (options & W_DISABLED) != 0)
+        send_message (w, NULL, MSG_DRAW, 0, NULL);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+/**
+ * Apply new options to widget.
+ *
+ * @param w       widget
+ * @param options options set
+ * @param enable  TRUE if specified options should be added, FALSE if options should be removed
+ */
+void
+widget_set_options (Widget * w, widget_options_t options, gboolean enable)
+{
+    w->set_options (w, options, enable);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 void
 widget_set_size (Widget * widget, int y, int x, int lines, int cols)
 {
@@ -186,7 +227,7 @@ widget_set_size (Widget * widget, int y, int x, int lines, int cols)
     widget->y = y;
     widget->cols = cols;
     widget->lines = lines;
-    send_message (widget, WIDGET_RESIZED, 0 /* unused */ );
+    send_message (widget, NULL, MSG_RESIZE, 0, NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -194,7 +235,7 @@ widget_set_size (Widget * widget, int y, int x, int lines, int cols)
 void
 widget_selectcolor (Widget * w, gboolean focused, gboolean hotkey)
 {
-    Dlg_head *h = w->owner;
+    WDialog *h = w->owner;
     int color;
 
     if ((w->options & W_DISABLED) != 0)
@@ -227,8 +268,92 @@ widget_erase (Widget * w)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+/**
+  * Check whether widget is active or not.
+  * @param w the widget
+  *
+  * @return TRUE if the widget is active, FALSE otherwise
+  */
 
+gboolean
+widget_is_active (const void *w)
+{
+    return (w == WIDGET (w)->owner->current->data);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+void
+widget_redraw (Widget * w)
+{
+    if (w != NULL)
+    {
+        WDialog *h = w->owner;
+
+        if (h != NULL && h->state == DLG_ACTIVE)
+            w->callback (w, NULL, MSG_DRAW, 0, NULL);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+  * Replace widget in the dialog.
+  *
+  * @param old_w old widget that need to be replaced
+  * @param new_w new widget that will replace @old_w
+  */
+
+void
+widget_replace (Widget * old_w, Widget * new_w)
+{
+    WDialog *h = old_w->owner;
+    gboolean should_focus = FALSE;
+
+    if (h->widgets == NULL)
+        return;
+
+    if (h->current == NULL)
+        h->current = h->widgets;
+
+    if (old_w == h->current->data)
+        should_focus = TRUE;
+
+    new_w->owner = h;
+    new_w->id = old_w->id;
+
+    if (should_focus)
+        h->current->data = new_w;
+    else
+        g_list_find (h->widgets, old_w)->data = new_w;
+
+    send_message (old_w, NULL, MSG_DESTROY, 0, NULL);
+    send_message (new_w, NULL, MSG_INIT, 0, NULL);
+
+    if (should_focus)
+        dlg_select_widget (new_w);
+
+    widget_redraw (new_w);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+  * Check whether two widgets are overlapped or not.
+  * @param a 1st widget
+  * @param b 2nd widget
+  *
+  * @return TRUE if widgets are overlapped, FALSE otherwise.
+  */
+
+gboolean
+widget_overlapped (const Widget * a, const Widget * b)
+{
+    return !((b->x >= a->x + a->cols)
+             || (a->x >= b->x + b->cols) || (b->y >= a->y + a->lines) || (a->y >= b->y + b->lines));
+}
+
+/* --------------------------------------------------------------------------------------------- */
 /* get mouse pointer location within widget */
+
 Gpm_Event
 mouse_get_local (const Gpm_Event * global, const Widget * w)
 {
