@@ -38,7 +38,6 @@
 #include "src/vfs/smbfs/smbfs.h"        /* smbfs_set_debugf()  */
 #endif
 
-#include "src/main.h"
 #include "src/textconf.h"
 
 #include "src/args.h"
@@ -72,6 +71,9 @@ char *mc_args__keymap_file = NULL;
 /* Debug level */
 int mc_args__debug_level = 0;
 
+void *mc_run_param0 = NULL;
+char *mc_run_param1 = NULL;
+
 /*** file scope macro definitions ****************************************************************/
 
 /*** file scope type declarations ****************************************************************/
@@ -86,9 +88,9 @@ static gboolean parse_mc_v_argument (const gchar * option_name, const gchar * va
 
 static GOptionContext *context;
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
 static gboolean mc_args__nouse_subshell = FALSE;
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
 static gboolean mc_args__show_datadirs = FALSE;
 static gboolean mc_args__show_datadirs_extended = FALSE;
 static gboolean mc_args__show_configure_opts = FALSE;
@@ -136,7 +138,7 @@ static const GOptionEntry argument_main_table[] = {
      "<file>"
     },
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
     {
      "subshell", 'U', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
      &mc_global.tty.use_subshell,
@@ -405,14 +407,12 @@ mc_args_add_extended_info_to_help (void)
     mc_args__loc__footer_string = g_strdup_printf ("%s",
                                                    _
                                                    ("\n"
-                                                    "Please send any bug reports (including the output of `mc -V')\n"
+                                                    "Please send any bug reports (including the output of 'mc -V')\n"
                                                     "as tickets at www.midnight-commander.org\n"));
     mc_args__loc__header_string = g_strdup_printf (_("GNU Midnight Commander %s\n"), VERSION);
 
-#if GLIB_CHECK_VERSION(2,12,0)
     g_option_context_set_description (context, mc_args__loc__footer_string);
     g_option_context_set_summary (context, mc_args__loc__header_string);
-#endif
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -481,7 +481,7 @@ parse_mcedit_arguments (int argc, char **argv)
 {
     GList *flist = NULL;
     int i;
-    int first_line_number = -1;
+    long first_line_number = -1;
 
     for (i = 0; i < argc; i++)
     {
@@ -504,7 +504,7 @@ parse_mcedit_arguments (int argc, char **argv)
             if (*error == '\0')
             {
                 /* this is line number */
-                first_line_number = (int) lineno;
+                first_line_number = lineno;
                 continue;
             }
             /* this is file name */
@@ -545,28 +545,25 @@ parse_mcedit_arguments (int argc, char **argv)
             }
             else
             {
-                arg = mcedit_arg_vpath_new (tmp_vpath, 1);
+                arg = mcedit_arg_vpath_new (tmp_vpath, 0);
                 vfs_path_free (fname_vpath);
             }
 
             g_free (fname);
         }
         else
-            arg = mcedit_arg_new (tmp, 1);
+            arg = mcedit_arg_new (tmp, 0);
 
         flist = g_list_prepend (flist, arg);
     }
 
     if (flist == NULL)
-        flist = g_list_prepend (flist, mcedit_arg_new (NULL, 1));
+        flist = g_list_prepend (flist, mcedit_arg_new (NULL, 0));
     else if (first_line_number != -1)
     {
         /* overwrite line number for first file */
         GList *l;
 
-        if (first_line_number == 0)
-            first_line_number = 1;
-
         l = g_list_last (flist);
         ((mcedit_arg_t *) l->data)->line_number = first_line_number;
     }
@@ -707,10 +704,10 @@ mc_setup_by_args (int argc, char **argv, GError ** error)
     if (mc_args__force_colors)
         mc_global.tty.disable_colors = FALSE;
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
     if (mc_args__nouse_subshell)
         mc_global.tty.use_subshell = FALSE;
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
 
 #ifdef ENABLE_VFS_SMB
     if (mc_args__debug_level != 0)
@@ -819,12 +816,12 @@ mc_setup_by_args (int argc, char **argv, GError ** error)
  * Create mcedit_arg_t object from file name and the line number.
  *
  * @param file_name   file name
- * @param line_number line number
+ * @param line_number line number. If value is 0, try to restore saved position.
  * @return mcedit_arg_t object
  */
 
 mcedit_arg_t *
-mcedit_arg_new (const char *file_name, int line_number)
+mcedit_arg_new (const char *file_name, long line_number)
 {
     return mcedit_arg_vpath_new (vfs_path_from_str (file_name), line_number);
 }
@@ -834,19 +831,17 @@ mcedit_arg_new (const char *file_name, int line_number)
  * Create mcedit_arg_t object from vfs_path_t object and the line number.
  *
  * @param file_vpath  file path object
- * @param line_number line number
+ * @param line_number line number. If value is 0, try to restore saved position.
  * @return mcedit_arg_t object
  */
 
 mcedit_arg_t *
-mcedit_arg_vpath_new (vfs_path_t * file_vpath, int line_number)
+mcedit_arg_vpath_new (vfs_path_t * file_vpath, long line_number)
 {
     mcedit_arg_t *arg;
 
     arg = g_new (mcedit_arg_t, 1);
     arg->file_vpath = file_vpath;
-    if (line_number == 0)
-        line_number = 1;
     arg->line_number = line_number;
 
     return arg;
