@@ -1,11 +1,11 @@
 /*
-   Copyright (C) 2007, 2010, 2011
+   Copyright (C) 2007, 2010, 2011, 2012, 2013
    The Free Software Foundation, Inc.
 
    Written by:
    Daniel Borca <dborca@yahoo.com>, 2007
-   Slava Zanko <slavazanko@gmail.com>, 2010
-   Andrew Borodin <aborodin@vmail.ru>, 2010
+   Slava Zanko <slavazanko@gmail.com>, 2010, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2010, 2012, 2013
    Ilia Maslakov <il.smind@gmail.com>, 2010
 
    This file is part of the Midnight Commander.
@@ -43,6 +43,7 @@
 #include "lib/util.h"
 #include "lib/widget.h"
 #include "lib/strutil.h"
+#include "lib/strescape.h"      /* strutils_glob_escape() */
 #ifdef HAVE_CHARSET
 #include "lib/charsets.h"
 #endif
@@ -53,6 +54,7 @@
 #include "src/filemanager/layout.h"     /* Needed for get_current_index and get_other_panel */
 
 #include "src/keybind-defaults.h"
+#include "src/setup.h"
 #include "src/history.h"
 #ifdef HAVE_CHARSET
 #include "src/selcodepage.h"
@@ -68,9 +70,11 @@
 #define g_array_foreach(a, TP, cbf) \
 do { \
     size_t g_array_foreach_i;\
-    TP *g_array_foreach_var = NULL; \
+    \
     for (g_array_foreach_i = 0; g_array_foreach_i < a->len; g_array_foreach_i++) \
     { \
+        TP *g_array_foreach_var; \
+        \
         g_array_foreach_var = &g_array_index (a, TP, g_array_foreach_i); \
         (*cbf) (g_array_foreach_var); \
     } \
@@ -79,9 +83,6 @@ do { \
 #define FILE_READ_BUF 4096
 #define FILE_FLAG_TEMP (1 << 0)
 
-#define OPTX 56
-#define OPTY 17
-
 #define ADD_CH '+'
 #define DEL_CH '-'
 #define CHG_CH '*'
@@ -157,7 +158,7 @@ rewrite_backup_content (const vfs_path_t * from_file_name_vpath, const char *to_
  * @note the name is not altered if this function fails
  *
  * @param[out] name address of a pointer to store the temporary name
- * @returns file descriptor on success, negative on error
+ * @return file descriptor on success, negative on error
  */
 
 static int
@@ -173,7 +174,7 @@ open_temp (void **name)
                  _("Cannot create temporary diff file\n%s"), unix_error_string (errno));
         return -1;
     }
-    *name = vfs_path_to_str (diff_file_name);
+    *name = g_strdup (vfs_path_as_str (diff_file_name));
     vfs_path_free (diff_file_name);
     return fd;
 }
@@ -184,7 +185,7 @@ open_temp (void **name)
  * Alocate file structure and associate file descriptor to it.
  *
  * @param fd file descriptor
- * @returns file structure
+ * @return file structure
  */
 
 static FBUF *
@@ -220,7 +221,7 @@ f_dopen (int fd)
  * Free file structure without closing the file.
  *
  * @param fs file structure
- * @returns 0 on success, non-zero on error
+ * @return 0 on success, non-zero on error
  */
 
 static int
@@ -244,8 +245,7 @@ f_free (FBUF * fs)
  * Open a binary temporary file in R/W mode.
  * @note the file will be deleted when closed
  *
- * @returns file structure
- *
+ * @return file structure
  */
 static FBUF *
 f_temp (void)
@@ -277,7 +277,7 @@ f_temp (void)
  * @param filename file name
  * @param flags open mode, a combination of O_RDONLY, O_WRONLY, O_RDWR
  *
- * @returns file structure
+ * @return file structure
  */
 
 static FBUF *
@@ -312,8 +312,7 @@ f_open (const char *filename, int flags)
  * @param size size of buffer
  * @param fs file structure
  *
- * @returns number of bytes read
- *
+ * @return number of bytes read
  */
 
 static size_t
@@ -355,8 +354,7 @@ f_gets (char *buf, size_t size, FBUF * fs)
  * @param off offset
  * @param whence seek directive: SEEK_SET, SEEK_CUR or SEEK_END
  *
- * @returns position in file, starting from begginning
- *
+ * @return position in file, starting from begginning
  */
 
 static off_t
@@ -395,7 +393,7 @@ f_seek (FBUF * fs, off_t off, int whence)
  *
  * @param fs file structure
  *
- * @returns 0 if success, non-zero on error
+ * @return 0 if success, non-zero on error
  */
 
 static off_t
@@ -419,8 +417,7 @@ f_reset (FBUF * fs)
  * @param buf source buffer
  * @param size size of buffer
  *
- * @returns number of written bytes, -1 on error
- *
+ * @return number of written bytes, -1 on error
  */
 
 static ssize_t
@@ -442,8 +439,7 @@ f_write (FBUF * fs, const char *buf, size_t size)
  *
  * @param fs file structure
  *
- * @returns current file size on success, negative on error
- *
+ * @return current file size on success, negative on error
  */
 
 static off_t
@@ -472,17 +468,20 @@ f_trunc (FBUF * fs)
  * @note if this is temporary file, it is deleted
  *
  * @param fs file structure
- * @returns 0 on success, non-zero on error
- *
+ * @return 0 on success, non-zero on error
  */
 
 static int
 f_close (FBUF * fs)
 {
-    int rv;
+    int rv = -1;
+
+    if (fs != NULL)
+    {
+        rv = close (fs->fd);
+        f_free (fs);
+    }
 
-    rv = close (fs->fd);
-    f_free (fs);
     return rv;
 }
 
@@ -494,7 +493,7 @@ f_close (FBUF * fs)
  * @param cmd shell command line
  * @param flags open mode, either O_RDONLY or O_WRONLY
  *
- * @returns file structure
+ * @return file structure
  */
 
 static FBUF *
@@ -534,25 +533,30 @@ p_open (const char *cmd, int flags)
  * Close pipe stream.
  *
  * @param fs structure
- * @returns 0 on success, non-zero on error
+ * @return 0 on success, non-zero on error
  */
 
 static int
 p_close (FBUF * fs)
 {
-    int rv;
+    int rv = -1;
+
+    if (fs != NULL)
+    {
+        rv = pclose (fs->data);
+        f_free (fs);
+    }
 
-    rv = pclose (fs->data);
-    f_free (fs);
     return rv;
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
 /**
  * Get one char (byte) from string
  *
  * @param char * str, gboolean * result
- * @returns int as character or 0 and result == FALSE if fail
+ * @return int as character or 0 and result == FALSE if fail
  */
 
 static int
@@ -573,8 +577,7 @@ dview_get_byte (char *str, gboolean * result)
  * Get utf multibyte char from string
  *
  * @param char * str, int * char_width, gboolean * result
- * @returns int as utf character or 0 and result == FALSE if fail
- *
+ * @return int as utf character or 0 and result == FALSE if fail
  */
 
 static int
@@ -655,7 +658,7 @@ dview_str_utf8_offset_to_pos (const char *text, size_t length)
  *
  * @param[in,out] str string to parse
  * @param[out] n extracted number
- * @returns 0 if success, otherwise non-zero
+ * @return 0 if success, otherwise non-zero
  */
 
 static int
@@ -679,7 +682,7 @@ scan_deci (const char **str, int *n)
  *
  * @param p string to parse
  * @param ops list of diff statements
- * @returns 0 if success, otherwise non-zero
+ * @return 0 if success, otherwise non-zero
  */
 
 static int
@@ -755,7 +758,7 @@ scan_line (const char *p, GArray * ops)
  *
  * @param f stream to read from
  * @param ops list of diff statements to fill
- * @returns positive number indicating number of hunks, otherwise negative
+ * @return positive number indicating number of hunks, otherwise negative
  */
 
 static int
@@ -796,7 +799,7 @@ scan_diff (FBUF * f, GArray * ops)
  * @param file2 second file to compare
  * @param ops list of diff statements to fill
  *
- * @returns positive number indicating number of hunks, otherwise negative
+ * @return positive number indicating number of hunks, otherwise negative
  */
 
 static int
@@ -813,8 +816,15 @@ dff_execute (const char *args, const char *extra, const char *file1, const char
     FBUF *f;
     char *cmd;
     int code;
+    char *file1_esc, *file2_esc;
+
+    /* escape potential $ to avoid shell variable substitutions in popen() */
+    file1_esc = strutils_shell_escape (file1);
+    file2_esc = strutils_shell_escape (file2);
+    cmd = g_strdup_printf ("diff %s %s %s %s %s", args, extra, opt, file1_esc, file2_esc);
+    g_free (file1_esc);
+    g_free (file2_esc);
 
-    cmd = g_strdup_printf ("diff %s %s %s \"%s\" \"%s\"", args, extra, opt, file1, file2);
     if (cmd == NULL)
         return -1;
 
@@ -844,7 +854,7 @@ dff_execute (const char *args, const char *extra, const char *file1, const char
  * @param printer printf-like function to be used for displaying
  * @param ctx printer context
  *
- * @returns 0 if success, otherwise non-zero
+ * @return 0 if success, otherwise non-zero
  */
 
 static int
@@ -1021,7 +1031,7 @@ dff_reparse (diff_place_t ord, const char *filename, const GArray * ops, DFUNC p
  * @param ret list of offsets for longest common substrings inside each string
  * @param min minimum length of common substrings
  *
- * @returns 0 if success, nonzero otherwise
+ * @return 0 if success, nonzero otherwise
  */
 
 static int
@@ -1101,8 +1111,8 @@ lcsubstr (const char *s, int m, const char *t, int n, GArray * ret, int min)
         }
     }
 
-    free (Lcurr);
-    free (Lprev);
+    g_free (Lcurr);
+    g_free (Lprev);
     return z;
 }
 
@@ -1118,7 +1128,7 @@ lcsubstr (const char *s, int m, const char *t, int n, GArray * ret, int min)
  * @param hdiff list of horizontal diff ranges to fill
  * @param depth recursion depth
  *
- * @returns 0 if success, nonzero otherwise
+ * @return 0 if success, nonzero otherwise
  */
 
 static gboolean
@@ -1198,7 +1208,7 @@ hdiff_multi (const char *s, const char *t, const BRACKET bracket, int min, GArra
  * @param hdiff list of horizontal diff ranges to fill
  * @param depth recursion depth
  *
- * @returns 0 if success, nonzero otherwise
+ * @return 0 if success, nonzero otherwise
  */
 
 static gboolean
@@ -1233,7 +1243,7 @@ hdiff_scan (const char *s, int m, const char *t, int n, int min, GArray * hdiff,
  * @param hdiff horizontal diff structure
  * @param ord DIFF_LEFT if reading from first file, DIFF_RIGHT if reading from 2nd file
  *
- * @returns TRUE if inside hdiff limits, FALSE otherwise
+ * @return TRUE if inside hdiff limits, FALSE otherwise
  */
 
 static gboolean
@@ -1267,7 +1277,7 @@ is_inside (int k, GArray * hdiff, diff_place_t ord)
  * @param base virtual base of this string, needed to calculate tabs
  * @param ts tab size
  *
- * @returns new virtual base
+ * @return new virtual base
  */
 
 static int
@@ -1304,7 +1314,7 @@ cvt_cpy (char *dst, const char *src, size_t srcsize, int base, int ts)
  * @param base virtual base of this string, needed to calculate tabs
  * @param ts tab size
  *
- * @returns new virtual base
+ * @return new virtual base
  *
  * @note The procedure returns when all bytes are consumed from 'src'
  *       or 'dstsize' bytes are written to 'dst'
@@ -1350,7 +1360,7 @@ cvt_ncpy (char *dst, int dstsize, const char **_src, size_t srcsize, int base, i
  * @param ts tab size
  * @param show_cr show trailing carriage return as ^M
  *
- * @returns negative on error, otherwise number of bytes except padding
+ * @return negative on error, otherwise number of bytes except padding
  */
 
 static int
@@ -1411,8 +1421,7 @@ cvt_mget (const char *src, size_t srcsize, char *dst, int dstsize, int skip, int
                 utf_ch = dview_get_utf ((char *) src, &w, &res);
                 if (w > 1)
                     skip += w - 1;
-                if (!g_unichar_isprint (utf_ch))
-                    utf_ch = '.';
+                (void) utf_ch;
             }
             else
             {
@@ -1447,7 +1456,7 @@ cvt_mget (const char *src, size_t srcsize, char *dst, int dstsize, int skip, int
  * @param ord DIFF_LEFT if reading from first file, DIFF_RIGHT if reading from 2nd file
  * @param att buffer of attributes
  *
- * @returns negative on error, otherwise number of bytes except padding
+ * @return negative on error, otherwise number of bytes except padding
  */
 
 static int
@@ -1513,8 +1522,7 @@ cvt_mgeta (const char *src, size_t srcsize, char *dst, int dstsize, int skip, in
                 utf_ch = dview_get_utf ((char *) src, &w, &res);
                 if (w > 1)
                     skip += w - 1;
-                if (!g_unichar_isprint (utf_ch))
-                    utf_ch = '.';
+                (void) utf_ch;
             }
             else
             {
@@ -1548,7 +1556,7 @@ cvt_mgeta (const char *src, size_t srcsize, char *dst, int dstsize, int skip, in
  * @param ts tab size
  * @param show_cr show trailing carriage return as ^M
  *
- * @returns negative on error, otherwise number of bytes except padding
+ * @return negative on error, otherwise number of bytes except padding
  */
 
 static int
@@ -1791,12 +1799,12 @@ redo_diff (WDiff * dview)
         if (dview->hdiff != NULL)
         {
             size_t i;
-            const DIFFLN *p;
-            const DIFFLN *q;
 
             for (i = 0; i < dview->a[DIFF_LEFT]->len; i++)
             {
                 GArray *h = NULL;
+                const DIFFLN *p;
+                const DIFFLN *q;
 
                 p = &g_array_index (dview->a[DIFF_LEFT], DIFFLN, i);
                 q = &g_array_index (dview->a[DIFF_RIGHT], DIFFLN, i);
@@ -1958,7 +1966,7 @@ find_next_hunk (const GArray * a, size_t pos)
  * Find start and end lines of the current hunk.
  *
  * @param dview WDiff widget
- * @returns boolean and
+ * @return boolean and
  * start_line1 first line of current hunk (file[0])
  * end_line1 last line of current hunk (file[0])
  * start_line1 first line of current hunk (file[0])
@@ -2176,7 +2184,6 @@ static void
 do_merge_hunk (WDiff * dview, action_direction_t merge_direction)
 {
     int from1, to1, from2, to2;
-    int res;
     int hunk;
     diff_place_t n_merge = (merge_direction == FROM_RIGHT_TO_LEFT) ? DIFF_RIGHT : DIFF_LEFT;
 
@@ -2233,7 +2240,12 @@ do_merge_hunk (WDiff * dview, action_direction_t merge_direction)
         }
         fflush (merge_file);
         fclose (merge_file);
-        res = rewrite_backup_content (merge_file_name_vpath, dview->file[n_merge]);
+        {
+            int res;
+
+            res = rewrite_backup_content (merge_file_name_vpath, dview->file[n_merge]);
+            (void) res;
+        }
         mc_unlink (merge_file_name_vpath);
         vfs_path_free (merge_file_name_vpath);
     }
@@ -2342,35 +2354,31 @@ dview_diff_options (WDiff * dview)
         N_("&Minimal (Find a smaller set of change)")
     };
 
-    QuickWidget diffopt_widgets[] = {
-        QUICK_BUTTON (6, 10, 14, OPTY, N_("&Cancel"), B_CANCEL, NULL),
-        QUICK_BUTTON (2, 10, 14, OPTY, N_("&OK"), B_ENTER, NULL),
-
-        QUICK_CHECKBOX (3, OPTX, 12, OPTY,
-                        N_("Strip &trailing carriage return"), &dview->opt.strip_trailing_cr),
-        QUICK_CHECKBOX (3, OPTX, 11, OPTY,
-                        N_("Ignore all &whitespace"), &dview->opt.ignore_all_space),
-        QUICK_CHECKBOX (3, OPTX, 10, OPTY,
-                        N_("Ignore &space change"), &dview->opt.ignore_space_change),
-        QUICK_CHECKBOX (3, OPTX, 9, OPTY,
-                        N_("Ignore tab &expansion"), &dview->opt.ignore_tab_expansion),
-        QUICK_CHECKBOX (3, OPTX, 8, OPTY,
-                        N_("&Ignore case"), &dview->opt.ignore_case),
-        QUICK_LABEL (3, OPTX, 7, OPTY, N_("Diff extra options")),
-        QUICK_RADIO (3, OPTX, 3, OPTY,
-                     3, (const char **) quality_str, (int *) &dview->opt.quality),
-        QUICK_LABEL (3, OPTX, 2, OPTY, N_("Diff algorithm")),
-
+    quick_widget_t quick_widgets[] = {
+        /* *INDENT-OFF* */
+        QUICK_START_GROUPBOX (N_("Diff algorithm")),
+            QUICK_RADIO (3, (const char **) quality_str, (int *) &dview->opt.quality, NULL),
+        QUICK_STOP_GROUPBOX,
+        QUICK_START_GROUPBOX (N_("Diff extra options")),
+            QUICK_CHECKBOX (N_("&Ignore case"), &dview->opt.ignore_case, NULL),
+            QUICK_CHECKBOX (N_("Ignore tab &expansion"), &dview->opt.ignore_tab_expansion, NULL),
+            QUICK_CHECKBOX (N_("Ignore &space change"), &dview->opt.ignore_space_change, NULL),
+            QUICK_CHECKBOX (N_("Ignore all &whitespace"), &dview->opt.ignore_all_space, NULL),
+            QUICK_CHECKBOX (N_("Strip &trailing carriage return"), &dview->opt.strip_trailing_cr,
+                            NULL),
+        QUICK_STOP_GROUPBOX,
+        QUICK_BUTTONS_OK_CANCEL,
         QUICK_END
+        /* *INDENT-ON* */
     };
 
-    QuickDialog diffopt = {
-        OPTX, OPTY, -1, -1,
+    quick_dialog_t qdlg = {
+        -1, -1, 56,
         N_("Diff Options"), "[Diff Options]",
-        diffopt_widgets, NULL, NULL, FALSE
+        quick_widgets, NULL, NULL
     };
 
-    if (quick_dialog (&diffopt) != B_CANCEL)
+    if (quick_dialog (&qdlg) != B_CANCEL)
         dview_reread (dview);
 }
 
@@ -2434,7 +2442,7 @@ dview_init (WDiff * dview, const char *args, const char *file1, const char *file
     ndiff = redo_diff (dview);
     if (ndiff < 0)
     {
-        /* goto WIDGET_DESTROY stage: dview_fini() */
+        /* goto MSG_DESTROY stage: dview_fini() */
         f_close (f[DIFF_LEFT]);
         f_close (f[DIFF_RIGHT]);
         return -1;
@@ -2856,7 +2864,7 @@ dview_update (WDiff * dview)
 static void
 dview_edit (WDiff * dview, diff_place_t ord)
 {
-    Dlg_head *h;
+    WDialog *h;
     gboolean h_modal;
     int linenum, lineofs;
 
@@ -2866,7 +2874,7 @@ dview_edit (WDiff * dview, diff_place_t ord)
         return;
     }
 
-    h = ((Widget *) dview)->owner;
+    h = WIDGET (dview)->owner;
     h_modal = h->modal;
 
     get_line_numbers (dview->a[ord], dview->skip_rows, &linenum, &lineofs);
@@ -2900,7 +2908,9 @@ dview_goto_cmd (WDiff * dview, diff_place_t ord)
     int newline;
     char *input;
 
-    input = input_dialog (_(title[ord]), _("Enter line:"), MC_HISTORY_YDIFF_GOTO_LINE, prev);
+    input =
+        input_dialog (_(title[ord]), _("Enter line:"), MC_HISTORY_YDIFF_GOTO_LINE, prev,
+                      INPUT_COMPLETE_NONE);
     if (input != NULL)
     {
         const char *s = input;
@@ -2911,10 +2921,10 @@ dview_goto_cmd (WDiff * dview, diff_place_t ord)
 
             if (newline > 0)
             {
-                const DIFFLN *p;
-
                 for (; i < dview->a[ord]->len; i++)
                 {
+                    const DIFFLN *p;
+
                     p = &g_array_index (dview->a[ord], DIFFLN, i);
                     if (p->line == newline)
                         break;
@@ -2932,19 +2942,21 @@ dview_goto_cmd (WDiff * dview, diff_place_t ord)
 static void
 dview_labels (WDiff * dview)
 {
-    Dlg_head *h;
+    Widget *d;
+    WDialog *h;
     WButtonBar *b;
 
-    h = dview->widget.owner;
+    d = WIDGET (dview);
+    h = d->owner;
     b = find_buttonbar (h);
 
-    buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), diff_map, (Widget *) dview);
-    buttonbar_set_label (b, 2, Q_ ("ButtonBar|Save"), diff_map, (Widget *) dview);
-    buttonbar_set_label (b, 4, Q_ ("ButtonBar|Edit"), diff_map, (Widget *) dview);
-    buttonbar_set_label (b, 5, Q_ ("ButtonBar|Merge"), diff_map, (Widget *) dview);
-    buttonbar_set_label (b, 7, Q_ ("ButtonBar|Search"), diff_map, (Widget *) dview);
-    buttonbar_set_label (b, 9, Q_ ("ButtonBar|Options"), diff_map, (Widget *) dview);
-    buttonbar_set_label (b, 10, Q_ ("ButtonBar|Quit"), diff_map, (Widget *) dview);
+    buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), diff_map, d);
+    buttonbar_set_label (b, 2, Q_ ("ButtonBar|Save"), diff_map, d);
+    buttonbar_set_label (b, 4, Q_ ("ButtonBar|Edit"), diff_map, d);
+    buttonbar_set_label (b, 5, Q_ ("ButtonBar|Merge"), diff_map, d);
+    buttonbar_set_label (b, 7, Q_ ("ButtonBar|Search"), diff_map, d);
+    buttonbar_set_label (b, 9, Q_ ("ButtonBar|Options"), diff_map, d);
+    buttonbar_set_label (b, 10, Q_ ("ButtonBar|Quit"), diff_map, d);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -3298,26 +3310,26 @@ dview_handle_key (WDiff * dview, int key)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-dview_callback (Widget * w, widget_msg_t msg, int parm)
+dview_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     WDiff *dview = (WDiff *) w;
-    Dlg_head *h = dview->widget.owner;
+    WDialog *h = w->owner;
     cb_ret_t i;
 
     switch (msg)
     {
-    case WIDGET_INIT:
+    case MSG_INIT:
         dview_labels (dview);
         dview_load_options (dview);
         dview_update (dview);
         return MSG_HANDLED;
 
-    case WIDGET_DRAW:
+    case MSG_DRAW:
         dview->new_frame = 1;
         dview_update (dview);
         return MSG_HANDLED;
 
-    case WIDGET_KEY:
+    case MSG_KEY:
         i = dview_handle_key (dview, parm);
         if (dview->view_quit)
             dlg_stop (h);
@@ -3325,7 +3337,7 @@ dview_callback (Widget * w, widget_msg_t msg, int parm)
             dview_update (dview);
         return i;
 
-    case WIDGET_COMMAND:
+    case MSG_ACTION:
         i = dview_execute_cmd (dview, parm);
         if (dview->view_quit)
             dlg_stop (h);
@@ -3333,20 +3345,20 @@ dview_callback (Widget * w, widget_msg_t msg, int parm)
             dview_update (dview);
         return i;
 
-    case WIDGET_DESTROY:
+    case MSG_DESTROY:
         dview_save_options (dview);
         dview_fini (dview);
         return MSG_HANDLED;
 
     default:
-        return default_proc (msg, parm);
+        return widget_default_callback (w, sender, msg, parm, data);
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-dview_adjust_size (Dlg_head * h)
+dview_adjust_size (WDialog * h)
 {
     WDiff *dview;
     WButtonBar *bar;
@@ -3354,8 +3366,8 @@ dview_adjust_size (Dlg_head * h)
     /* Look up the viewer and the buttonbar, we assume only two widgets here */
     dview = (WDiff *) find_widget_type (h, dview_callback);
     bar = find_buttonbar (h);
-    widget_set_size (&dview->widget, 0, 0, LINES - 1, COLS);
-    widget_set_size ((Widget *) bar, LINES - 1, 0, 1, COLS);
+    widget_set_size (WIDGET (dview), 0, 0, LINES - 1, COLS);
+    widget_set_size (WIDGET (bar), LINES - 1, 0, 1, COLS);
 
     dview_compute_areas (dview);
 }
@@ -3363,32 +3375,33 @@ dview_adjust_size (Dlg_head * h)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-dview_dialog_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *data)
+dview_dialog_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     WDiff *dview = (WDiff *) data;
+    WDialog *h = DIALOG (w);
 
     switch (msg)
     {
-    case DLG_RESIZE:
+    case MSG_RESIZE:
         dview_adjust_size (h);
         return MSG_HANDLED;
 
-    case DLG_ACTION:
+    case MSG_ACTION:
         /* shortcut */
         if (sender == NULL)
             return dview_execute_cmd (NULL, parm);
         /* message from buttonbar */
-        if (sender == (Widget *) find_buttonbar (h))
+        if (sender == WIDGET (find_buttonbar (h)))
         {
             if (data != NULL)
-                return send_message ((Widget *) data, WIDGET_COMMAND, parm);
+                return send_message (data, NULL, MSG_ACTION, parm, NULL);
 
             dview = (WDiff *) find_widget_type (h, dview_callback);
             return dview_execute_cmd (dview, parm);
         }
         return MSG_NOT_HANDLED;
 
-    case DLG_VALIDATE:
+    case MSG_VALIDATE:
         dview = (WDiff *) find_widget_type (h, dview_callback);
         h->state = DLG_ACTIVE;  /* don't stop the dialog before final decision */
         if (dview_ok_to_exit (dview))
@@ -3396,14 +3409,14 @@ dview_dialog_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, v
         return MSG_HANDLED;
 
     default:
-        return default_dlg_callback (h, sender, msg, parm, data);
+        return dlg_default_callback (w, sender, msg, parm, data);
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static char *
-dview_get_title (const Dlg_head * h, size_t len)
+dview_get_title (const WDialog * h, size_t len)
 {
     const WDiff *dview;
     const char *modified = " (*) ";
@@ -3432,19 +3445,18 @@ diff_view (const char *file1, const char *file2, const char *label1, const char
 {
     int error;
     WDiff *dview;
-    Dlg_head *dview_dlg;
+    Widget *w;
+    WDialog *dview_dlg;
 
     /* Create dialog and widgets, put them on the dialog */
     dview_dlg =
-        create_dlg (FALSE, 0, 0, LINES, COLS, NULL, dview_dialog_callback, NULL,
+        dlg_create (FALSE, 0, 0, LINES, COLS, NULL, dview_dialog_callback, NULL,
                     "[Diff Viewer]", NULL, DLG_WANT_TAB);
 
     dview = g_new0 (WDiff, 1);
-
-    init_widget (&dview->widget, 0, 0, LINES - 1, COLS,
-                 (callback_fn) dview_callback, (mouse_h) dview_event);
-
-    widget_want_cursor (dview->widget, 0);
+    w = WIDGET (dview);
+    widget_init (w, 0, 0, LINES - 1, COLS, dview_callback, dview_event);
+    widget_want_cursor (w, FALSE);
 
     add_widget (dview_dlg, dview);
     add_widget (dview_dlg, buttonbar_new (TRUE));
@@ -3458,10 +3470,10 @@ diff_view (const char *file1, const char *file2, const char *label1, const char
      * be aware of it
      */
     if (error == 0)
-        run_dlg (dview_dlg);
+        dlg_run (dview_dlg);
 
     if ((error != 0) || (dview_dlg->state == DLG_CLOSED))
-        destroy_dlg (dview_dlg);
+        dlg_destroy (dview_dlg);
 
     return error == 0 ? 1 : 0;
 }
@@ -3606,21 +3618,11 @@ dview_diff_cmd (const void *f0, const void *f1)
 
             GET_FILE_AND_STAMP (0);
             GET_FILE_AND_STAMP (1);
-            if (real_file0 != NULL && real_file1 != NULL)
-            {
-                char *real_file0_str, *real_file1_str;
-                char *file0_str, *file1_str;
 
-                real_file0_str = vfs_path_to_str (real_file0);
-                real_file1_str = vfs_path_to_str (real_file1);
-                file0_str = vfs_path_to_str (file0);
-                file1_str = vfs_path_to_str (file1);
-                rv = diff_view (real_file0_str, real_file1_str, file0_str, file1_str);
-                g_free (real_file0_str);
-                g_free (real_file1_str);
-                g_free (file0_str);
-                g_free (file1_str);
-            }
+            if (real_file0 != NULL && real_file1 != NULL)
+                rv = diff_view (vfs_path_as_str (real_file0), vfs_path_as_str (real_file1),
+                                vfs_path_as_str (file0), vfs_path_as_str (file1));
+
             UNGET_FILE (1);
             UNGET_FILE (0);
         }
