@@ -62,8 +62,8 @@
    appended after each other and the last one is always the one found
    by book_mark_found() i.e. last in is the one seen */
 
-static struct _book_mark *
-double_marks (WEdit * edit, struct _book_mark *p)
+static edit_book_mark_t *
+double_marks (WEdit * edit, edit_book_mark_t * p)
 {
     (void) edit;
 
@@ -76,15 +76,15 @@ double_marks (WEdit * edit, struct _book_mark *p)
 /* --------------------------------------------------------------------------------------------- */
 /** returns the first bookmark on or before this line */
 
-struct _book_mark *
-book_mark_find (WEdit * edit, int line)
+edit_book_mark_t *
+book_mark_find (WEdit * edit, long line)
 {
-    struct _book_mark *p;
+    edit_book_mark_t *p;
 
     if (edit->book_mark == NULL)
     {
         /* must have an imaginary top bookmark at line -1 to make things less complicated  */
-        edit->book_mark = g_malloc0 (sizeof (struct _book_mark));
+        edit->book_mark = g_new0 (edit_book_mark_t, 1);
         edit->book_mark->line = -1;
         return edit->book_mark;
     }
@@ -139,33 +139,41 @@ book_mark_find (WEdit * edit, int line)
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-/** returns true if a bookmark exists at this line of color c */
+/** 
+ * Check if bookmark bookmark exists at this line of this color
+ *
+ * @param edit editor object
+ * @param line line where book mark is
+ * @param c color of book mark
+ * @return TRUE if bookmark exists at this line of color c, FALSE otherwise
+ */
 
-int
-book_mark_query_color (WEdit * edit, int line, int c)
+gboolean
+book_mark_query_color (WEdit * edit, long line, int c)
 {
-    struct _book_mark *p;
-
-    if (edit->book_mark == NULL)
-        return 0;
-
-    for (p = book_mark_find (edit, line); p != NULL; p = p->prev)
+    if (edit->book_mark != NULL)
     {
-        if (p->line != line)
-            return 0;
-        if (p->c == c)
-            return 1;
+        edit_book_mark_t *p;
+
+        for (p = book_mark_find (edit, line); p != NULL; p = p->prev)
+        {
+            if (p->line != line)
+                return FALSE;
+            if (p->c == c)
+                return TRUE;
+        }
     }
-    return 0;
+
+    return FALSE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 /** insert a bookmark at this line */
 
 void
-book_mark_insert (WEdit * edit, size_t line, int c)
+book_mark_insert (WEdit * edit, long line, int c)
 {
-    struct _book_mark *p, *q;
+    edit_book_mark_t *p, *q;
 
     p = book_mark_find (edit, line);
 #if 0
@@ -180,10 +188,9 @@ book_mark_insert (WEdit * edit, size_t line, int c)
         return;
     }
 #endif
-    edit->force |= REDRAW_LINE;
     /* create list entry */
-    q = g_malloc0 (sizeof (struct _book_mark));
-    q->line = (int) line;
+    q = g_new (edit_book_mark_t, 1);
+    q->line = line;
     q->c = c;
     q->next = p->next;
     /* insert into list */
@@ -191,18 +198,25 @@ book_mark_insert (WEdit * edit, size_t line, int c)
     if (p->next != NULL)
         p->next->prev = q;
     p->next = q;
+
+    edit->force |= REDRAW_LINE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
-/** remove a bookmark if there is one at this line matching this color - c of -1 clear all
- * @returns non-zero on not-found
+/** 
+ * Remove a bookmark if there is one at this line matching this color - c of -1 clear all
+ *
+ * @param edit editor object
+ * @param line line where book mark is
+ * @param c color of book mark or -1 to clear all book marks on this line
+ * @return FALSE if not found, TRUE otherwise
  */
 
-int
-book_mark_clear (WEdit * edit, int line, int c)
+gboolean
+book_mark_clear (WEdit * edit, long line, int c)
 {
-    struct _book_mark *p, *q;
-    int r = 1;
+    edit_book_mark_t *p, *q;
+    gboolean r = FALSE;
 
     if (edit->book_mark == NULL)
         return r;
@@ -212,7 +226,7 @@ book_mark_clear (WEdit * edit, int line, int c)
         q = p->prev;
         if (p->line == line && (p->c == c || c == -1))
         {
-            r = 0;
+            r = TRUE;
             edit->book_mark = p->prev;
             p->prev->next = p->next;
             if (p->next != NULL)
@@ -223,7 +237,7 @@ book_mark_clear (WEdit * edit, int line, int c)
         }
     }
     /* if there is only our dummy book mark left, clear it for speed */
-    if (edit->book_mark->line == -1 && !edit->book_mark->next)
+    if (edit->book_mark->line == -1 && edit->book_mark->next == NULL)
     {
         g_free (edit->book_mark);
         edit->book_mark = NULL;
@@ -237,7 +251,7 @@ book_mark_clear (WEdit * edit, int line, int c)
 void
 book_mark_flush (WEdit * edit, int c)
 {
-    struct _book_mark *p, *q;
+    edit_book_mark_t *p, *q;
 
     if (edit->book_mark == NULL)
         return;
@@ -269,11 +283,12 @@ book_mark_flush (WEdit * edit, int c)
 /** shift down bookmarks after this line */
 
 void
-book_mark_inc (WEdit * edit, int line)
+book_mark_inc (WEdit * edit, long line)
 {
-    if (edit->book_mark)
+    if (edit->book_mark != NULL)
     {
-        struct _book_mark *p;
+        edit_book_mark_t *p;
+
         p = book_mark_find (edit, line);
         for (p = p->next; p != NULL; p = p->next)
             p->line++;
@@ -284,11 +299,12 @@ book_mark_inc (WEdit * edit, int line)
 /** shift up bookmarks after this line */
 
 void
-book_mark_dec (WEdit * edit, int line)
+book_mark_dec (WEdit * edit, long line)
 {
     if (edit->book_mark != NULL)
     {
-        struct _book_mark *p;
+        edit_book_mark_t *p;
+
         p = book_mark_find (edit, line);
         for (p = p->next; p != NULL; p = p->next)
             p->line--;
@@ -301,13 +317,13 @@ book_mark_dec (WEdit * edit, int line)
 void
 book_mark_serialize (WEdit * edit, int color)
 {
-    struct _book_mark *p;
-
     if (edit->serialized_bookmarks != NULL)
         g_array_set_size (edit->serialized_bookmarks, 0);
 
     if (edit->book_mark != NULL)
     {
+        edit_book_mark_t *p;
+
         if (edit->serialized_bookmarks == NULL)
             edit->serialized_bookmarks = g_array_sized_new (FALSE, FALSE, sizeof (size_t),
                                                             MAX_SAVED_BOOKMARKS);
