@@ -41,7 +41,7 @@
 #include "lib/widget.h"         /* Listbox */
 
 #include "edit-impl.h"
-#include "edit-widget.h"
+#include "editwidget.h"
 
 /*** global variables ****************************************************************************/
 
