@@ -584,8 +584,6 @@ edit_buffer_backspace (edit_buffer_t * buf)
 off_t
 edit_buffer_move_forward (const edit_buffer_t * buf, off_t current, long lines, off_t upto)
 {
-    long next;
-
     if (upto != 0)
         return (off_t) edit_buffer_count_lines (buf, current, upto);
 
@@ -593,6 +591,8 @@ edit_buffer_move_forward (const edit_buffer_t * buf, off_t current, long lines,
 
     while (lines-- != 0)
     {
+        long next;
+
         next = edit_buffer_get_eol (buf, current) + 1;
         if (next > buf->size)
             break;
@@ -640,10 +640,11 @@ off_t
 edit_buffer_read_file (edit_buffer_t * buf, int fd, off_t size)
 {
     off_t ret = 0;
-    off_t i;
+    off_t i, j;
     off_t data_size;
     void *b;
 
+    buf->lines = 0;
     buf->curs2 = size;
     i = buf->curs2 >> S_EDIT_BUF_SIZE;
 
@@ -653,13 +654,19 @@ edit_buffer_read_file (edit_buffer_t * buf, int fd, off_t size)
     {
         b = g_malloc0 (EDIT_BUF_SIZE);
         g_ptr_array_add (buf->b2, b);
-        ret = mc_read (fd, (char *) b + EDIT_BUF_SIZE - data_size, data_size);
+        b = (char *) b + EDIT_BUF_SIZE - data_size;
+        ret = mc_read (fd, b, data_size);
+
+        /* count lines */
+        for (j = 0; j < ret; j++)
+            if (*((char *) b + j) == '\n')
+                buf->lines++;
+
         if (ret < 0 || ret != data_size)
             return ret;
-
     }
 
-    /* fullfill other parts of b2 from end to begin */
+    /* fulfill other parts of b2 from end to begin */
     data_size = EDIT_BUF_SIZE;
     for (--i; i >= 0; i--)
     {
@@ -670,6 +677,12 @@ edit_buffer_read_file (edit_buffer_t * buf, int fd, off_t size)
         sz = mc_read (fd, b, data_size);
         if (sz >= 0)
             ret += sz;
+
+        /* count lines */
+        for (j = 0; j < sz; j++)
+            if (*((char *) b + j) == '\n')
+                buf->lines++;
+
         if (sz != data_size)
             break;
     }
@@ -708,7 +721,7 @@ edit_buffer_write_file (edit_buffer_t * buf, int fd)
     off_t data_size, sz;
     void *b;
 
-    /* write all fullfilled parts of b1 from begin to end */
+    /* write all fulfilled parts of b1 from begin to end */
     if (buf->b1->len != 0)
     {
         data_size = EDIT_BUF_SIZE;
@@ -747,7 +760,7 @@ edit_buffer_write_file (edit_buffer_t * buf, int fd)
 
         if (sz == data_size)
         {
-            /* write other fullfilled parts of b2 from end to begin */
+            /* write other fulfilled parts of b2 from end to begin */
             data_size = EDIT_BUF_SIZE;
             while (--i >= 0)
             {
