@@ -1,9 +1,12 @@
 /*
    Execution routines for GNU Midnight Commander
 
-   Copyright (C) 2003, 2004, 2005, 2007, 2011
+   Copyright (C) 2003, 2004, 2005, 2007, 2011, 2013
    The Free Software Foundation, Inc.
 
+   Written by:
+   Slava Zanko <slavazanko@gmail.com>, 2013
+
    This file is part of the Midnight Commander.
 
    The Midnight Commander is free software: you can redistribute it
@@ -27,6 +30,7 @@
 #include <config.h>
 
 #include <signal.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 
@@ -38,12 +42,15 @@
 #include "lib/vfs/vfs.h"
 #include "lib/mcconfig.h"
 #include "lib/util.h"
+#include "lib/strutil.h"        /* str_replace_all_substrings() */
 #include "lib/widget.h"
 
 #include "filemanager/midnight.h"
 #include "filemanager/layout.h" /* use_dash() */
 #include "consaver/cons.saver.h"
+#ifdef ENABLE_SUBSHELL
 #include "subshell.h"
+#endif
 #include "setup.h"              /* clear_before_exec */
 
 #include "execute.h"
@@ -59,6 +66,13 @@ int pause_after_run = pause_on_dumb_terminals;
 /*** file scope variables ************************************************************************/
 
 /*** file scope functions ************************************************************************/
+
+void do_execute (const char *shell, const char *command, int flags);
+void do_executev (const char *shell, int flags, char *const argv[]);
+char *execute_get_external_cmd_opts_from_config (const char *command,
+                                                 const vfs_path_t * filename_vpath,
+                                                 long start_line);
+
 /* --------------------------------------------------------------------------------------------- */
 
 static void
@@ -74,6 +88,7 @@ edition_post_exec (void)
     tty_raw_mode ();
     channels_up ();
     enable_mouse ();
+    enable_bracketed_paste ();
     if (mc_global.tty.alternate_plus_minus)
         application_keypad_mode ();
 }
@@ -93,6 +108,7 @@ edition_pre_exec (void)
 
     channels_down ();
     disable_mouse ();
+    disable_bracketed_paste ();
 
     tty_reset_shell_mode ();
     tty_keypad (FALSE);
@@ -111,7 +127,7 @@ edition_pre_exec (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
 static void
 do_possible_cd (const vfs_path_t * new_dir_vpath)
 {
@@ -122,100 +138,7 @@ do_possible_cd (const vfs_path_t * new_dir_vpath)
                    "deleted your working directory, or given yourself\n"
                    "extra access permissions with the \"su\" command?"));
 }
-#endif /* HAVE_SUBSHELL_SUPPORT */
-
-/* --------------------------------------------------------------------------------------------- */
-
-static void
-do_execute (const char *lc_shell, const char *command, int flags)
-{
-#ifdef HAVE_SUBSHELL_SUPPORT
-    vfs_path_t *new_dir_vpath = NULL;
-#endif /* HAVE_SUBSHELL_SUPPORT */
-
-    vfs_path_t *old_vfs_dir_vpath = NULL;
-
-    if (!vfs_current_is_local ())
-        old_vfs_dir_vpath = vfs_path_clone (vfs_get_raw_current_dir ());
-
-    if (mc_global.mc_run_mode == MC_RUN_FULL)
-        save_cwds_stat ();
-    pre_exec ();
-    if (mc_global.tty.console_flag != '\0')
-        handle_console (CONSOLE_RESTORE);
-
-    if (!mc_global.tty.use_subshell && command && !(flags & EXECUTE_INTERNAL))
-    {
-        printf ("%s%s\n", mc_prompt, command);
-        fflush (stdout);
-    }
-#ifdef HAVE_SUBSHELL_SUPPORT
-    if (mc_global.tty.use_subshell && !(flags & EXECUTE_INTERNAL))
-    {
-        do_update_prompt ();
-
-        /* We don't care if it died, higher level takes care of this */
-        invoke_subshell (command, VISIBLY, old_vfs_dir_vpath != NULL ? NULL : &new_dir_vpath);
-    }
-    else
-#endif /* HAVE_SUBSHELL_SUPPORT */
-        my_system (flags, lc_shell, command);
-
-    if (!(flags & EXECUTE_INTERNAL))
-    {
-        if ((pause_after_run == pause_always
-             || (pause_after_run == pause_on_dumb_terminals && !mc_global.tty.xterm_flag
-                 && mc_global.tty.console_flag == '\0')) && quit == 0
-#ifdef HAVE_SUBSHELL_SUPPORT
-            && subshell_state != RUNNING_COMMAND
-#endif /* HAVE_SUBSHELL_SUPPORT */
-            )
-        {
-            printf (_("Press any key to continue..."));
-            fflush (stdout);
-            tty_raw_mode ();
-            get_key_code (0);
-            printf ("\r\n");
-            fflush (stdout);
-        }
-        if (mc_global.tty.console_flag != '\0')
-        {
-            if (output_lines && mc_global.keybar_visible)
-            {
-                putchar ('\n');
-                fflush (stdout);
-            }
-        }
-    }
-
-    if (mc_global.tty.console_flag != '\0')
-        handle_console (CONSOLE_SAVE);
-    edition_post_exec ();
-
-#ifdef HAVE_SUBSHELL_SUPPORT
-    if (new_dir_vpath != NULL)
-    {
-        do_possible_cd (new_dir_vpath);
-        vfs_path_free (new_dir_vpath);
-    }
-
-#endif /* HAVE_SUBSHELL_SUPPORT */
-
-    if (old_vfs_dir_vpath != NULL)
-    {
-        mc_chdir (old_vfs_dir_vpath);
-        vfs_path_free (old_vfs_dir_vpath);
-    }
-
-    if (mc_global.mc_run_mode == MC_RUN_FULL)
-    {
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-        update_xterm_title_path ();
-    }
-
-    do_refresh ();
-    use_dash (TRUE);
-}
+#endif /* ENABLE_SUBSHELL */
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -231,6 +154,7 @@ do_suspend_cmd (void)
     {
         struct sigaction sigtstp_action;
 
+        memset (&sigtstp_action, 0, sizeof (sigtstp_action));
         /* Make sure that the SIGTSTP below will suspend us directly,
            without calling ncurses' SIGTSTP handler; we *don't* want
            ncurses to redraw the screen immediately after the SIGCONT */
@@ -249,10 +173,223 @@ do_suspend_cmd (void)
     edition_post_exec ();
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+static gboolean
+execute_prepare_with_vfs_arg (const vfs_path_t * filename_vpath, vfs_path_t ** localcopy_vpath,
+                              time_t * mtime)
+{
+    struct stat st;
+
+    /* Simplest case, this file is local */
+    if ((filename_vpath == NULL && vfs_file_is_local (vfs_get_raw_current_dir ()))
+        || vfs_file_is_local (filename_vpath))
+        return TRUE;
+
+    /* FIXME: Creation of new files on VFS is not supported */
+    if (filename_vpath == NULL)
+        return FALSE;
+
+    *localcopy_vpath = mc_getlocalcopy (filename_vpath);
+    if (*localcopy_vpath == NULL)
+    {
+        message (D_ERROR, MSG_ERROR, _("Cannot fetch a local copy of %s"),
+                 vfs_path_as_str (filename_vpath));
+        return FALSE;
+    }
+
+    mc_stat (*localcopy_vpath, &st);
+    *mtime = st.st_mtime;
+    return TRUE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+execute_cleanup_with_vfs_arg (const vfs_path_t * filename_vpath, vfs_path_t ** localcopy_vpath,
+                              time_t * mtime)
+{
+    if (*localcopy_vpath != NULL)
+    {
+        struct stat st;
+
+        /*
+         * filename can be an entry on panel, it can be changed by executing
+         * the command, so make a copy.  Smarter VFS code would make the code
+         * below unnecessary.
+         */
+        mc_stat (*localcopy_vpath, &st);
+        mc_ungetlocalcopy (filename_vpath, *localcopy_vpath, *mtime != st.st_mtime);
+        vfs_path_free (*localcopy_vpath);
+        *localcopy_vpath = NULL;
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static char *
+execute_get_opts_from_cfg (const char *command, const char *default_str)
+{
+    char *str_from_config;
+
+    str_from_config =
+        mc_config_get_string_raw (mc_main_config, CONFIG_EXT_EDITOR_VIEWER_SECTION, command, NULL);
+
+    if (str_from_config == NULL)
+    {
+        mc_config_t *cfg;
+
+        cfg = mc_config_init (global_profile_name, TRUE);
+        if (cfg == NULL)
+            return g_strdup (default_str);
+
+        str_from_config =
+            mc_config_get_string_raw (cfg, CONFIG_EXT_EDITOR_VIEWER_SECTION, command, default_str);
+
+        mc_config_deinit (cfg);
+    }
+
+    return str_from_config;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
+char *
+execute_get_external_cmd_opts_from_config (const char *command, const vfs_path_t * filename_vpath,
+                                           long start_line)
+{
+    char *str_from_config, *return_str;
+    char *parameter;
+
+    if (filename_vpath == NULL)
+        return g_strdup ("");
+
+    str_from_config = execute_get_opts_from_cfg (command, "%filename");
+
+    parameter = g_shell_quote (vfs_path_get_last_path_str (filename_vpath));
+    return_str = str_replace_all (str_from_config, "%filename", parameter);
+    g_free (parameter);
+    g_free (str_from_config);
+    str_from_config = return_str;
+
+    parameter = g_strdup_printf ("%ld", start_line);
+    return_str = str_replace_all (str_from_config, "%lineno", parameter);
+    g_free (parameter);
+    g_free (str_from_config);
+
+    return return_str;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+void
+do_executev (const char *shell, int flags, char *const argv[])
+{
+#ifdef ENABLE_SUBSHELL
+    vfs_path_t *new_dir_vpath = NULL;
+#endif /* ENABLE_SUBSHELL */
+
+    vfs_path_t *old_vfs_dir_vpath = NULL;
+
+    if (!vfs_current_is_local ())
+        old_vfs_dir_vpath = vfs_path_clone (vfs_get_raw_current_dir ());
+
+    if (mc_global.mc_run_mode == MC_RUN_FULL)
+        save_cwds_stat ();
+    pre_exec ();
+    if (mc_global.tty.console_flag != '\0')
+        handle_console (CONSOLE_RESTORE);
+
+    if (!mc_global.tty.use_subshell && *argv != NULL && (flags & EXECUTE_INTERNAL) == 0)
+    {
+        printf ("%s%s\n", mc_prompt, *argv);
+        fflush (stdout);
+    }
+#ifdef ENABLE_SUBSHELL
+    if (mc_global.tty.use_subshell && (flags & EXECUTE_INTERNAL) == 0)
+    {
+        do_update_prompt ();
+
+        /* We don't care if it died, higher level takes care of this */
+        invoke_subshell (*argv, VISIBLY, old_vfs_dir_vpath != NULL ? NULL : &new_dir_vpath);
+    }
+    else
+#endif /* ENABLE_SUBSHELL */
+        my_systemv_flags (flags, shell, argv);
+
+    if ((flags & EXECUTE_INTERNAL) == 0)
+    {
+        if ((pause_after_run == pause_always
+             || (pause_after_run == pause_on_dumb_terminals && !mc_global.tty.xterm_flag
+                 && mc_global.tty.console_flag == '\0')) && quit == 0
+#ifdef ENABLE_SUBSHELL
+            && subshell_state != RUNNING_COMMAND
+#endif /* ENABLE_SUBSHELL */
+            )
+        {
+            printf (_("Press any key to continue..."));
+            fflush (stdout);
+            tty_raw_mode ();
+            get_key_code (0);
+            printf ("\r\n");
+            fflush (stdout);
+        }
+        if (mc_global.tty.console_flag != '\0' && output_lines != 0 && mc_global.keybar_visible)
+        {
+            putchar ('\n');
+            fflush (stdout);
+        }
+    }
+
+    if (mc_global.tty.console_flag != '\0')
+        handle_console (CONSOLE_SAVE);
+    edition_post_exec ();
+
+#ifdef ENABLE_SUBSHELL
+    if (new_dir_vpath != NULL)
+    {
+        do_possible_cd (new_dir_vpath);
+        vfs_path_free (new_dir_vpath);
+    }
+
+#endif /* ENABLE_SUBSHELL */
+
+    if (old_vfs_dir_vpath != NULL)
+    {
+        mc_chdir (old_vfs_dir_vpath);
+        vfs_path_free (old_vfs_dir_vpath);
+    }
+
+    if (mc_global.mc_run_mode == MC_RUN_FULL)
+    {
+        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
+        update_xterm_title_path ();
+    }
+
+    do_refresh ();
+    use_dash (TRUE);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+void
+do_execute (const char *shell, const char *command, int flags)
+{
+    GPtrArray *args_array;
+
+    args_array = g_ptr_array_new ();
+    g_ptr_array_add (args_array, (char *) command);
+    g_ptr_array_add (args_array, NULL);
+
+    do_executev (shell, flags, (char *const *) args_array->pdata);
+
+    g_ptr_array_free (args_array, TRUE);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 /** Set up the terminal before executing a program */
 
 void
@@ -286,15 +423,15 @@ shell_execute (const char *command, int flags)
         flags ^= EXECUTE_HIDE;
     }
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
     if (mc_global.tty.use_subshell)
         if (subshell_state == INACTIVE)
-            do_execute (shell, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
+            do_execute (mc_global.tty.shell, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
         else
             message (D_ERROR, MSG_ERROR, _("The shell is already running a command"));
     else
-#endif /* HAVE_SUBSHELL_SUPPORT */
-        do_execute (shell, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
+#endif /* ENABLE_SUBSHELL */
+        do_execute (mc_global.tty.shell, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
 
     g_free (cmd);
 }
@@ -304,7 +441,7 @@ shell_execute (const char *command, int flags)
 void
 exec_shell (void)
 {
-    do_execute (shell, 0, 0);
+    do_execute (mc_global.tty.shell, 0, 0);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -312,13 +449,15 @@ exec_shell (void)
 void
 toggle_panels (void)
 {
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
     vfs_path_t *new_dir_vpath = NULL;
-    vfs_path_t **new_dir_p;
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
+
+    SIG_ATOMIC_VOLATILE_T was_sigwinch = 0;
 
     channels_down ();
     disable_mouse ();
+    disable_bracketed_paste ();
     if (clear_before_exec)
         clr_scr ();
     if (mc_global.tty.alternate_plus_minus)
@@ -337,21 +476,23 @@ toggle_panels (void)
     if (mc_global.tty.console_flag != '\0')
         handle_console (CONSOLE_RESTORE);
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
     if (mc_global.tty.use_subshell)
     {
+        vfs_path_t **new_dir_p;
+
         new_dir_p = vfs_current_is_local ()? &new_dir_vpath : NULL;
         invoke_subshell (NULL, VISIBLY, new_dir_p);
     }
     else
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
     {
         if (output_starts_shell)
         {
-            fprintf (stderr, _("Type `exit' to return to the Midnight Commander"));
+            fprintf (stderr, _("Type 'exit' to return to the Midnight Commander"));
             fprintf (stderr, "\n\r\n\r");
 
-            my_system (EXECUTE_INTERNAL, shell, NULL);
+            my_system (EXECUTE_INTERNAL, mc_global.tty.shell, NULL);
         }
         else
             get_key_code (0);
@@ -369,24 +510,32 @@ toggle_panels (void)
        subshell */
     if ((quit & SUBSHELL_EXIT) != 0)
     {
-        /* User did `exit' or `logout': quit MC */
+        /* User did 'exit' or 'logout': quit MC */
         if (quiet_quit_cmd ())
             return;
 
         quit = 0;
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
         /* restart subshell */
         if (mc_global.tty.use_subshell)
             init_subshell ();
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
     }
 
     enable_mouse ();
+    enable_bracketed_paste ();
     channels_up ();
     if (mc_global.tty.alternate_plus_minus)
         application_keypad_mode ();
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+    /* HACK:
+     * Save sigwinch flag that will be reset in mc_refresh() called via update_panels().
+     * There is some problem with screen redraw in ncurses-based mc in this situation.
+     */
+    was_sigwinch = mc_global.tty.winch_flag;
+    mc_global.tty.winch_flag = 0;
+
+#ifdef ENABLE_SUBSHELL
     if (mc_global.tty.use_subshell)
     {
         do_load_prompt ();
@@ -399,14 +548,18 @@ toggle_panels (void)
     }
 
     vfs_path_free (new_dir_vpath);
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
 
     if (mc_global.mc_run_mode == MC_RUN_FULL)
     {
         update_panels (UP_OPTIMIZE, UP_KEEPSEL);
         update_xterm_title_path ();
     }
-    repaint_screen ();
+
+    if (was_sigwinch != 0 || mc_global.tty.winch_flag != 0)
+        dialog_change_screen_size ();
+    else
+        repaint_screen ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -432,6 +585,7 @@ execute_suspend (const gchar * event_group_name, const gchar * event_name,
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
 /**
  * Execute command on a filename that can be on VFS.
  * Errors are reported to the user.
@@ -440,43 +594,64 @@ execute_suspend (const gchar * event_group_name, const gchar * event_name,
 void
 execute_with_vfs_arg (const char *command, const vfs_path_t * filename_vpath)
 {
-    struct stat st;
+    vfs_path_t *localcopy_vpath = NULL;
+    const vfs_path_t *do_execute_vpath;
     time_t mtime;
-    vfs_path_t *localcopy_vpath;
 
-    /* Simplest case, this file is local */
-    if (filename_vpath == NULL || vfs_file_is_local (filename_vpath))
-    {
-        do_execute (command, vfs_path_get_last_path_str (filename_vpath), EXECUTE_INTERNAL);
-        return;
-    }
-
-    /* FIXME: Creation of new files on VFS is not supported */
-    if (vfs_path_len (filename_vpath) == 0)
+    if (!execute_prepare_with_vfs_arg (filename_vpath, &localcopy_vpath, &mtime))
         return;
 
-    localcopy_vpath = mc_getlocalcopy (filename_vpath);
-    if (localcopy_vpath == NULL)
-    {
-        char *filename;
+    do_execute_vpath = (localcopy_vpath == NULL) ? filename_vpath : localcopy_vpath;
 
-        filename = vfs_path_to_str (filename_vpath);
-        message (D_ERROR, MSG_ERROR, _("Cannot fetch a local copy of %s"), filename);
-        g_free (filename);
-        return;
-    }
+    do_execute (command, vfs_path_get_last_path_str (do_execute_vpath), EXECUTE_INTERNAL);
 
-    /*
-     * filename can be an entry on panel, it can be changed by executing
-     * the command, so make a copy.  Smarter VFS code would make the code
-     * below unnecessary.
-     */
-    mc_stat (localcopy_vpath, &st);
-    mtime = st.st_mtime;
-    do_execute (command, vfs_path_get_last_path_str (localcopy_vpath), EXECUTE_INTERNAL);
-    mc_stat (localcopy_vpath, &st);
-    mc_ungetlocalcopy (filename_vpath, localcopy_vpath, mtime != st.st_mtime);
-    vfs_path_free (localcopy_vpath);
+    execute_cleanup_with_vfs_arg (filename_vpath, &localcopy_vpath, &mtime);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Execute external editor or viewer.
+ *
+ * @param command editor/viewer to run
+ * @param filename_vpath path for edit/view
+ * @param start_line cursor will be placed at the 'start_line' position after opening file
+ */
+
+void
+execute_external_editor_or_viewer (const char *command, const vfs_path_t * filename_vpath,
+                                   long start_line)
+{
+    vfs_path_t *localcopy_vpath = NULL;
+    const vfs_path_t *do_execute_vpath;
+    char *extern_cmd_options;
+    time_t mtime;
+
+    if (!execute_prepare_with_vfs_arg (filename_vpath, &localcopy_vpath, &mtime))
+        return;
+
+    do_execute_vpath = (localcopy_vpath == NULL) ? filename_vpath : localcopy_vpath;
+
+    extern_cmd_options =
+        execute_get_external_cmd_opts_from_config (command, do_execute_vpath, start_line);
+
+    if (extern_cmd_options != NULL)
+    {
+        char **argv_cmd_options;
+        int argv_count;
+
+        if (g_shell_parse_argv (extern_cmd_options, &argv_count, &argv_cmd_options, NULL))
+        {
+            do_executev (command, EXECUTE_INTERNAL, argv_cmd_options);
+            g_strfreev (argv_cmd_options);
+        }
+        else
+            do_executev (command, EXECUTE_INTERNAL, NULL);
+
+        g_free (extern_cmd_options);
+
+    }
+
+    execute_cleanup_with_vfs_arg (filename_vpath, &localcopy_vpath, &mtime);
 }
 
 /* --------------------------------------------------------------------------------------------- */
