@@ -2,7 +2,7 @@
    Chown-advanced command -- for the Midnight Commander
 
    Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2007, 2011
+   2005, 2007, 2011, 2012, 2013
    The Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -58,16 +58,13 @@
 /*** file scope macro definitions ****************************************************************/
 
 #define BX              5
-#define BY              6
-
-#define TX              50
-#define TY              2
+#define BY              5
 
 #define BUTTONS         9
+#define BUTTONS_PERM    5
 
 #define B_SETALL        B_USER
 #define B_SKIP          (B_USER + 1)
-
 #define B_OWN           (B_USER + 3)
 #define B_GRP           (B_USER + 4)
 #define B_OTH           (B_USER + 5)
@@ -78,43 +75,47 @@
 
 /*** file scope variables ************************************************************************/
 
-static struct Dlg_head *ch_dlg;
+static struct WDialog *ch_dlg;
 
 static struct
 {
-    int ret_cmd, flags, y, x;
+    unsigned long id;
+    int ret_cmd, flags, x, len;
     const char *text;
 } chown_advanced_but[BUTTONS] =
 {
     /* *INDENT-OFF* */
-    { B_CANCEL, NORMAL_BUTTON, 4, 53, N_("&Cancel") },
-    { B_ENTER,  DEFPUSH_BUTTON,4, 40, N_("&Set") },
-    { B_SKIP,   NORMAL_BUTTON, 4, 23, N_("S&kip") },
-    { B_SETALL, NORMAL_BUTTON, 4,  0, N_("Set &all")},
-    { B_ENTER,  NARROW_BUTTON, 0, 47, ""},
-    { B_ENTER,  NARROW_BUTTON, 0, 29, ""},
-    { B_ENTER,  NARROW_BUTTON, 0, 19, "   "},
-    { B_ENTER,  NARROW_BUTTON, 0, 11, "   "},
-    { B_ENTER,  NARROW_BUTTON, 0,  3, "   "}
+    { 0, B_ENTER,  NARROW_BUTTON,   3, 0, "   "},
+    { 0, B_ENTER,  NARROW_BUTTON,  11, 0, "   "},
+    { 0, B_ENTER,  NARROW_BUTTON,  19, 0, "   "},
+    { 0, B_ENTER,  NARROW_BUTTON,  29, 0, ""},
+    { 0, B_ENTER,  NARROW_BUTTON,  47, 0, ""},
+
+    { 0, B_SETALL, NORMAL_BUTTON,   0, 0, N_("Set &all")},
+    { 0, B_SKIP,   NORMAL_BUTTON,   0, 0, N_("S&kip") },
+    { 0, B_ENTER,  DEFPUSH_BUTTON,  0, 0, N_("&Set") },
+    { 0, B_CANCEL, NORMAL_BUTTON,   0, 0, N_("&Cancel") }
     /* *INDENT-ON* */
 };
 
 static WButton *b_att[3];       /* permission */
 static WButton *b_user, *b_group;       /* owner */
+static WLabel *l_filename;
+static WLabel *l_mode;
 
-static int files_on_begin;      /* Number of files at startup */
 static int flag_pos;
 static int x_toggle;
 static char ch_flags[11];
 static const char ch_perm[] = "rwx";
 static mode_t ch_cmode;
 static struct stat *sf_stat;
-static int need_update;
-static int end_chown;
+static gboolean need_update = FALSE;
+static gboolean end_chown = FALSE;
 static int current_file;
-static int single_set;
+static gboolean single_set = FALSE;
 static char *fname;
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -136,7 +137,7 @@ inc_flag_pos (int f_pos)
         return MSG_NOT_HANDLED;
     }
     flag_pos++;
-    if (!(flag_pos % 3) || f_pos > 2)
+    if ((flag_pos % 3) == 0 || f_pos > 2)
         return MSG_NOT_HANDLED;
     return MSG_HANDLED;
 }
@@ -146,13 +147,13 @@ inc_flag_pos (int f_pos)
 static cb_ret_t
 dec_flag_pos (int f_pos)
 {
-    if (!flag_pos)
+    if (flag_pos == 0)
     {
         flag_pos = 10;
         return MSG_NOT_HANDLED;
     }
     flag_pos--;
-    if (!((flag_pos + 1) % 3) || f_pos > 2)
+    if (((flag_pos + 1) % 3) == 0 || f_pos > 2)
         return MSG_NOT_HANDLED;
     return MSG_HANDLED;
 }
@@ -230,19 +231,19 @@ print_flags (void)
 
     for (i = 0; i < 3; i++)
     {
-        dlg_move (ch_dlg, BY + 1, 9 + i);
+        widget_move (ch_dlg, BY + 1, 9 + i);
         tty_print_char (ch_flags[i]);
     }
 
     for (i = 0; i < 3; i++)
     {
-        dlg_move (ch_dlg, BY + 1, 17 + i);
+        widget_move (ch_dlg, BY + 1, 17 + i);
         tty_print_char (ch_flags[i + 3]);
     }
 
     for (i = 0; i < 3; i++)
     {
-        dlg_move (ch_dlg, BY + 1, 25 + i);
+        widget_move (ch_dlg, BY + 1, 25 + i);
         tty_print_char (ch_flags[i + 6]);
     }
 
@@ -250,12 +251,12 @@ print_flags (void)
 
     for (i = 0; i < 15; i++)
     {
-        dlg_move (ch_dlg, BY + 1, 35 + i);
+        widget_move (ch_dlg, BY + 1, 35 + i);
         tty_print_char (ch_flags[9]);
     }
     for (i = 0; i < 15; i++)
     {
-        dlg_move (ch_dlg, BY + 1, 53 + i);
+        widget_move (ch_dlg, BY + 1, 53 + i);
         tty_print_char (ch_flags[10]);
     }
 }
@@ -263,76 +264,91 @@ print_flags (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-update_mode (Dlg_head * h)
+chown_info_update (void)
+{
+    char buffer[BUF_SMALL];
+
+    /* mode */
+    g_snprintf (buffer, sizeof (buffer), "Permissions (octal): %o", get_mode ());
+    label_set_text (l_mode, buffer);
+
+    /* permissions */
+    update_permissions ();
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+update_mode (WDialog * h)
 {
     print_flags ();
-    tty_setcolor (COLOR_NORMAL);
-    dlg_move (h, BY + 2, 9);
-    tty_printf ("%12o", get_mode ());
-    send_message ((Widget *) h->current->data, WIDGET_FOCUS, 0);
+    chown_info_update ();
+    send_message (h->current->data, NULL, MSG_FOCUS, 0, NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-chl_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *data)
+chl_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     switch (msg)
     {
-    case DLG_KEY:
+    case MSG_KEY:
         switch (parm)
         {
         case KEY_LEFT:
         case KEY_RIGHT:
-            h->ret_value = parm;
-            dlg_stop (h);
+            {
+                WDialog *h = DIALOG (w);
+
+                h->ret_value = parm;
+                dlg_stop (h);
+            }
         }
 
     default:
-        return default_dlg_callback (h, sender, msg, parm, data);
+        return dlg_default_callback (w, sender, msg, parm, data);
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-do_enter_key (Dlg_head * h, int f_pos)
+do_enter_key (WDialog * h, int f_pos)
 {
-    Dlg_head *chl_dlg;
     WListbox *chl_list;
     struct passwd *chl_pass;
     struct group *chl_grp;
     int fe;
-    int lxx, lyy, chl_end, b_pos;
-    int is_owner;
-    const char *title;
+    gboolean chl_end, is_owner;
 
     do
     {
+        int result;
+        WDialog *chl_dlg;
+        const char *title;
+        int lxx, lyy, b_pos;
+
         is_owner = (f_pos == 3);
         title = is_owner ? _("owner") : _("group");
 
         lxx = (COLS - 74) / 2 + (is_owner ? 35 : 53);
         lyy = (LINES - 13) / 2;
-        chl_end = 0;
+        chl_end = FALSE;
 
         chl_dlg =
-            create_dlg (TRUE, lyy, lxx, 13, 17, dialog_colors, chl_callback, NULL,
-                        "[Advanced Chown]", title, DLG_COMPACT | DLG_REVERSE);
+            dlg_create (TRUE, lyy, lxx, 13, 17, dialog_colors, chl_callback, NULL,
+                        "[Advanced Chown]", title, DLG_COMPACT);
 
         /* get new listboxes */
         chl_list = listbox_new (1, 1, 11, 15, FALSE, NULL);
-
         listbox_add_item (chl_list, LISTBOX_APPEND_AT_END, 0, "<Unknown>", NULL);
-
         if (is_owner)
         {
             /* get and put user names in the listbox */
             setpwent ();
-            while ((chl_pass = getpwent ()))
-            {
+            while ((chl_pass = getpwent ()) != NULL)
                 listbox_add_item (chl_list, LISTBOX_APPEND_SORTED, 0, chl_pass->pw_name, NULL);
-            }
             endpwent ();
             fe = listbox_search_text (chl_list, get_owner (sf_stat->st_uid));
         }
@@ -340,10 +356,8 @@ do_enter_key (Dlg_head * h, int f_pos)
         {
             /* get and put group names in the listbox */
             setgrent ();
-            while ((chl_grp = getgrent ()))
-            {
+            while ((chl_grp = getgrent ()) != NULL)
                 listbox_add_item (chl_list, LISTBOX_APPEND_SORTED, 0, chl_grp->gr_name, NULL);
-            }
             endgrent ();
             fe = listbox_search_text (chl_list, get_group (sf_stat->st_gid));
         }
@@ -353,30 +367,30 @@ do_enter_key (Dlg_head * h, int f_pos)
         b_pos = chl_list->pos;
         add_widget (chl_dlg, chl_list);
 
-        run_dlg (chl_dlg);
+        result = dlg_run (chl_dlg);
 
         if (b_pos != chl_list->pos)
         {
-            int ok = 0;
+            gboolean ok = FALSE;
             char *text;
 
             listbox_get_current (chl_list, &text, NULL);
             if (is_owner)
             {
                 chl_pass = getpwnam (text);
-                if (chl_pass)
+                if (chl_pass != NULL)
                 {
-                    ok = 1;
+                    ok = TRUE;
                     sf_stat->st_uid = chl_pass->pw_uid;
                 }
             }
             else
             {
                 chl_grp = getgrnam (text);
-                if (chl_grp)
+                if (chl_grp != NULL)
                 {
                     sf_stat->st_gid = chl_grp->gr_gid;
-                    ok = 1;
+                    ok = TRUE;
                 }
             }
             if (ok)
@@ -388,22 +402,22 @@ do_enter_key (Dlg_head * h, int f_pos)
             if (ok)
                 print_flags ();
         }
-        if (chl_dlg->ret_value == KEY_LEFT)
+        if (result == KEY_LEFT)
         {
             if (!is_owner)
-                chl_end = 1;
+                chl_end = TRUE;
             dlg_one_up (ch_dlg);
             f_pos--;
         }
-        else if (chl_dlg->ret_value == KEY_RIGHT)
+        else if (result == KEY_RIGHT)
         {
             if (is_owner)
-                chl_end = 1;
+                chl_end = TRUE;
             dlg_one_down (ch_dlg);
             f_pos++;
         }
         /* Here we used to redraw the window */
-        destroy_dlg (chl_dlg);
+        dlg_destroy (chl_dlg);
     }
     while (chl_end);
 }
@@ -413,58 +427,30 @@ do_enter_key (Dlg_head * h, int f_pos)
 static void
 chown_refresh (void)
 {
-    common_dialog_repaint (ch_dlg);
+    dlg_default_repaint (ch_dlg);
 
     tty_setcolor (COLOR_NORMAL);
 
-    dlg_move (ch_dlg, BY - 1, 8);
+    widget_move (ch_dlg, BY - 1, 8);
     tty_print_string (_("owner"));
-    dlg_move (ch_dlg, BY - 1, 16);
+    widget_move (ch_dlg, BY - 1, 16);
     tty_print_string (_("group"));
-    dlg_move (ch_dlg, BY - 1, 24);
+    widget_move (ch_dlg, BY - 1, 24);
     tty_print_string (_("other"));
 
-    dlg_move (ch_dlg, BY - 1, 35);
+    widget_move (ch_dlg, BY - 1, 35);
     tty_print_string (_("owner"));
-    dlg_move (ch_dlg, BY - 1, 53);
+    widget_move (ch_dlg, BY - 1, 53);
     tty_print_string (_("group"));
 
-    dlg_move (ch_dlg, 3, 4);
-    tty_print_string (_("On"));
-    dlg_move (ch_dlg, BY + 1, 4);
+    widget_move (ch_dlg, BY + 1, 3);
     tty_print_string (_("Flag"));
-    dlg_move (ch_dlg, BY + 2, 4);
-    tty_print_string (_("Mode"));
-
-    if (!single_set)
-    {
-        dlg_move (ch_dlg, 3, 54);
-        tty_printf (_("%6d of %d"), files_on_begin - (current_panel->marked) + 1, files_on_begin);
-    }
 
     print_flags ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
-chown_info_update (void)
-{
-    /* display file info */
-    tty_setcolor (COLOR_NORMAL);
-
-    /* name && mode */
-    dlg_move (ch_dlg, 3, 8);
-    tty_print_string (str_fit_to_term (fname, 45, J_LEFT_FIT));
-    dlg_move (ch_dlg, BY + 2, 9);
-    tty_printf ("%12o", get_mode ());
-
-    /* permissions */
-    update_permissions ();
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 static void
 b_setpos (int f_pos)
 {
@@ -477,46 +463,57 @@ b_setpos (int f_pos)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *data)
+advanced_chown_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
-    int i = 0, f_pos = BUTTONS - dlg_get_current_widget_id (h) - single_set - 1;
+    WDialog *h = DIALOG (w);
+    int i;
+    int f_pos;
+    unsigned int id;
+
+    id = dlg_get_current_widget_id (h);
+
+    for (i = 0; i < BUTTONS_PERM; i++)
+        if (chown_advanced_but[i].id == id)
+            break;
+
+    f_pos = i;
+    i = 0;
 
     switch (msg)
     {
-    case DLG_DRAW:
+    case MSG_DRAW:
         chown_refresh ();
         chown_info_update ();
         return MSG_HANDLED;
 
-    case DLG_POST_KEY:
+    case MSG_POST_KEY:
         if (f_pos < 3)
             b_setpos (f_pos);
         return MSG_HANDLED;
 
-    case DLG_FOCUS:
+    case MSG_FOCUS:
         if (f_pos < 3)
         {
             if ((flag_pos / 3) != f_pos)
                 flag_pos = f_pos * 3;
             b_setpos (f_pos);
         }
-        else if (f_pos < 5)
+        else if (f_pos < BUTTONS_PERM)
             flag_pos = f_pos + 6;
         return MSG_HANDLED;
 
-    case DLG_KEY:
+    case MSG_KEY:
         switch (parm)
         {
-
         case XCTRL ('b'):
         case KEY_LEFT:
-            if (f_pos < 5)
+            if (f_pos < BUTTONS_PERM)
                 return (dec_flag_pos (f_pos));
             break;
 
         case XCTRL ('f'):
         case KEY_RIGHT:
-            if (f_pos < 5)
+            if (f_pos < BUTTONS_PERM)
                 return (inc_flag_pos (f_pos));
             break;
 
@@ -527,7 +524,7 @@ advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm,
 
         case '\n':
         case KEY_ENTER:
-            if (f_pos <= 2 || f_pos >= 5)
+            if (f_pos <= 2 || f_pos >= BUTTONS_PERM)
                 break;
             do_enter_key (h, f_pos);
             return MSG_HANDLED;
@@ -544,8 +541,8 @@ advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm,
                 ch_flags[i * 3 + parm - 3] = (x_toggle & (1 << parm)) ? '-' : '+';
             x_toggle ^= (1 << parm);
             update_mode (h);
-            dlg_broadcast_msg (h, WIDGET_DRAW, FALSE);
-            send_message ((Widget *) h->current->data, WIDGET_FOCUS, 0);
+            dlg_broadcast_msg (h, MSG_DRAW);
+            send_message (h->current->data, NULL, MSG_FOCUS, 0, NULL);
             break;
 
         case XCTRL ('x'):
@@ -560,8 +557,8 @@ advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm,
                 ch_flags[i * 3 + parm] = (x_toggle & (1 << parm)) ? '-' : '+';
             x_toggle ^= (1 << parm);
             update_mode (h);
-            dlg_broadcast_msg (h, WIDGET_DRAW, FALSE);
-            send_message ((Widget *) h->current->data, WIDGET_FOCUS, 0);
+            dlg_broadcast_msg (h, MSG_DRAW);
+            send_message (h->current->data, NULL, MSG_FOCUS, 0, NULL);
             break;
 
         case 'x':
@@ -574,7 +571,7 @@ advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm,
             if (f_pos > 2)
                 break;
             flag_pos = f_pos * 3 + i;   /* (strchr(ch_perm,parm)-ch_perm); */
-            if (((WButton *) h->current->data)->text.start[(flag_pos % 3)] == '-')
+            if (BUTTON (h->current->data)->text.start[(flag_pos % 3)] == '-')
                 ch_flags[flag_pos] = '+';
             else
                 ch_flags[flag_pos] = '-';
@@ -588,11 +585,12 @@ advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm,
             i++;
 
         case '1':
-            if (f_pos > 2)
-                break;
-            flag_pos = i + f_pos * 3;
-            ch_flags[flag_pos] = '=';
-            update_mode (h);
+            if (f_pos <= 2)
+            {
+                flag_pos = i + f_pos * 3;
+                ch_flags[flag_pos] = '=';
+                update_mode (h);
+            }
             break;
 
         case '-':
@@ -609,8 +607,8 @@ advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm,
             {
                 ch_flags[flag_pos] = parm;
                 update_mode (h);
-                advanced_chown_callback (h, sender, DLG_KEY, KEY_RIGHT, NULL);
-                if (flag_pos > 8 || !(flag_pos % 3))
+                send_message (h, sender, MSG_KEY, KEY_RIGHT, NULL);
+                if (flag_pos > 8 || (flag_pos % 3) == 0)
                     dlg_one_down (h);
             }
             break;
@@ -618,7 +616,7 @@ advanced_chown_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm,
         return MSG_NOT_HANDLED;
 
     default:
-        return default_dlg_callback (h, sender, msg, parm, data);
+        return dlg_default_callback (w, sender, msg, parm, data);
     }
 }
 
@@ -628,63 +626,107 @@ static void
 init_chown_advanced (void)
 {
     int i;
-    enum
-    { dlg_h = 13, dlg_w = 74, n_elem = 4 };
-#ifdef ENABLE_NLS
-    static int i18n_len = 0;
+    int dlg_h = 12;
+    int dlg_w = 74;
+    int y;
 
-    if (i18n_len == 0)
+    static gboolean i18n = FALSE;
+
+    if (!i18n)
     {
-        int dx, cx;
-        for (i = 0; i < n_elem; i++)
+        for (i = BUTTONS_PERM; i < BUTTONS; i++)
         {
+#ifdef ENABLE_NLS
             chown_advanced_but[i].text = _(chown_advanced_but[i].text);
-            i18n_len += str_term_width1 (chown_advanced_but[i].text) + 3;
-            if (DEFPUSH_BUTTON == chown_advanced_but[i].flags)
-                i18n_len += 2;  /* "<>" */
-        }
-        cx = dx = (dlg_w - i18n_len - 2) / (n_elem + 1);
-
-        /* Reversed order */
-        for (i = n_elem - 1; i >= 0; i--)
-        {
-            chown_advanced_but[i].x = cx;
-            cx += str_term_width1 (chown_advanced_but[i].text) + 3 + dx;
-        }
-    }
 #endif /* ENABLE_NLS */
 
-    sf_stat = g_new (struct stat, 1);
+            chown_advanced_but[i].len = str_term_width1 (chown_advanced_but[i].text) + 3;
+            if (chown_advanced_but[i].flags == DEFPUSH_BUTTON)
+                chown_advanced_but[i].len += 2; /* "<>" */
+        }
+
+        i18n = TRUE;
+    }
+
     do_refresh ();
-    end_chown = need_update = current_file = 0;
-    single_set = (current_panel->marked < 2) ? 2 : 0;
+
+    sf_stat = g_new (struct stat, 1);
+    current_file = 0;
+    end_chown = need_update = FALSE;
+    single_set = (current_panel->marked < 2);
     memset (ch_flags, '=', 11);
     flag_pos = 0;
     x_toggle = 070;
 
+    if (!single_set)
+        dlg_h += 2;
+
     ch_dlg =
-        create_dlg (TRUE, 0, 0, dlg_h, dlg_w, dialog_colors, advanced_chown_callback, NULL,
-                    "[Advanced Chown]", _("Chown advanced command"), DLG_CENTER | DLG_REVERSE);
+        dlg_create (TRUE, 0, 0, dlg_h, dlg_w, dialog_colors, advanced_chown_callback, NULL,
+                    "[Advanced Chown]", _("Chown advanced command"), DLG_CENTER);
 
-#define XTRACT(i) BY+chown_advanced_but[i].y, BX+chown_advanced_but[i].x, \
+
+    l_filename = label_new (2, 3, "");
+    add_widget (ch_dlg, l_filename);
+
+    add_widget (ch_dlg, hline_new (3, -1, -1));
+
+#define XTRACT(i,y) y, BX+chown_advanced_but[i].x, \
         chown_advanced_but[i].ret_cmd, chown_advanced_but[i].flags, \
-        (chown_advanced_but[i].text), 0
+        (chown_advanced_but[i].text), NULL
+    b_att[0] = button_new (XTRACT (0, BY));
+    chown_advanced_but[0].id = add_widget (ch_dlg, b_att[0]);
+    b_att[1] = button_new (XTRACT (1, BY));
+    chown_advanced_but[1].id = add_widget (ch_dlg, b_att[1]);
+    b_att[2] = button_new (XTRACT (2, BY));
+    chown_advanced_but[2].id = add_widget (ch_dlg, b_att[2]);
+    b_user = button_new (XTRACT (3, BY));
+    chown_advanced_but[3].id = add_widget (ch_dlg, b_user);
+    b_group = button_new (XTRACT (4, BY));
+    chown_advanced_but[4].id = add_widget (ch_dlg, b_group);
+#undef XTRACT
 
-    for (i = 0; i < BUTTONS - 5; i++)
-        if (!single_set || i < 2)
-            add_widget (ch_dlg, button_new (XTRACT (i)));
+    l_mode = label_new (BY + 2, 3, "");
+    add_widget (ch_dlg, l_mode);
 
-    b_att[0] = button_new (XTRACT (8));
-    b_att[1] = button_new (XTRACT (7));
-    b_att[2] = button_new (XTRACT (6));
-    b_user = button_new (XTRACT (5));
-    b_group = button_new (XTRACT (4));
+    y = BY + 3;
+    if (!single_set)
+    {
+        i = BUTTONS_PERM;
+        add_widget (ch_dlg, hline_new (y++, -1, -1));
+        chown_advanced_but[i].id = add_widget (ch_dlg,
+                                               button_new (y,
+                                                           WIDGET (ch_dlg)->cols / 2 -
+                                                           chown_advanced_but[i].len,
+                                                           chown_advanced_but[i].ret_cmd,
+                                                           chown_advanced_but[i].flags,
+                                                           chown_advanced_but[i].text, NULL));
+        i++;
+        chown_advanced_but[i].id = add_widget (ch_dlg,
+                                               button_new (y, WIDGET (ch_dlg)->cols / 2 + 1,
+                                                           chown_advanced_but[i].ret_cmd,
+                                                           chown_advanced_but[i].flags,
+                                                           chown_advanced_but[i].text, NULL));
+        y++;
+    }
 
-    add_widget (ch_dlg, b_group);
-    add_widget (ch_dlg, b_user);
-    add_widget (ch_dlg, b_att[2]);
-    add_widget (ch_dlg, b_att[1]);
-    add_widget (ch_dlg, b_att[0]);
+    i = BUTTONS_PERM + 2;
+    add_widget (ch_dlg, hline_new (y++, -1, -1));
+    chown_advanced_but[i].id = add_widget (ch_dlg,
+                                           button_new (y,
+                                                       WIDGET (ch_dlg)->cols / 2 -
+                                                       chown_advanced_but[i].len,
+                                                       chown_advanced_but[i].ret_cmd,
+                                                       chown_advanced_but[i].flags,
+                                                       chown_advanced_but[i].text, NULL));
+    i++;
+    chown_advanced_but[i].id = add_widget (ch_dlg,
+                                           button_new (y, WIDGET (ch_dlg)->cols / 2 + 1,
+                                                       chown_advanced_but[i].ret_cmd,
+                                                       chown_advanced_but[i].flags,
+                                                       chown_advanced_but[i].text, NULL));
+
+    dlg_select_widget (b_att[0]);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -732,13 +774,13 @@ apply_advanced_chowns (struct stat *sf)
 
     lc_fname = current_panel->dir.list[current_file].fname;
     vpath = vfs_path_from_str (lc_fname);
-    need_update = end_chown = 1;
+    need_update = end_chown = TRUE;
     if (mc_chmod (vpath, get_mode ()) == -1)
         message (D_ERROR, MSG_ERROR, _("Cannot chmod \"%s\"\n%s"),
                  lc_fname, unix_error_string (errno));
     /* call mc_chown only, if mc_chmod didn't fail */
-    else if (mc_chown (vpath, (ch_flags[9] == '+') ? sf->st_uid : (uid_t) - 1,
-                       (ch_flags[10] == '+') ? sf->st_gid : (gid_t) - 1) == -1)
+    else if (mc_chown (vpath, (ch_flags[9] == '+') ? sf->st_uid : (uid_t) (-1),
+                       (ch_flags[10] == '+') ? sf->st_gid : (gid_t) (-1)) == -1)
         message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"),
                  lc_fname, unix_error_string (errno));
     do_file_mark (current_panel, current_file, 0);
@@ -754,20 +796,22 @@ apply_advanced_chowns (struct stat *sf)
             vfs_path_free (vpath);
             break;
         }
+
         ch_cmode = sf->st_mode;
+
         if (mc_chmod (vpath, get_mode ()) == -1)
             message (D_ERROR, MSG_ERROR, _("Cannot chmod \"%s\"\n%s"),
                      lc_fname, unix_error_string (errno));
         /* call mc_chown only, if mc_chmod didn't fail */
-        else if (mc_chown (vpath, (ch_flags[9] == '+') ? a_uid : (uid_t) - 1,
-                           (ch_flags[10] == '+') ? a_gid : (gid_t) - 1) == -1)
+        else if (mc_chown (vpath, (ch_flags[9] == '+') ? a_uid : (uid_t) (-1),
+                           (ch_flags[10] == '+') ? a_gid : (gid_t) (-1)) == -1)
             message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"),
                      lc_fname, unix_error_string (errno));
 
         do_file_mark (current_panel, current_file, 0);
         vfs_path_free (vpath);
     }
-    while (current_panel->marked);
+    while (current_panel->marked != 0);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -777,11 +821,18 @@ apply_advanced_chowns (struct stat *sf)
 void
 chown_advanced_cmd (void)
 {
-    files_on_begin = current_panel->marked;
+    /* Number of files at startup */
+    int files_on_begin;
+
+    files_on_begin = max (1, current_panel->marked);
 
     do
     {                           /* do while any files remaining */
+        int file_idx;
+        char buffer[BUF_MEDIUM];
         vfs_path_t *vpath;
+        int result;
+
         init_chown_advanced ();
 
         if (current_panel->marked)
@@ -792,60 +843,57 @@ chown_advanced_cmd (void)
 
         if (mc_stat (vpath, sf_stat) != 0)
         {                       /* get status of file */
-            destroy_dlg (ch_dlg);
+            dlg_destroy (ch_dlg);
             vfs_path_free (vpath);
             break;
         }
+
         ch_cmode = sf_stat->st_mode;
 
+        file_idx = files_on_begin == 1 ? 1 : (files_on_begin - current_panel->marked + 1);
+        g_snprintf (buffer, sizeof (buffer), "%s (%d/%d)",
+                    str_fit_to_term (fname, WIDGET (ch_dlg)->cols - 20, J_LEFT_FIT),
+                    file_idx, files_on_begin);
+        label_set_text (l_filename, buffer);
         chown_refresh ();
-
         update_ownership ();
 
-        /* game can begin */
-        run_dlg (ch_dlg);
+        result = dlg_run (ch_dlg);
 
-        switch (ch_dlg->ret_value)
+        switch (result)
         {
         case B_CANCEL:
-            end_chown = 1;
+            end_chown = TRUE;
             break;
 
         case B_ENTER:
-            {
-                vfs_path_t *fname_vpath;
-
-                fname_vpath = vfs_path_from_str (fname);
-                need_update = 1;
-                if (mc_chmod (fname_vpath, get_mode ()) == -1)
-                    message (D_ERROR, MSG_ERROR, _("Cannot chmod \"%s\"\n%s"),
-                             fname, unix_error_string (errno));
-                /* call mc_chown only, if mc_chmod didn't fail */
-                else if (mc_chown
-                         (fname_vpath, (ch_flags[9] == '+') ? sf_stat->st_uid : (uid_t) - 1,
-                          (ch_flags[10] == '+') ? sf_stat->st_gid : (gid_t) - 1) == -1)
-                    message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"), fname,
-                             unix_error_string (errno));
-                vfs_path_free (fname_vpath);
-            }
+            need_update = TRUE;
+            if (mc_chmod (vpath, get_mode ()) == -1)
+                message (D_ERROR, MSG_ERROR, _("Cannot chmod \"%s\"\n%s"),
+                         fname, unix_error_string (errno));
+            /* call mc_chown only, if mc_chmod didn't fail */
+            else if (mc_chown
+                     (vpath, (ch_flags[9] == '+') ? sf_stat->st_uid : (uid_t) (-1),
+                      (ch_flags[10] == '+') ? sf_stat->st_gid : (gid_t) (-1)) == -1)
+                message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"), fname,
+                         unix_error_string (errno));
             break;
+
         case B_SETALL:
             apply_advanced_chowns (sf_stat);
             break;
 
         case B_SKIP:
             break;
-
         }
 
-        if (current_panel->marked && ch_dlg->ret_value != B_CANCEL)
+        if (current_panel->marked && result != B_CANCEL)
         {
             do_file_mark (current_panel, current_file, 0);
-            need_update = 1;
+            need_update = TRUE;
         }
-        destroy_dlg (ch_dlg);
+        dlg_destroy (ch_dlg);
         vfs_path_free (vpath);
-
     }
     while (current_panel->marked && !end_chown);
 
