@@ -7,6 +7,7 @@
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2013
 
    This file is part of the Midnight Commander.
 
@@ -72,7 +73,7 @@ static int case_sensitive = OS_SORT_CASE_SENSITIVE_DEFAULT;
 /* Are the exec_bit files top in list */
 static gboolean exec_first = TRUE;
 
-static dir_list dir_copy = { 0, 0 };
+static dir_list dir_copy = { NULL, 0, 0 };
 
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -129,57 +130,34 @@ clean_sort_keys (dir_list * list, int start, int count)
 
     for (i = 0; i < count; i++)
     {
-        str_release_key (list->list[i + start].sort_key, case_sensitive);
-        list->list[i + start].sort_key = NULL;
-        str_release_key (list->list[i + start].second_sort_key, case_sensitive);
-        list->list[i + start].second_sort_key = NULL;
+        file_entry_t *fentry;
+
+        fentry = &list->list[i + start];
+        str_release_key (fentry->sort_key, case_sensitive);
+        fentry->sort_key = NULL;
+        str_release_key (fentry->second_sort_key, case_sensitive);
+        fentry->second_sort_key = NULL;
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 /**
- * Increase directory list by RESIZE_STEPS
- *
- * @param list directory list
- * @return FALSE on failure, TRUE on success
+ * If you change handle_dirent then check also handle_path.
+ * @return FALSE = don't add, TRUE = add to the list
  */
 
 static gboolean
-grow_list (dir_list * list)
-{
-    if (list == NULL)
-        return FALSE;
-
-    list->list = g_try_realloc (list->list, sizeof (file_entry) * (list->size + RESIZE_STEPS));
-
-    if (list->list == NULL)
-        return FALSE;
-
-    list->size += RESIZE_STEPS;
-
-    return TRUE;
-}
-
-/* --------------------------------------------------------------------------------------------- */
-/**
- * If you change handle_dirent then check also handle_path.
- * @return -1 = failure, 0 = don't add, 1 = add to the list
- */
-
-static int
-handle_dirent (dir_list * list, const char *fltr, struct dirent *dp,
-               struct stat *buf1, int next_free, int *link_to_dir, int *stale_link)
+handle_dirent (struct dirent *dp, const char *fltr, struct stat *buf1, int *link_to_dir,
+               int *stale_link)
 {
     vfs_path_t *vpath;
 
-    if (dp->d_name[0] == '.' && dp->d_name[1] == 0)
-        return 0;
-    if (dp->d_name[0] == '.' && dp->d_name[1] == '.' && dp->d_name[2] == 0)
-        return 0;
+    if (DIR_IS_DOT (dp->d_name) || DIR_IS_DOTDOT (dp->d_name))
+        return FALSE;
     if (!panels_options.show_dot_files && (dp->d_name[0] == '.'))
-        return 0;
-    if (!panels_options.show_backups && dp->d_name[NLENGTH (dp) - 1] == '~')
-        return 0;
+        return FALSE;
+    if (!panels_options.show_backups && dp->d_name[strlen (dp->d_name) - 1] == '~')
+        return FALSE;
 
     vpath = vfs_path_from_str (dp->d_name);
     if (mc_lstat (vpath, buf1) == -1)
@@ -201,28 +179,24 @@ handle_dirent (dir_list * list, const char *fltr, struct dirent *dp,
     if (S_ISLNK (buf1->st_mode))
     {
         struct stat buf2;
+
         if (mc_stat (vpath, &buf2) == 0)
             *link_to_dir = S_ISDIR (buf2.st_mode) != 0;
         else
             *stale_link = 1;
     }
+
     vfs_path_free (vpath);
-    if (!(S_ISDIR (buf1->st_mode) || *link_to_dir) && (fltr != NULL)
-        && !mc_search (fltr, dp->d_name, MC_SEARCH_T_GLOB))
-        return 0;
 
-    /* Need to grow the *list? */
-    if (next_free == list->size && !grow_list (list))
-        return -1;
-
-    return 1;
+    return (S_ISDIR (buf1->st_mode) || *link_to_dir != 0 || fltr == NULL
+            || mc_search (fltr, NULL, dp->d_name, MC_SEARCH_T_GLOB));
 }
 
 /* --------------------------------------------------------------------------------------------- */
 /** get info about ".." */
 
 static gboolean
-get_dotdot_dir_stat (const vfs_path_t * vpath, struct stat *st)
+dir_get_dotdot_stat (const vfs_path_t * vpath, struct stat *st)
 {
     gboolean ret = FALSE;
 
@@ -234,12 +208,10 @@ get_dotdot_dir_stat (const vfs_path_t * vpath, struct stat *st)
         if (path != NULL && *path != '\0')
         {
             vfs_path_t *tmp_vpath;
-            struct stat s;
 
             tmp_vpath = vfs_path_append_new (vpath, "..", NULL);
-            ret = mc_stat (tmp_vpath, &s) == 0;
+            ret = mc_stat (tmp_vpath, st) == 0;
             vfs_path_free (tmp_vpath);
-            *st = s;
         }
     }
 
@@ -253,25 +225,117 @@ alloc_dir_copy (int size)
 {
     if (dir_copy.size < size)
     {
-        if (dir_copy.list)
+        if (dir_copy.list != NULL)
         {
             int i;
-            for (i = 0; i < dir_copy.size; i++)
-                g_free (dir_copy.list[i].fname);
+
+            for (i = 0; i < dir_copy.len; i++)
+            {
+                file_entry_t *fentry;
+
+                fentry = &(dir_copy.list)[i];
+                g_free (fentry->fname);
+            }
             g_free (dir_copy.list);
         }
 
-        dir_copy.list = g_new0 (file_entry, size);
+        dir_copy.list = g_new0 (file_entry_t, size);
         dir_copy.size = size;
+        dir_copy.len = 0;
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Increase or decrease directory list size.
+ *
+ * @param list directory list
+ * @param delta value by increase (if positive) or decrease (if negative) list size
+ *
+ * @return FALSE on failure, TRUE on success
+ */
+
+gboolean
+dir_list_grow (dir_list * list, int delta)
+{
+    int size;
+    gboolean clear = FALSE;
+
+    if (list == NULL)
+        return FALSE;
+
+    if (delta == 0)
+        return TRUE;
+
+    size = list->size + delta;
+    if (size <= 0)
+    {
+        size = DIR_LIST_MIN_SIZE;
+        clear = TRUE;
+    }
+
+    if (size != list->size)
+    {
+        file_entry_t *fe;
+
+        fe = g_try_renew (file_entry_t, list->list, size);
+        if (fe == NULL)
+            return FALSE;
+
+        list->list = fe;
+        list->size = size;
+    }
+
+    list->len = clear ? 0 : min (list->len, size);
+
+    return TRUE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Append file info to the directory list.
+ *
+ * @param list directory list
+ * @param fname file name
+ * @param st file stat info
+ * @param link_to_dir is file link to directory
+ * @param stale_link is file stale elink
+ *
+ * @return FALSE on failure, TRUE on success
+ */
+
+gboolean
+dir_list_append (dir_list * list, const char *fname, const struct stat * st,
+                 gboolean link_to_dir, gboolean stale_link)
+{
+    file_entry_t *fentry;
+
+    /* Need to grow the *list? */
+    if (list->len == list->size && !dir_list_grow (list, DIR_LIST_RESIZE_STEP))
+        return FALSE;
+
+    fentry = &list->list[list->len];
+    fentry->fnamelen = strlen (fname);
+    fentry->fname = g_strndup (fname, fentry->fnamelen);
+    fentry->f.marked = 0;
+    fentry->f.link_to_dir = link_to_dir ? 1 : 0;
+    fentry->f.stale_link = stale_link ? 1 : 0;
+    fentry->f.dir_size_computed = 0;
+    fentry->st = *st;
+    fentry->sort_key = NULL;
+    fentry->second_sort_key = NULL;
+
+    list->len++;
+
+    return TRUE;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 
 int
-unsorted (file_entry * a, file_entry * b)
+unsorted (file_entry_t * a, file_entry_t * b)
 {
     (void) a;
     (void) b;
@@ -281,7 +345,7 @@ unsorted (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_name (file_entry * a, file_entry * b)
+sort_name (file_entry_t * a, file_entry_t * b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -302,7 +366,7 @@ sort_name (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_vers (file_entry * a, file_entry * b)
+sort_vers (file_entry_t * a, file_entry_t * b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -320,14 +384,15 @@ sort_vers (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_ext (file_entry * a, file_entry * b)
+sort_ext (file_entry_t * a, file_entry_t * b)
 {
-    int r;
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
 
     if (ad == bd || panels_options.mix_all_files)
     {
+        int r;
+
         if (a->second_sort_key == NULL)
             a->second_sort_key = str_create_key (extension (a->fname), case_sensitive);
         if (b->second_sort_key == NULL)
@@ -346,7 +411,7 @@ sort_ext (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_time (file_entry * a, file_entry * b)
+sort_time (file_entry_t * a, file_entry_t * b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -366,7 +431,7 @@ sort_time (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_ctime (file_entry * a, file_entry * b)
+sort_ctime (file_entry_t * a, file_entry_t * b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -386,7 +451,7 @@ sort_ctime (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_atime (file_entry * a, file_entry * b)
+sort_atime (file_entry_t * a, file_entry_t * b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -406,7 +471,7 @@ sort_atime (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_inode (file_entry * a, file_entry * b)
+sort_inode (file_entry_t * a, file_entry_t * b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -420,7 +485,7 @@ sort_inode (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-sort_size (file_entry * a, file_entry * b)
+sort_size (file_entry_t * a, file_entry_t * b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -439,59 +504,74 @@ sort_size (file_entry * a, file_entry * b)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-do_sort (dir_list * list, sortfn * sort, int top, gboolean reverse_f, gboolean case_sensitive_f,
-         gboolean exec_first_f)
+dir_list_sort (dir_list * list, GCompareFunc sort, const dir_sort_options_t * sort_op)
 {
+    file_entry_t *fentry;
     int dot_dot_found = 0;
 
-    if (top == 0)
+    if (list->len < 2)
         return;
 
     /* If there is an ".." entry the caller must take care to
        ensure that it occupies the first list element. */
-    if (strcmp (list->list[0].fname, "..") == 0)
+    fentry = &list->list[0];
+    if (DIR_IS_DOTDOT (fentry->fname))
         dot_dot_found = 1;
 
-    reverse = reverse_f ? -1 : 1;
-    case_sensitive = case_sensitive_f ? 1 : 0;
-    exec_first = exec_first_f;
-    qsort (&(list->list)[dot_dot_found], top + 1 - dot_dot_found, sizeof (file_entry), sort);
+    reverse = sort_op->reverse ? -1 : 1;
+    case_sensitive = sort_op->case_sensitive ? 1 : 0;
+    exec_first = sort_op->exec_first;
+    qsort (&(list->list)[dot_dot_found], list->len - dot_dot_found, sizeof (file_entry_t), sort);
 
-    clean_sort_keys (list, dot_dot_found, top + 1 - dot_dot_found);
+    clean_sort_keys (list, dot_dot_found, list->len - dot_dot_found);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 void
-clean_dir (dir_list * list, int count)
+dir_list_clean (dir_list * list)
 {
     int i;
 
-    for (i = 0; i < count; i++)
+    for (i = 0; i < list->len; i++)
     {
-        g_free (list->list[i].fname);
-        list->list[i].fname = NULL;
+        file_entry_t *fentry;
+
+        fentry = &list->list[i];
+        g_free (fentry->fname);
+        fentry->fname = NULL;
     }
+
+    list->len = 0;
+    /* reduce memory usage */
+    dir_list_grow (list, DIR_LIST_MIN_SIZE - list->size);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 /** Used to set up a directory list when there is no access to a directory */
 
 gboolean
-set_zero_dir (dir_list * list)
+dir_list_init (dir_list * list)
 {
-    /* Need to grow the *list? */
-    if (list->size == 0 && !grow_list (list))
-        return FALSE;
+    file_entry_t *fentry;
 
-    memset (&(list->list)[0], 0, sizeof (file_entry));
-    list->list[0].fnamelen = 2;
-    list->list[0].fname = g_strndup ("..", list->list[0].fnamelen);
-    list->list[0].f.link_to_dir = 0;
-    list->list[0].f.stale_link = 0;
-    list->list[0].f.dir_size_computed = 0;
-    list->list[0].f.marked = 0;
-    list->list[0].st.st_mode = 040755;
+    /* Need to grow the *list? */
+    if (list->size == 0 && !dir_list_grow (list, DIR_LIST_RESIZE_STEP))
+    {
+        list->len = 0;
+        return FALSE;
+    }
+
+    fentry = &list->list[0];
+    memset (fentry, 0, sizeof (file_entry_t));
+    fentry->fnamelen = 2;
+    fentry->fname = g_strndup ("..", fentry->fnamelen);
+    fentry->f.link_to_dir = 0;
+    fentry->f.stale_link = 0;
+    fentry->f.dir_size_computed = 0;
+    fentry->f.marked = 0;
+    fentry->st.st_mode = 040755;
+    list->len = 1;
     return TRUE;
 }
 
@@ -502,24 +582,21 @@ set_zero_dir (dir_list * list)
    and panels_options.show_backups.
    Moreover handle_path can't be used with a filemask.
    If you change handle_path then check also handle_dirent. */
-/* Return values: -1 = failure, 0 = don't add, 1 = add to the list */
+/* Return values: FALSE = don't add, TRUE = add to the list */
 
-int
-handle_path (dir_list * list, const char *path,
-             struct stat *buf1, int next_free, int *link_to_dir, int *stale_link)
+gboolean
+handle_path (const char *path, struct stat * buf1, int *link_to_dir, int *stale_link)
 {
     vfs_path_t *vpath;
 
-    if (path[0] == '.' && path[1] == 0)
-        return 0;
-    if (path[0] == '.' && path[1] == '.' && path[2] == 0)
-        return 0;
+    if (DIR_IS_DOT (path) || DIR_IS_DOTDOT (path))
+        return FALSE;
 
     vpath = vfs_path_from_str (path);
     if (mc_lstat (vpath, buf1) == -1)
     {
         vfs_path_free (vpath);
-        return 0;
+        return FALSE;
     }
 
     if (S_ISDIR (buf1->st_mode))
@@ -531,6 +608,7 @@ handle_path (dir_list * list, const char *path,
     if (S_ISLNK (buf1->st_mode))
     {
         struct stat buf2;
+
         if (mc_stat (vpath, &buf2) == 0)
             *link_to_dir = S_ISDIR (buf2.st_mode) != 0;
         else
@@ -539,38 +617,34 @@ handle_path (dir_list * list, const char *path,
 
     vfs_path_free (vpath);
 
-    /* Need to grow the *list? */
-    if (next_free == list->size && !grow_list (list))
-        return -1;
-
-    return 1;
+    return TRUE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-int
-do_load_dir (const vfs_path_t * vpath, dir_list * list, sortfn * sort, gboolean lc_reverse,
-             gboolean lc_case_sensitive, gboolean exec_ff, const char *fltr)
+void
+dir_list_load (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
+               const dir_sort_options_t * sort_op, const char *fltr)
 {
     DIR *dirp;
     struct dirent *dp;
-    int status, link_to_dir, stale_link;
-    int next_free = 0;
+    int link_to_dir, stale_link;
     struct stat st;
+    file_entry_t *fentry;
 
     /* ".." (if any) must be the first entry in the list */
-    if (!set_zero_dir (list))
-        return next_free;
+    if (!dir_list_init (list))
+        return;
 
-    if (get_dotdot_dir_stat (vpath, &st))
-        list->list[next_free].st = st;
-    next_free++;
+    fentry = &list->list[0];
+    if (dir_get_dotdot_stat (vpath, &st))
+        fentry->st = st;
 
     dirp = mc_opendir (vpath);
     if (dirp == NULL)
     {
         message (D_ERROR, MSG_ERROR, _("Cannot read directory contents"));
-        return next_free;
+        return;
     }
 
     tree_store_start_check (vpath);
@@ -581,47 +655,33 @@ do_load_dir (const vfs_path_t * vpath, dir_list * list, sortfn * sort, gboolean
         vpath_str = vfs_path_as_str (vpath);
         /* Do not add a ".." entry to the root directory */
         if ((vpath_str[0] == PATH_SEP) && (vpath_str[1] == '\0'))
-            next_free--;
+            list->len--;
     }
 
     while ((dp = mc_readdir (dirp)) != NULL)
     {
-        status = handle_dirent (list, fltr, dp, &st, next_free, &link_to_dir, &stale_link);
-        if (status == 0)
+        if (!handle_dirent (dp, fltr, &st, &link_to_dir, &stale_link))
             continue;
-        if (status == -1)
+
+        if (!dir_list_append (list, dp->d_name, &st, link_to_dir != 0, stale_link != 0))
             goto ret;
 
-        list->list[next_free].fnamelen = NLENGTH (dp);
-        list->list[next_free].fname = g_strndup (dp->d_name, list->list[next_free].fnamelen);
-        list->list[next_free].f.marked = 0;
-        list->list[next_free].f.link_to_dir = link_to_dir;
-        list->list[next_free].f.stale_link = stale_link;
-        list->list[next_free].f.dir_size_computed = 0;
-        list->list[next_free].st = st;
-        list->list[next_free].sort_key = NULL;
-        list->list[next_free].second_sort_key = NULL;
-        next_free++;
-
-        if ((next_free & 31) == 0)
+        if ((list->len & 31) == 0)
             rotate_dash (TRUE);
     }
 
-    if (next_free != 0)
-        do_sort (list, sort, next_free - 1, lc_reverse, lc_case_sensitive, exec_ff);
+    dir_list_sort (list, sort, sort_op);
 
   ret:
     mc_closedir (dirp);
     tree_store_end_check ();
     rotate_dash (FALSE);
-    return next_free;
 }
 
-
 /* --------------------------------------------------------------------------------------------- */
 
 gboolean
-if_link_is_exe (const vfs_path_t * full_name_vpath, const file_entry * file)
+if_link_is_exe (const vfs_path_t * full_name_vpath, const file_entry_t * file)
 {
     struct stat b;
 
@@ -633,14 +693,13 @@ if_link_is_exe (const vfs_path_t * full_name_vpath, const file_entry * file)
 /* --------------------------------------------------------------------------------------------- */
 /** If fltr is null, then it is a match */
 
-int
-do_reload_dir (const vfs_path_t * vpath, dir_list * list, sortfn * sort, int count,
-               gboolean lc_reverse, gboolean lc_case_sensitive, gboolean exec_ff, const char *fltr)
+void
+dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
+                 const dir_sort_options_t * sort_op, const char *fltr)
 {
     DIR *dirp;
     struct dirent *dp;
-    int next_free = 0;
-    int i, status, link_to_dir, stale_link;
+    int i, link_to_dir, stale_link;
     struct stat st;
     int marked_cnt;
     GHashTable *marked_files;
@@ -650,27 +709,33 @@ do_reload_dir (const vfs_path_t * vpath, dir_list * list, sortfn * sort, int cou
     if (dirp == NULL)
     {
         message (D_ERROR, MSG_ERROR, _("Cannot read directory contents"));
-        clean_dir (list, count);
-        return set_zero_dir (list) ? 1 : 0;
+        dir_list_clean (list);
+        dir_list_init (list);
+        return;
     }
 
     tree_store_start_check (vpath);
 
     marked_files = g_hash_table_new (g_str_hash, g_str_equal);
-    alloc_dir_copy (list->size);
-    for (marked_cnt = i = 0; i < count; i++)
+    alloc_dir_copy (list->len);
+    for (marked_cnt = i = 0; i < list->len; i++)
     {
-        dir_copy.list[i].fnamelen = list->list[i].fnamelen;
-        dir_copy.list[i].fname = list->list[i].fname;
-        dir_copy.list[i].f.marked = list->list[i].f.marked;
-        dir_copy.list[i].f.dir_size_computed = list->list[i].f.dir_size_computed;
-        dir_copy.list[i].f.link_to_dir = list->list[i].f.link_to_dir;
-        dir_copy.list[i].f.stale_link = list->list[i].f.stale_link;
-        dir_copy.list[i].sort_key = NULL;
-        dir_copy.list[i].second_sort_key = NULL;
-        if (list->list[i].f.marked)
+        file_entry_t *fentry, *dfentry;
+
+        fentry = &list->list[i];
+        dfentry = &dir_copy.list[i];
+
+        dfentry->fnamelen = fentry->fnamelen;
+        dfentry->fname = fentry->fname;
+        dfentry->f.marked = fentry->f.marked;
+        dfentry->f.dir_size_computed = fentry->f.dir_size_computed;
+        dfentry->f.link_to_dir = fentry->f.link_to_dir;
+        dfentry->f.stale_link = fentry->f.stale_link;
+        dfentry->sort_key = NULL;
+        dfentry->second_sort_key = NULL;
+        if (fentry->f.marked)
         {
-            g_hash_table_insert (marked_files, dir_copy.list[i].fname, &dir_copy.list[i]);
+            g_hash_table_insert (marked_files, dfentry->fname, dfentry);
             marked_cnt++;
         }
     }
@@ -678,29 +743,36 @@ do_reload_dir (const vfs_path_t * vpath, dir_list * list, sortfn * sort, int cou
     /* Add ".." except to the root directory. The ".." entry
        (if any) must be the first in the list. */
     tmp_path = vfs_path_get_by_index (vpath, 0)->path;
-    if (!
-        (vfs_path_elements_count (vpath) == 1 && (tmp_path[0] == PATH_SEP)
-         && (tmp_path[1] == '\0')))
+    if (vfs_path_elements_count (vpath) == 1 && tmp_path[0] == PATH_SEP && tmp_path[1] == '\0')
     {
-        if (!set_zero_dir (list))
+        /* root directory */
+        dir_list_clean (list);
+    }
+    else
+    {
+        if (!dir_list_init (list))
         {
-            clean_dir (list, count);
-            clean_dir (&dir_copy, count);
-            return next_free;
+            dir_list_clean (&dir_copy);
+            return;
         }
 
-        if (get_dotdot_dir_stat (vpath, &st))
-            list->list[next_free].st = st;
+        if (dir_get_dotdot_stat (vpath, &st))
+        {
+            file_entry_t *fentry;
 
-        next_free++;
+            fentry = &list->list[0];
+            fentry->st = st;
+        }
     }
 
-    while ((dp = mc_readdir (dirp)))
+    while ((dp = mc_readdir (dirp)) != NULL)
     {
-        status = handle_dirent (list, fltr, dp, &st, next_free, &link_to_dir, &stale_link);
-        if (status == 0)
+        file_entry_t *fentry;
+
+        if (!handle_dirent (dp, fltr, &st, &link_to_dir, &stale_link))
             continue;
-        if (status == -1)
+
+        if (!dir_list_append (list, dp->d_name, &st, link_to_dir != 0, stale_link != 0))
         {
             mc_closedir (dirp);
             /* Norbert (Feb 12, 1997):
@@ -708,55 +780,41 @@ do_reload_dir (const vfs_path_t * vpath, dir_list * list, sortfn * sort, int cou
                -1 means big trouble (at the moment no memory left),
                I don't bother with further cleanup because if one gets to
                this point he will have more problems than a few memory
-               leaks and because one 'clean_dir' would not be enough (and
+               leaks and because one 'dir_list_clean' would not be enough (and
                because I don't want to spent the time to make it working,
                IMHO it's not worthwhile).
-               clean_dir (&dir_copy, count);
+               dir_list_clean (&dir_copy);
              */
             tree_store_end_check ();
             g_hash_table_destroy (marked_files);
-            return next_free;
+            return;
         }
+        fentry = &list->list[list->len - 1];
 
-        list->list[next_free].f.marked = 0;
+        fentry->f.marked = 0;
 
         /*
          * If we have marked files in the copy, scan through the copy
          * to find matching file.  Decrease number of remaining marks if
          * we copied one.
          */
-        if (marked_cnt > 0)
+        if (marked_cnt > 0 && g_hash_table_lookup (marked_files, dp->d_name) != NULL)
         {
-            if ((g_hash_table_lookup (marked_files, dp->d_name)))
-            {
-                list->list[next_free].f.marked = 1;
-                marked_cnt--;
-            }
+            fentry->f.marked = 1;
+            marked_cnt--;
         }
 
-        list->list[next_free].fnamelen = NLENGTH (dp);
-        list->list[next_free].fname = g_strndup (dp->d_name, list->list[next_free].fnamelen);
-        list->list[next_free].f.link_to_dir = link_to_dir;
-        list->list[next_free].f.stale_link = stale_link;
-        list->list[next_free].f.dir_size_computed = 0;
-        list->list[next_free].st = st;
-        list->list[next_free].sort_key = NULL;
-        list->list[next_free].second_sort_key = NULL;
-        next_free++;
-        if ((next_free % 16) == 0)
+        if ((list->len & 15) == 0)
             rotate_dash (TRUE);
     }
     mc_closedir (dirp);
     tree_store_end_check ();
     g_hash_table_destroy (marked_files);
-    if (next_free)
-    {
-        do_sort (list, sort, next_free - 1, lc_reverse, lc_case_sensitive, exec_ff);
-    }
-    clean_dir (&dir_copy, count);
-    rotate_dash (FALSE);
 
-    return next_free;
+    dir_list_sort (list, sort, sort_op);
+
+    dir_list_clean (&dir_copy);
+    rotate_dash (FALSE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
