@@ -241,7 +241,7 @@ typedef struct
     replace_action_t replace_result;
 
     struct stat *s_stat, *d_stat;
-} FileOpContextUI;
+} file_op_context_ui_t;
 
 /*** file scope variables ************************************************************************/
 
@@ -284,8 +284,8 @@ statvfs_works (void)
     if (statvfs_works_cache < 0)
         statvfs_works_cache = (uname (&name) == 0 && 0 <= str_verscmp (name.release, "2.6.36"));
     return statvfs_works_cache;
-}
 #endif
+}
 #endif
 
 /* --------------------------------------------------------------------------------------------- */
@@ -396,7 +396,7 @@ file_bps_prepare_for_show (char *buffer, long bps)
  *   alex
  */
 static replace_action_t
-overwrite_query_dialog (FileOpContext * ctx, enum OperationMode mode)
+overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
 {
 #define ADD_RD_BUTTON(i, ypos) \
     add_widget_autopos (ui->replace_dlg, \
@@ -461,7 +461,7 @@ overwrite_query_dialog (FileOpContext * ctx, enum OperationMode mode)
     const size_t num = G_N_ELEMENTS (rd_widgets);
     int *widgets_len;
 
-    FileOpContextUI *ui = ctx->ui;
+    file_op_context_ui_t *ui = ctx->ui;
 
     char buffer[BUF_SMALL];
     char fsize_buffer[BUF_SMALL];
@@ -658,11 +658,11 @@ progress_button_callback (WButton * button, int action)
 /* --------------------------------------------------------------------------------------------- */
 
 FileProgressStatus
-check_progress_buttons (FileOpContext * ctx)
+check_progress_buttons (file_op_context_t * ctx)
 {
     int c;
     Gpm_Event event;
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
         return FILE_CONT;
@@ -710,10 +710,10 @@ check_progress_buttons (FileOpContext * ctx)
 /* {{{ File progress display routines */
 
 void
-file_op_context_create_ui (FileOpContext * ctx, gboolean with_eta,
+file_op_context_create_ui (file_op_context_t * ctx, gboolean with_eta,
                            filegui_dialog_type_t dialog_type)
 {
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
     int buttons_width;
     int dlg_width = 58, dlg_height = 17;
     int y = 2, x = 3;
@@ -733,7 +733,7 @@ file_op_context_create_ui (FileOpContext * ctx, gboolean with_eta,
 
     ctx->dialog_type = dialog_type;
     ctx->recursive_result = RECURSIVE_YES;
-    ctx->ui = g_new0 (FileOpContextUI, 1);
+    ctx->ui = g_new0 (file_op_context_ui_t, 1);
 
     ui = ctx->ui;
     ui->replace_result = REPLACE_YES;
@@ -855,11 +855,11 @@ file_op_context_create_ui (FileOpContext * ctx, gboolean with_eta,
 /* --------------------------------------------------------------------------------------------- */
 
 void
-file_op_context_destroy_ui (FileOpContext * ctx)
+file_op_context_destroy_ui (file_op_context_t * ctx)
 {
     if (ctx != NULL && ctx->ui != NULL)
     {
-        FileOpContextUI *ui = (FileOpContextUI *) ctx->ui;
+        file_op_context_ui_t *ui = (file_op_context_ui_t *) ctx->ui;
 
         dlg_run_done (ui->op_dlg);
         dlg_destroy (ui->op_dlg);
@@ -874,13 +874,11 @@ file_op_context_destroy_ui (FileOpContext * ctx)
  */
 
 void
-file_progress_show (FileOpContext * ctx, off_t done, off_t total,
+file_progress_show (file_op_context_t * ctx, off_t done, off_t total,
                     const char *stalled_msg, gboolean force_update)
 {
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
     char buffer[BUF_TINY];
-    char buffer2[BUF_TINY];
-    char buffer3[BUF_TINY];
 
     if (!verbose || ctx == NULL || ctx->ui == NULL)
         return;
@@ -901,11 +899,15 @@ file_progress_show (FileOpContext * ctx, off_t done, off_t total,
 
     if (ui->showing_eta && ctx->eta_secs > 0.5)
     {
+        char buffer2[BUF_TINY];
+
         file_eta_prepare_for_show (buffer2, ctx->eta_secs, FALSE);
         if (ctx->bps == 0)
             g_snprintf (buffer, BUF_TINY, "%s %s", buffer2, stalled_msg);
         else
         {
+            char buffer3[BUF_TINY];
+
             file_bps_prepare_for_show (buffer3, ctx->bps);
             g_snprintf (buffer, BUF_TINY, "%s (%s) %s", buffer2, buffer3, stalled_msg);
         }
@@ -921,15 +923,18 @@ file_progress_show (FileOpContext * ctx, off_t done, off_t total,
 /* --------------------------------------------------------------------------------------------- */
 
 void
-file_progress_show_count (FileOpContext * ctx, size_t done, size_t total)
+file_progress_show_count (file_op_context_t * ctx, size_t done, size_t total)
 {
     char buffer[BUF_TINY];
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
         return;
 
     ui = ctx->ui;
+    if (ui->total_files_processed_label == NULL)
+        return;
+
     if (ctx->progress_totals_computed)
         g_snprintf (buffer, BUF_TINY, _("Files processed: %zu/%zu"), done, total);
     else
@@ -940,22 +945,20 @@ file_progress_show_count (FileOpContext * ctx, size_t done, size_t total)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-file_progress_show_total (FileOpTotalContext * tctx, FileOpContext * ctx, uintmax_t copied_bytes,
-                          gboolean show_summary)
+file_progress_show_total (FileOpTotalContext * tctx, file_op_context_t * ctx,
+                          uintmax_t copied_bytes, gboolean show_summary)
 {
     char buffer[BUF_TINY];
     char buffer2[BUF_TINY];
     char buffer3[BUF_TINY];
-    char buffer4[BUF_TINY];
-    struct timeval tv_current;
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
         return;
 
     ui = ctx->ui;
 
-    if (ctx->progress_totals_computed)
+    if (ui->progress_total_gauge != NULL)
     {
         if (ctx->progress_bytes == 0)
             gauge_show (ui->progress_total_gauge, 0);
@@ -970,43 +973,53 @@ file_progress_show_total (FileOpTotalContext * tctx, FileOpContext * ctx, uintma
     if (!show_summary && tctx->bps == 0)
         return;
 
-    gettimeofday (&tv_current, NULL);
-    file_frmt_time (buffer2, tv_current.tv_sec - tctx->transfer_start.tv_sec);
-
-    if (ctx->progress_totals_computed)
+    if (ui->time_label != NULL)
     {
-        file_eta_prepare_for_show (buffer3, tctx->eta_secs, TRUE);
-        if (tctx->bps == 0)
-            g_snprintf (buffer, BUF_TINY, _("Time: %s %s"), buffer2, buffer3);
+        struct timeval tv_current;
+        char buffer4[BUF_TINY];
+
+        gettimeofday (&tv_current, NULL);
+        file_frmt_time (buffer2, tv_current.tv_sec - tctx->transfer_start.tv_sec);
+
+        if (ctx->progress_totals_computed)
+        {
+            file_eta_prepare_for_show (buffer3, tctx->eta_secs, TRUE);
+            if (tctx->bps == 0)
+                g_snprintf (buffer, BUF_TINY, _("Time: %s %s"), buffer2, buffer3);
+            else
+            {
+
+                file_bps_prepare_for_show (buffer4, (long) tctx->bps);
+                g_snprintf (buffer, BUF_TINY, _("Time: %s %s (%s)"), buffer2, buffer3, buffer4);
+            }
+        }
         else
         {
-            file_bps_prepare_for_show (buffer4, (long) tctx->bps);
-            g_snprintf (buffer, BUF_TINY, _("Time: %s %s (%s)"), buffer2, buffer3, buffer4);
+            if (tctx->bps == 0)
+                g_snprintf (buffer, BUF_TINY, _("Time: %s"), buffer2);
+            else
+            {
+                file_bps_prepare_for_show (buffer4, (long) tctx->bps);
+                g_snprintf (buffer, BUF_TINY, _("Time: %s (%s)"), buffer2, buffer4);
+            }
         }
+
+        label_set_text (ui->time_label, buffer);
     }
-    else
+
+    if (ui->total_bytes_label != NULL)
     {
-        if (tctx->bps == 0)
-            g_snprintf (buffer, BUF_TINY, _("Time: %s"), buffer2);
+        size_trunc_len (buffer2, 5, tctx->copied_bytes, 0, panels_options.kilobyte_si);
+        if (!ctx->progress_totals_computed)
+            g_snprintf (buffer, BUF_TINY, _(" Total: %s "), buffer2);
         else
         {
-            file_bps_prepare_for_show (buffer4, (long) tctx->bps);
-            g_snprintf (buffer, BUF_TINY, _("Time: %s (%s)"), buffer2, buffer4);
+            size_trunc_len (buffer3, 5, ctx->progress_bytes, 0, panels_options.kilobyte_si);
+            g_snprintf (buffer, BUF_TINY, _(" Total: %s/%s "), buffer2, buffer3);
         }
+
+        hline_set_text (ui->total_bytes_label, buffer);
     }
-
-    label_set_text (ui->time_label, buffer);
-
-    size_trunc_len (buffer2, 5, tctx->copied_bytes, 0, panels_options.kilobyte_si);
-    if (!ctx->progress_totals_computed)
-        g_snprintf (buffer, BUF_TINY, _(" Total: %s "), buffer2);
-    else
-    {
-        size_trunc_len (buffer3, 5, ctx->progress_bytes, 0, panels_options.kilobyte_si);
-        g_snprintf (buffer, BUF_TINY, _(" Total: %s/%s "), buffer2, buffer3);
-    }
-
-    hline_set_text (ui->total_bytes_label, buffer);
 }
 
 /* }}} */
@@ -1014,9 +1027,9 @@ file_progress_show_total (FileOpTotalContext * tctx, FileOpContext * ctx, uintma
 /* --------------------------------------------------------------------------------------------- */
 
 void
-file_progress_show_source (FileOpContext * ctx, const vfs_path_t * s_vpath)
+file_progress_show_source (file_op_context_t * ctx, const vfs_path_t * s_vpath)
 {
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
         return;
@@ -1042,9 +1055,9 @@ file_progress_show_source (FileOpContext * ctx, const vfs_path_t * s_vpath)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-file_progress_show_target (FileOpContext * ctx, const vfs_path_t * s_vpath)
+file_progress_show_target (file_op_context_t * ctx, const vfs_path_t * s_vpath)
 {
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
         return;
@@ -1067,9 +1080,9 @@ file_progress_show_target (FileOpContext * ctx, const vfs_path_t * s_vpath)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-file_progress_show_deleting (FileOpContext * ctx, const char *s, size_t * count)
+file_progress_show_deleting (file_op_context_t * ctx, const char *s, size_t * count)
 {
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
         return;
@@ -1084,11 +1097,11 @@ file_progress_show_deleting (FileOpContext * ctx, const char *s, size_t * count)
 /* --------------------------------------------------------------------------------------------- */
 
 FileProgressStatus
-file_progress_real_query_replace (FileOpContext * ctx,
+file_progress_real_query_replace (file_op_context_t * ctx,
                                   enum OperationMode mode, const char *destname,
                                   struct stat *_s_stat, struct stat *_d_stat)
 {
-    FileOpContextUI *ui;
+    file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
         return FILE_CONT;
@@ -1143,7 +1156,7 @@ file_progress_real_query_replace (FileOpContext * ctx,
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-file_mask_dialog (FileOpContext * ctx, FileOperation operation,
+file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
                   gboolean only_one,
                   const char *format, const void *text, const char *def_text, gboolean * do_bg)
 {
@@ -1280,7 +1293,7 @@ file_mask_dialog (FileOpContext * ctx, FileOperation operation,
             return dest_dir;
         }
 
-        ctx->search_handle = mc_search_new (source_mask, -1);
+        ctx->search_handle = mc_search_new (source_mask, -1, NULL);
 
         if (ctx->search_handle == NULL)
         {
@@ -1299,9 +1312,6 @@ file_mask_dialog (FileOpContext * ctx, FileOperation operation,
         else
             ctx->search_handle->search_type = MC_SEARCH_T_REGEX;
 
-        tmp = dest_dir;
-        dest_dir = tilde_expand (tmp);
-        g_free (tmp);
         vpath = vfs_path_from_str (dest_dir);
 
         ctx->dest_mask = strrchr (dest_dir, PATH_SEP);
