@@ -18,25 +18,25 @@
 
 /*** declarations of public functions ************************************************************/
 
-void file_op_context_create_ui (FileOpContext * ctx, gboolean with_eta,
+void file_op_context_create_ui (file_op_context_t * ctx, gboolean with_eta,
                                 filegui_dialog_type_t dialog_type);
-void file_op_context_destroy_ui (FileOpContext * ctx);
+void file_op_context_destroy_ui (file_op_context_t * ctx);
 
-char *file_mask_dialog (FileOpContext * ctx, FileOperation operation,
+char *file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
                         gboolean only_one,
                         const char *format, const void *text,
                         const char *def_text, gboolean * do_bg);
 
-FileProgressStatus check_progress_buttons (FileOpContext * ctx);
+FileProgressStatus check_progress_buttons (file_op_context_t * ctx);
 
-void file_progress_show (FileOpContext * ctx, off_t done, off_t total,
+void file_progress_show (file_op_context_t * ctx, off_t done, off_t total,
                          const char *stalled_msg, gboolean force_update);
-void file_progress_show_count (FileOpContext * ctx, size_t done, size_t total);
-void file_progress_show_total (FileOpTotalContext * tctx, FileOpContext * ctx,
+void file_progress_show_count (file_op_context_t * ctx, size_t done, size_t total);
+void file_progress_show_total (FileOpTotalContext * tctx, file_op_context_t * ctx,
                                uintmax_t copied_bytes, gboolean show_summary);
-void file_progress_show_source (FileOpContext * ctx, const vfs_path_t * s_vpath);
-void file_progress_show_target (FileOpContext * ctx, const vfs_path_t * path);
-void file_progress_show_deleting (FileOpContext * ctx, const char *path, size_t * count);
+void file_progress_show_source (file_op_context_t * ctx, const vfs_path_t * s_vpath);
+void file_progress_show_target (file_op_context_t * ctx, const vfs_path_t * path);
+void file_progress_show_deleting (file_op_context_t * ctx, const char *path, size_t * count);
 
 /*** inline functions ****************************************************************************/
 #endif /* MC__FILEGUI_H */
