@@ -8,6 +8,8 @@
 #include "lib/global.h"
 #include "lib/widget.h"
 
+#include "panel.h"
+
 /*** typedefs(not structures) and defined constants **********************************************/
 
 typedef enum
@@ -23,8 +25,6 @@ typedef enum
 
 /*** structures declarations (and typedefs of structures)*****************************************/
 
-struct WPanel;
-
 typedef struct
 {
     int horizontal_split;
@@ -70,10 +70,10 @@ const char *get_nth_panel_name (int num);
 
 struct Widget *get_panel_widget (int idx);
 
-struct WPanel *get_other_panel (void);
+WPanel *get_other_panel (void);
 
 void save_panel_dir (int idx);
-char *get_panel_dir_for (const struct WPanel *widget);
+char *get_panel_dir_for (const WPanel * widget);
 
 void set_hintbar (const char *str);
 
