@@ -72,7 +72,7 @@
 #include "hotlist.h"
 #include "panelize.h"
 #include "command.h"            /* cmdline */
-#include "dir.h"                /* clean_dir() */
+#include "dir.h"                /* dir_list_clean() */
 
 #include "chmod.h"
 #include "chown.h"
@@ -403,7 +403,7 @@ sort_cmd (void)
         return;
 
     p = MENU_PANEL;
-    sort_order = sort_box (&p->sort_info);
+    sort_order = sort_box (&p->sort_info, p->sort_field);
     panel_set_sort_order (p, sort_order);
 }
 
@@ -580,6 +580,22 @@ create_panels (void)
     char *current_dir, *other_dir;
     vfs_path_t *original_dir;
 
+    /*
+     * Following cases from command line are possible:
+     * 'mc' (no arguments):            mc_run_param0 == NULL, mc_run_param1 == NULL
+     *                                 active panel uses current directory
+     *                                 passive panel uses "other_dir" from panels.ini
+     *
+     * 'mc dir1 dir2' (two arguments): mc_run_param0 != NULL, mc_run_param1 != NULL
+     *                                 active panel uses mc_run_param0
+     *                                 passive panel uses mc_run_param1
+     *
+     * 'mc dir1' (single argument):    mc_run_param0 != NULL, mc_run_param1 == NULL
+     *                                 active panel uses mc_run_param0
+     *                                 passive panel uses "other_dir" from panels.ini
+     */
+
+    /* Set up panel directories */
     if (boot_current_is_left)
     {
         /* left panel is active */
@@ -587,11 +603,25 @@ create_panels (void)
         other_index = 1;
         current_mode = startup_left_mode;
         other_mode = startup_right_mode;
-        /* if mc_run_param0 is NULL, working directory will be used for the left panel */
-        current_dir = (char *) mc_run_param0;
-        /* mc_run_param1 is never NULL. It is setup from command line or from panels.ini
-         * (value of other_dir). mc_run_param1 will be used for the right panel */
-        other_dir = mc_run_param1;
+
+        if (mc_run_param0 == NULL && mc_run_param1 == NULL)
+        {
+            /* no arguments */
+            current_dir = NULL; /* assume current dir */
+            other_dir = saved_other_dir;        /* from ini */
+        }
+        else if (mc_run_param0 != NULL && mc_run_param1 != NULL)
+        {
+            /* two arguments */
+            current_dir = (char *) mc_run_param0;
+            other_dir = mc_run_param1;
+        }
+        else                    /* mc_run_param0 != NULL && mc_run_param1 == NULL */
+        {
+            /* one argument */
+            current_dir = (char *) mc_run_param0;
+            other_dir = saved_other_dir;        /* from ini */
+        }
     }
     else
     {
@@ -601,20 +631,24 @@ create_panels (void)
         current_mode = startup_right_mode;
         other_mode = startup_left_mode;
 
-        /* if mc_run_param0 is not NULL (it was setup from command line), it will be used
-         * for the left panel, working directory will be used for the right one;
-         * if mc_run_param0 is NULL, working directory will be used for the right (active) panel,
-         * mc_run_param1 will be used for the left one */
-        if (mc_run_param0 != NULL)
+        if (mc_run_param0 == NULL && mc_run_param1 == NULL)
         {
-            current_dir = NULL;
-            other_dir = (char *) mc_run_param0;
+            /* no arguments */
+            current_dir = NULL; /* assume current dir */
+            other_dir = saved_other_dir;        /* from ini */
         }
-        else
+        else if (mc_run_param0 != NULL && mc_run_param1 != NULL)
         {
-            current_dir = NULL;
+            /* two arguments */
+            current_dir = (char *) mc_run_param0;
             other_dir = mc_run_param1;
         }
+        else                    /* mc_run_param0 != NULL && mc_run_param1 == NULL */
+        {
+            /* one argument */
+            current_dir = (char *) mc_run_param0;
+            other_dir = saved_other_dir;        /* from ini */
+        }
     }
 
     /* 1. Get current dir */
@@ -677,7 +711,7 @@ create_panels (void)
     the_hint->auto_adjust_cols = 0;
     WIDGET (the_hint)->cols = COLS;
 
-    the_menubar = menubar_new (0, 0, COLS, NULL);
+    the_menubar = menubar_new (0, 0, COLS, NULL, menubar_visible);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -780,14 +814,14 @@ put_prog_name (void)
 static void
 put_tagged (WPanel * panel)
 {
-    int i;
-
     if (!command_prompt)
         return;
     input_disable_update (cmdline);
     if (panel->marked)
     {
-        for (i = 0; i < panel->count; i++)
+        int i;
+
+        for (i = 0; i < panel->dir.len; i++)
         {
             if (panel->dir.list[i].f.marked)
                 command_insert (cmdline, panel->dir.list[i].fname, TRUE);
@@ -1384,6 +1418,9 @@ midnight_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
         return MSG_HANDLED;
 
     case MSG_RESIZE:
+        /* dlg_set_size() is surplus for this case */
+        w->lines = LINES;
+        w->cols = COLS;
         setup_panels ();
         menubar_arrange (the_menubar);
         return MSG_HANDLED;
@@ -1746,7 +1783,7 @@ do_nc (void)
         /* don't handle VFS timestamps for dirs opened in panels */
         mc_event_destroy (MCEVENT_GROUP_CORE, "vfs_timestamp");
 
-        clean_dir (&panelized_panel.list, panelized_panel.count);
+        dir_list_clean (&panelized_panel.list);
     }
 
     /* Program end */
