@@ -193,6 +193,9 @@ gboolean is_autodetect_codeset_enabled = FALSE;
 char *spell_language = NULL;
 #endif
 
+/* Value of "other_dir" key in ini file */
+char *saved_other_dir = NULL;
+
 /* If set, then print to the given file the last directory we were at */
 char *last_wd_string = NULL;
 
@@ -507,7 +510,6 @@ setup__move_panels_config_into_separate_file (const char *profile)
 {
     mc_config_t *tmp_cfg;
     char **groups, **curr_grp;
-    const char *need_grp;
 
     if (!exist_file (profile))
         return;
@@ -544,6 +546,8 @@ setup__move_panels_config_into_separate_file (const char *profile)
 
     while (*curr_grp)
     {
+        const char *need_grp;
+
         need_grp = setup__is_cfg_group_must_panel_config (*curr_grp);
         if (need_grp != NULL)
         {
@@ -986,13 +990,13 @@ load_setup (void)
     if (startup_left_mode != view_listing && startup_right_mode != view_listing)
         startup_left_mode = view_listing;
 
-    if (mc_run_param1 == NULL)
     {
         vfs_path_t *vpath;
+
         buffer = mc_config_get_string (mc_panels_config, "Dirs", "other_dir", ".");
         vpath = vfs_path_from_str (buffer);
         if (vfs_file_is_local (vpath))
-            mc_run_param1 = buffer;
+            saved_other_dir = buffer;
         else
             g_free (buffer);
         vfs_path_free (vpath);
@@ -1382,9 +1386,9 @@ panel_load_setup (WPanel * panel, const char *section)
 
     /* Load sort order */
     buffer = mc_config_get_string (mc_panels_config, section, "sort_order", "name");
-    panel->sort_info.sort_field = panel_get_field_by_id (buffer);
-    if (panel->sort_info.sort_field == NULL)
-        panel->sort_info.sort_field = panel_get_field_by_id ("name");
+    panel->sort_field = panel_get_field_by_id (buffer);
+    if (panel->sort_field == NULL)
+        panel->sort_field = panel_get_field_by_id ("name");
 
     g_free (buffer);
 
@@ -1418,7 +1422,7 @@ panel_load_setup (WPanel * panel, const char *section)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-panel_save_setup (struct WPanel *panel, const char *section)
+panel_save_setup (WPanel * panel, const char *section)
 {
     char buffer[BUF_TINY];
     size_t i;
@@ -1428,7 +1432,7 @@ panel_save_setup (struct WPanel *panel, const char *section)
                        panel->sort_info.case_sensitive);
     mc_config_set_int (mc_panels_config, section, "exec_first", panel->sort_info.exec_first);
 
-    mc_config_set_string (mc_panels_config, section, "sort_order", panel->sort_info.sort_field->id);
+    mc_config_set_string (mc_panels_config, section, "sort_order", panel->sort_field->id);
 
     for (i = 0; list_types[i].key != NULL; i++)
         if (list_types[i].list_type == panel->list_type)
