@@ -54,7 +54,7 @@ check_for_default (const vfs_path_t * default_file_vpath, const vfs_path_t * fil
 {
     if (!exist_file (vfs_path_as_str (file_vpath)))
     {
-        FileOpContext *ctx;
+        file_op_context_t *ctx;
         FileOpTotalContext *tctx;
 
         if (!exist_file (vfs_path_as_str (default_file_vpath)))
