@@ -1,11 +1,12 @@
 /*
    Virtual File System: GNU Tar file system.
 
-   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2011
+   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2011, 2013
    The Free Software Foundation, Inc.
 
    Written by:
    Jan Hudec, 2000
+   Slava Zanko <slavazanko@gmail.com>, 2013
 
    This file is part of the Midnight Commander.
 
@@ -130,7 +131,7 @@ struct new_cpio_header
 typedef struct
 {
     unsigned long inumber;
-    unsigned short device;
+    dev_t device;
     struct vfs_s_inode *inode;
 } defer_inode;
 
@@ -146,13 +147,12 @@ typedef struct
 
 static struct vfs_class vfs_cpiofs_ops;
 
-/* FIXME: should be off_t instead of int. */
-static int cpio_position;
+static off_t cpio_position;
 
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-static int cpio_find_head (struct vfs_class *me, struct vfs_s_super *super);
+static ssize_t cpio_find_head (struct vfs_class *me, struct vfs_s_super *super);
 static ssize_t cpio_read_bin_head (struct vfs_class *me, struct vfs_s_super *super);
 static ssize_t cpio_read_oldc_head (struct vfs_class *me, struct vfs_s_super *super);
 static ssize_t cpio_read_crc_head (struct vfs_class *me, struct vfs_s_super *super);
@@ -171,7 +171,7 @@ cpio_defer_find (const void *a, const void *b)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static ssize_t
 cpio_skip_padding (struct vfs_s_super *super)
 {
     switch (((cpio_super_data_t *) super->data)->type)
@@ -205,8 +205,7 @@ cpio_free_archive (struct vfs_class *me, struct vfs_s_super *super)
     if (arch->fd != -1)
         mc_close (arch->fd);
     arch->fd = -1;
-    g_slist_foreach (arch->deferred, (GFunc) g_free, NULL);
-    g_slist_free (arch->deferred);
+    g_slist_free_full (arch->deferred, g_free);
     arch->deferred = NULL;
     g_free (super->data);
     super->data = NULL;
@@ -225,15 +224,11 @@ cpio_open_cpio_file (struct vfs_class *me, struct vfs_s_super *super, const vfs_
     fd = mc_open (vpath, O_RDONLY);
     if (fd == -1)
     {
-        char *name;
-
-        name = vfs_path_to_str (vpath);
-        message (D_ERROR, MSG_ERROR, _("Cannot open cpio archive\n%s"), name);
-        g_free (name);
+        message (D_ERROR, MSG_ERROR, _("Cannot open cpio archive\n%s"), vfs_path_as_str (vpath));
         return -1;
     }
 
-    super->name = vfs_path_to_str (vpath);
+    super->name = g_strdup (vfs_path_as_str (vpath));
     super->data = g_new (cpio_super_data_t, 1);
     arch = (cpio_super_data_t *) super->data;
     arch->fd = -1;              /* for now */
@@ -306,12 +301,12 @@ cpio_read_head (struct vfs_class *me, struct vfs_s_super *super)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static ssize_t
 cpio_find_head (struct vfs_class *me, struct vfs_s_super *super)
 {
     cpio_super_data_t *arch = (cpio_super_data_t *) super->data;
     char buf[BUF_SMALL * 2];
-    int ptr = 0;
+    ssize_t ptr = 0;
     ssize_t top;
     ssize_t tmp;
 
@@ -384,12 +379,15 @@ cpio_create_entry (struct vfs_class *me, struct vfs_s_super *super, struct stat
     case S_IFCHR:
     case S_IFBLK:
 #ifdef S_IFSOCK
+        /* cppcheck-suppress syntaxError */
     case S_IFSOCK:
 #endif
 #ifdef S_IFIFO
+        /* cppcheck-suppress syntaxError */
     case S_IFIFO:
 #endif
 #ifdef S_IFNAM
+        /* cppcheck-suppress syntaxError */
     case S_IFNAM:
 #endif
         if ((st->st_size != 0) && (st->st_rdev == 0x0001))
@@ -731,8 +729,6 @@ static int
 cpio_open_archive (struct vfs_s_super *super, const vfs_path_t * vpath,
                    const vfs_path_element_t * vpath_element)
 {
-    int status = STATUS_START;
-
     (void) vpath_element;
 
     if (cpio_open_cpio_file (vpath_element->class, super, vpath) == -1)
@@ -740,17 +736,18 @@ cpio_open_archive (struct vfs_s_super *super, const vfs_path_t * vpath,
 
     while (TRUE)
     {
+        ssize_t status;
+
         status = cpio_read_head (vpath_element->class, super);
+        if (status < 0)
+            return (-1);
 
         switch (status)
         {
         case STATUS_EOF:
             {
-                char *archive_name;
-
-                archive_name = vfs_path_to_str (vpath);
-                message (D_ERROR, MSG_ERROR, _("Unexpected end of file\n%s"), archive_name);
-                g_free (archive_name);
+                message (D_ERROR, MSG_ERROR, _("Unexpected end of file\n%s"),
+                         vfs_path_as_str (vpath));
                 return 0;
             }
         case STATUS_OK:
@@ -784,16 +781,11 @@ cpio_super_same (const vfs_path_element_t * vpath_element, struct vfs_s_super *p
                  const vfs_path_t * vpath, void *cookie)
 {
     struct stat *archive_stat = cookie; /* stat of main archive */
-    char *archive_name = vfs_path_to_str (vpath);
 
     (void) vpath_element;
 
-    if (strcmp (parc->name, archive_name))
-    {
-        g_free (archive_name);
+    if (strcmp (parc->name, vfs_path_as_str (vpath)))
         return 0;
-    }
-    g_free (archive_name);
 
     /* Has the cached archive been changed on the disk? */
     if (((cpio_super_data_t *) parc->data)->st.st_mtime < archive_stat->st_mtime)
