@@ -129,7 +129,7 @@ sfs_vfmake (const vfs_path_t * vpath, vfs_path_t * cache_vpath)
     char pad[10240];
     char *s_iter, *t = pad;
     int was_percent = 0;
-    vfs_path_t *pname, *s;      /* name of parent archive */
+    vfs_path_t *pname;          /* name of parent archive */
     char *pqname;               /* name of parent archive, quoted */
     const vfs_path_element_t *path_element;
 
@@ -150,6 +150,8 @@ sfs_vfmake (const vfs_path_t * vpath, vfs_path_t * cache_vpath)
     /*    if ((sfs_flags[w] & F_2) || (!inpath) || (!*inpath)); else return -1; */
     if ((sfs_flags[w] & F_NOLOCALCOPY) == 0)
     {
+        vfs_path_t *s;
+
         s = mc_getlocalcopy (pname);
         if (s == NULL)
         {
