@@ -3,7 +3,7 @@
    Function for whow info on display
 
    Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2011
+   2004, 2005, 2006, 2007, 2009, 2011, 2013
    The Free Software Foundation, Inc.
 
    Written by:
@@ -15,7 +15,7 @@
    Pavel Machek, 1998
    Roland Illig <roland.illig@gmx.de>, 2004, 2005
    Slava Zanko <slavazanko@google.com>, 2009
-   Andrew Borodin <aborodin@vmail.ru>, 2009
+   Andrew Borodin <aborodin@vmail.ru>, 2009, 2013
    Ilia Maslakov <il.smind@gmail.com>, 2009, 2010
 
    This file is part of the Midnight Commander.
@@ -49,11 +49,9 @@
 #endif
 
 #include "src/setup.h"          /* panels_options */
-#include "src/main.h"
 #include "src/keybind-defaults.h"
 
 #include "internal.h"
-#include "mcviewer.h"
 
 /*** global variables ****************************************************************************/
 
@@ -81,48 +79,48 @@ static enum ruler_type
 static void
 mcview_set_buttonbar (mcview_t * view)
 {
-    Dlg_head *h = view->widget.owner;
+    WDialog *h = WIDGET (view)->owner;
     WButtonBar *b = find_buttonbar (h);
     const global_keymap_t *keymap = view->hex_mode ? viewer_hex_map : viewer_map;
 
-    buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), keymap, (Widget *) view);
+    buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), keymap, WIDGET (view));
 
     if (view->hex_mode)
     {
         if (view->hexedit_mode)
-            buttonbar_set_label (b, 2, Q_ ("ButtonBar|View"), keymap, (Widget *) view);
+            buttonbar_set_label (b, 2, Q_ ("ButtonBar|View"), keymap, WIDGET (view));
         else if (view->datasource == DS_FILE)
-            buttonbar_set_label (b, 2, Q_ ("ButtonBar|Edit"), keymap, (Widget *) view);
+            buttonbar_set_label (b, 2, Q_ ("ButtonBar|Edit"), keymap, WIDGET (view));
         else
-            buttonbar_set_label (b, 2, "", keymap, (Widget *) view);
+            buttonbar_set_label (b, 2, "", keymap, WIDGET (view));
 
-        buttonbar_set_label (b, 4, Q_ ("ButtonBar|Ascii"), keymap, (Widget *) view);
-        buttonbar_set_label (b, 6, Q_ ("ButtonBar|Save"), keymap, (Widget *) view);
-        buttonbar_set_label (b, 7, Q_ ("ButtonBar|HxSrch"), keymap, (Widget *) view);
+        buttonbar_set_label (b, 4, Q_ ("ButtonBar|Ascii"), keymap, WIDGET (view));
+        buttonbar_set_label (b, 6, Q_ ("ButtonBar|Save"), keymap, WIDGET (view));
+        buttonbar_set_label (b, 7, Q_ ("ButtonBar|HxSrch"), keymap, WIDGET (view));
 
     }
     else
     {
         buttonbar_set_label (b, 2, view->text_wrap_mode ? Q_ ("ButtonBar|UnWrap")
-                             : Q_ ("ButtonBar|Wrap"), keymap, (Widget *) view);
-        buttonbar_set_label (b, 4, Q_ ("ButtonBar|Hex"), keymap, (Widget *) view);
-        buttonbar_set_label (b, 6, "", keymap, (Widget *) view);
-        buttonbar_set_label (b, 7, Q_ ("ButtonBar|Search"), keymap, (Widget *) view);
+                             : Q_ ("ButtonBar|Wrap"), keymap, WIDGET (view));
+        buttonbar_set_label (b, 4, Q_ ("ButtonBar|Hex"), keymap, WIDGET (view));
+        buttonbar_set_label (b, 6, "", keymap, WIDGET (view));
+        buttonbar_set_label (b, 7, Q_ ("ButtonBar|Search"), keymap, WIDGET (view));
     }
 
-    buttonbar_set_label (b, 5, Q_ ("ButtonBar|Goto"), keymap, (Widget *) view);
+    buttonbar_set_label (b, 5, Q_ ("ButtonBar|Goto"), keymap, WIDGET (view));
     buttonbar_set_label (b, 8, view->magic_mode ? Q_ ("ButtonBar|Raw")
-                         : Q_ ("ButtonBar|Parse"), keymap, (Widget *) view);
+                         : Q_ ("ButtonBar|Parse"), keymap, WIDGET (view));
 
     if (mcview_is_in_panel (view))
-        buttonbar_set_label (b, 10, "", keymap, (Widget *) view);
+        buttonbar_set_label (b, 10, "", keymap, WIDGET (view));
     else
     {
         /* don't override some panel buttonbar keys  */
-        buttonbar_set_label (b, 3, Q_ ("ButtonBar|Quit"), keymap, (Widget *) view);
+        buttonbar_set_label (b, 3, Q_ ("ButtonBar|Quit"), keymap, WIDGET (view));
         buttonbar_set_label (b, 9, view->text_nroff_mode ? Q_ ("ButtonBar|Unform")
-                             : Q_ ("ButtonBar|Format"), keymap, (Widget *) view);
-        buttonbar_set_label (b, 10, Q_ ("ButtonBar|Quit"), keymap, (Widget *) view);
+                             : Q_ ("ButtonBar|Format"), keymap, WIDGET (view));
+        buttonbar_set_label (b, 10, Q_ ("ButtonBar|Quit"), keymap, WIDGET (view));
     }
 }
 
@@ -141,7 +139,7 @@ mcview_display_status (mcview_t * view)
         return;
 
     tty_setcolor (STATUSBAR_COLOR);
-    tty_draw_hline (view->widget.y + top, view->widget.x + left, ' ', width);
+    tty_draw_hline (WIDGET (view)->y + top, WIDGET (view)->x + left, ' ', width);
 
     file_label =
         view->filename_vpath != NULL ?
@@ -150,13 +148,13 @@ mcview_display_status (mcview_t * view)
 
     if (width > 40)
     {
-        char buffer[BUF_TRUNC_LEN + 1];
-
         widget_move (view, top, width - 32);
         if (view->hex_mode)
             tty_printf ("0x%08" PRIxMAX, (uintmax_t) view->hex_cursor);
         else
         {
+            char buffer[BUF_TRUNC_LEN + 1];
+
             size_trunc_len (buffer, BUF_TRUNC_LEN, mcview_get_filesize (view), 0,
                             panels_options.kilobyte_si);
             tty_printf ("%9" PRIuMAX "/%s%s %s", (uintmax_t) view->dpy_end,
@@ -190,7 +188,7 @@ mcview_update (mcview_t * view)
     {
         view->dpy_bbar_dirty = FALSE;
         mcview_set_buttonbar (view);
-        buttonbar_redraw (find_buttonbar (view->widget.owner));
+        widget_redraw (WIDGET (find_buttonbar (WIDGET (view)->owner)));
     }
 
     if (view->dirty > dirt_limit)
@@ -260,8 +258,8 @@ mcview_compute_areas (mcview_t * view)
 
     view_area.top = view->dpy_frame_size;
     view_area.left = view->dpy_frame_size;
-    view_area.height = mcview_dimen_doz (view->widget.lines, 2 * view->dpy_frame_size);
-    view_area.width = mcview_dimen_doz (view->widget.cols, 2 * view->dpy_frame_size);
+    view_area.height = mcview_dimen_doz (WIDGET (view)->lines, 2 * view->dpy_frame_size);
+    view_area.width = mcview_dimen_doz (WIDGET (view)->cols, 2 * view->dpy_frame_size);
 
     /* Most coordinates of the areas equal those of the whole viewer */
     view->status_area = view_area;
@@ -345,10 +343,12 @@ mcview_display_toggle_ruler (mcview_t * view)
 void
 mcview_display_clean (mcview_t * view)
 {
+    Widget *w = WIDGET (view);
+
     tty_setcolor (NORMAL_COLOR);
-    widget_erase ((Widget *) view);
+    widget_erase (w);
     if (view->dpy_frame_size != 0)
-        tty_draw_box (view->widget.y, view->widget.x, view->widget.lines, view->widget.cols, FALSE);
+        tty_draw_box (w->y, w->x, w->lines, w->cols, FALSE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -420,6 +420,8 @@ mcview_percent (mcview_t * view, off_t p)
 
     widget_move (view, top, right - 4);
     tty_printf ("%3d%%", percent);
+    /* avoid cursor wrapping in NCurses-base MC */
+    widget_move (view, top, right - 1);
 }
 
 /* --------------------------------------------------------------------------------------------- */
