@@ -3,7 +3,7 @@
    Function for hex view
 
    Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2011
+   2004, 2005, 2006, 2007, 2009, 2011, 2013
    The Free Software Foundation, Inc.
 
    Written by:
@@ -14,7 +14,7 @@
    Norbert Warmuth, 1997
    Pavel Machek, 1998
    Roland Illig <roland.illig@gmx.de>, 2004, 2005
-   Slava Zanko <slavazanko@google.com>, 2009
+   Slava Zanko <slavazanko@google.com>, 2009, 2013
    Andrew Borodin <aborodin@vmail.ru>, 2009
    Ilia Maslakov <il.smind@gmail.com>, 2009
 
@@ -110,6 +110,22 @@ utf8_to_int (char *str, int *char_width, gboolean * result)
 }
 #endif /* HAVE_CHARSET */
 
+/* --------------------------------------------------------------------------------------------- */
+/** Determine the state of the current byte.
+ *
+ * @param view viewer object
+ * @param from offset
+ * @param curr current node
+ */
+
+static mark_t
+mcview_hex_calculate_boldflag (mcview_t * view, off_t from, struct hexedit_change_node *curr)
+{
+    return (from == view->hex_cursor) ? MARK_CURSOR
+        : (curr != NULL && from == curr->offset) ? MARK_CHANGED
+        : (view->search_start <= from && from < view->search_end) ? MARK_SELECTED : MARK_NORMAL;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -129,7 +145,7 @@ mcview_display_hex (mcview_t * view)
      * text column.
      */
 
-    screen_dimen row, col;
+    screen_dimen row;
     off_t from;
     int c;
     mark_t boldflag = MARK_NORMAL;
@@ -152,7 +168,9 @@ mcview_display_hex (mcview_t * view)
 
     for (row = 0; mcview_get_byte (view, from, NULL) == TRUE && row < height; row++)
     {
+        screen_dimen col = 0;
         size_t i;
+
         col = 0;
 
         /* Print the hex offset */
@@ -173,17 +191,22 @@ mcview_display_hex (mcview_t * view)
 #ifdef HAVE_CHARSET
             if (view->utf8)
             {
-                char corr_buf[6 + 1];
-                int cnt, cw = 1;
+                int cw = 1;
                 gboolean read_res = TRUE;
+
                 ch = mcview_get_utf (view, from, &cw, &read_res);
                 if (!read_res)
                     break;
                 /* char width is greater 0 bytes */
                 if (cw != 0)
                 {
+                    int cnt;
+                    char corr_buf[UTF8_CHAR_LEN + 1];
                     struct hexedit_change_node *corr = curr;
-                    int res = g_unichar_to_utf8 (ch, (char *) corr_buf);
+                    int res;
+
+                    res = g_unichar_to_utf8 (ch, (char *) corr_buf);
+
                     for (cnt = 0; cnt < cw; cnt++)
                     {
                         if (curr != NULL && from + cnt == curr->offset)
@@ -211,11 +234,7 @@ mcview_display_hex (mcview_t * view)
             }
 
             /* Determine the state of the current byte */
-            boldflag =
-                (from == view->hex_cursor) ? MARK_CURSOR
-                : (curr != NULL && from == curr->offset) ? MARK_CHANGED
-                : (view->search_start <= from &&
-                   from < view->search_end) ? MARK_SELECTED : MARK_NORMAL;
+            boldflag = mcview_hex_calculate_boldflag (view, from, curr);
 
             /* Determine the value of the current byte */
             if (curr != NULL && from == curr->offset)
