@@ -78,7 +78,7 @@ mcview_toggle_magic_mode (mcview_t * view)
 {
     char *command;
     dir_list *dir;
-    int *dir_count, *dir_idx;
+    int *dir_idx;
 
     mcview_altered_magic_flag = 1;
     view->magic_mode = !view->magic_mode;
@@ -86,16 +86,13 @@ mcview_toggle_magic_mode (mcview_t * view)
     /* reinit view */
     command = g_strdup (view->command);
     dir = view->dir;
-    dir_count = view->dir_count;
     dir_idx = view->dir_idx;
     view->dir = NULL;
-    view->dir_count = NULL;
     view->dir_idx = NULL;
     mcview_done (view);
     mcview_init (view);
     mcview_load (view, command, vfs_path_as_str (view->filename_vpath), 0);
     view->dir = dir;
-    view->dir_count = dir_count;
     view->dir_idx = dir_idx;
     g_free (command);
 
@@ -253,9 +250,8 @@ mcview_done (mcview_t * view)
     if (mc_global.mc_run_mode == MC_RUN_VIEWER && view->dir != NULL)
     {
         /* mcviewer is the owner of file list */
-        clean_dir (view->dir, *view->dir_count);
+        dir_list_clean (view->dir);
         g_free (view->dir->list);
-        g_free (view->dir_count);
         g_free (view->dir_idx);
         g_free (view->dir);
     }
