@@ -1,11 +1,12 @@
 /*
    src/filemanager - examine_cd() function testing
 
-   Copyright (C) 2012
+   Copyright (C) 2012, 2013
    The Free Software Foundation, Inc.
 
    Written by:
    Andrew Borodin <aborodin@vmail.ru>, 2012
+   Slava Zanko <slavazanko@gmail.com>, 2013
 
    This file is part of the Midnight Commander.
 
@@ -21,33 +22,30 @@
 
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ */
 
 #define TEST_SUITE_NAME "/src/filemanager"
 
-#include <config.h>
-
-#include <check.h>
+#include "tests/mctest.h"
 
 #include <stdio.h>
 
-#include "lib/global.h"
 #include "lib/vfs/path.h"
 #include "src/filemanager/layout.h"
 #include "src/filemanager/midnight.h"
 #include "src/filemanager/tree.h"
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
 #include "src/subshell.h"
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
 
 #include "src/filemanager/command.c"
 
 /* --------------------------------------------------------------------------------------------- */
 
 gboolean command_prompt = FALSE;
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
 enum subshell_state_enum subshell_state = INACTIVE;
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
 int quit = 0;
 WPanel *current_panel = NULL;
 
@@ -82,7 +80,7 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
     return NULL;
 }
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
 void
 init_subshell (void)
 {
@@ -93,7 +91,7 @@ do_load_prompt (void)
 {
     return TRUE;
 }
-#endif /* HAVE_SUBSHELL_SUPPORT */
+#endif /* ENABLE_SUBSHELL */
 
 void
 shell_execute (const char *command, int flags)
@@ -129,7 +127,9 @@ teardown (void)
     g_free (result); \
 }
 
+/* *INDENT-OFF* */
 START_TEST (test_examine_cd)
+/* *INDENT-ON* */
 {
     char *result;
 
@@ -156,7 +156,9 @@ START_TEST (test_examine_cd)
     check_examine_cd ("/test/path/\\${AAA}test2", "/test/path/${AAA}test2");
     check_examine_cd ("/test/path/test1\\${AAA}test2", "/test/path/test1${AAA}test2");
 }
+/* *INDENT-OFF* */
 END_TEST
+/* *INDENT-ON* */
 
 /* --------------------------------------------------------------------------------------------- */
 
